//////////////////////////////////////////////////////////////////
// mysql JDBC Driver
//
// Version : 0.9.2
//
// JDBC : 1.22
//
// LastModified: 16.03.1997
//
// Copyright (c) 1997 GWE Technologies Ltd. All rights reserved
// See http://gwe.co.uk/mysql/jdbc/copyright.html for full details

package gwe.sql;

import java.net.*;
import java.io.*;
import java.util.Hashtable;
import gwe.sql.gweMysqlColumn;
import gwe.sql.gweMysqlException;


public class gweMysql {

  // Constants defined from mysql
  public static final int SLEEP=0;
  public static final int QUIT=1;
  public static final int INIT_DB=2;
  public static final int QUERY=3;
  public static final int FIELD_LIST=4;
  public static final int CREATE_DB=5;
  public static final int DROP_DB=6;
  public static final int RELOAD=7;
  public static final int SHUTDOWN=8;
  public static final int STATISTICS=9;
  public static final int PROCESS_INFO=10;
  public static final int CONNECT=11;

  Socket sock;
  gweMysqlPacket packet;
  String host;
  int port;
  String database;
  String user;
  String password;
  byte protocolVersion;
  String serverVersion;
  int serverMajorVersion;
  int serverMinorVersion;
  int serverSubminorVersion;
  long threadId;
  String cryptSeed;
  long columnCount;
  gweMysqlColumn[] columnResult;
  Hashtable columnName;
  Hashtable columnFullName;
  int[] dataStart;
  String encoding;
  gweNativeUnicodeMapper mapperclass;

  public gweMysql(String enc, gweNativeUnicodeMapper mapper) {
    encoding = enc;
    mapperclass = mapper;
  }

  // Connect to mysql host
  public synchronized void Connect(String h, int p, String d, String u, String pw) throws gweMysqlException {
    host=h;
    port=p;
    database=d;
    user=u;
    password=pw;
    if (user.equals("")) {user="nobody";}

    try {
      // Open TCP/IP socket
      sock = new Socket(host, port);
      DataInputStream sin = new DataInputStream(sock.getInputStream());
      DataOutputStream sout = new DataOutputStream(sock.getOutputStream());

      // Create new packet handler
      packet = new gweMysqlPacket(sin,sout,encoding,mapperclass);

//      packet.TRACING=true;

      // Get the first packet from mysql
      packet.next();
      protocolVersion = packet.readByte();
      serverVersion = packet.readString(); // major.minor.subminor-??

      parseServerVersion(serverVersion);

      long threadId = packet.read4Bytes();
      cryptSeed = packet.readString();

      // Send who we are
      packet.clear_w();
      packet.write2Bytes(0);
      packet.write3Bytes(packet.MAXBUFFER);
      packet.writeString(user);
      packet.writeString(cryptPassword(password,cryptSeed));
      packet.send();

      // Check if all is ok
      packet.next();
      byte statusCode = packet.readByte();
      if (statusCode==0xff) {
        String errorString = packet.readString();
        throw new gweMysqlException(errorString);
      } else if (statusCode==0x00) {
        long affectedRows = packet.readLength();
        long uniqueId = packet.readLength();
      } else throw new gweMysqlException("Unknown status code '"+statusCode+"'");

      // Ok, we're authenticated, so select our database
      packet.clear_w();
      packet.resetPacketSequence();
      packet.writeByte((byte)gweMysql.INIT_DB);
      packet.writeStringNoNull(database);
      packet.send();

      // Check if all is well
      packet.next();
      statusCode = packet.readByte();
      if (statusCode==0xff) {
        String errorString = packet.readString();
        throw new gweMysqlException(errorString);
      } else if (statusCode==0x00) {
        long affectedRows = packet.readLength();
        long uniqueId = packet.readLength();
      } else throw new gweMysqlException("Unknown status code '"+statusCode+"'");


    } catch (Exception e) {throw new gweMysqlException("gweMysql.Connect: "+e);}
  }

  // major.minor.subminor-??
  void parseServerVersion(String verstr) throws gweMysqlException {
    serverMajorVersion    = -1;
    serverMinorVersion    = -1;
    serverSubminorVersion = -1;
    String temp_verstr;
    int pidx = verstr.indexOf('.');
    if( pidx >= 1 )
    {
      try {
        serverMajorVersion = Integer.parseInt(verstr.substring(0, pidx));
      }
      catch(NumberFormatException ex) {
        throw new gweMysqlException("Illegal Server Version : "+verstr);
      }
    }
    else
    {
      throw new gweMysqlException("Illegal Server Version : "+verstr);
    }

    temp_verstr = verstr.substring(pidx+1, verstr.length());
    pidx = temp_verstr.indexOf('.');
    if( pidx >= 1 )
    {
      try {
        serverMinorVersion = Integer.parseInt(temp_verstr.substring(0, pidx));
      }
      catch(NumberFormatException ex) {
        throw new gweMysqlException("Illegal Server Version : "+verstr);
      }
    }
    else
    {
      throw new gweMysqlException("Illegal Server Version : "+verstr);
    }

    temp_verstr = temp_verstr.substring(pidx+1, temp_verstr.length());
    int lastidx;
    for(lastidx = 0; lastidx < temp_verstr.length(); lastidx++)
    {
      char c = temp_verstr.charAt(lastidx);
      if( c < '0' || c > '9' )
        break;
    }
    try {
      serverSubminorVersion = Integer.parseInt(temp_verstr.substring(0, lastidx));
    }
    catch(NumberFormatException ex) {
      throw new gweMysqlException("Illegal Server Version : "+verstr);
    }
  }

  // return    x > 0 or 0 or x < 0
  int judgeServerVersion(int maj, int min, int sub) {
    if( serverMajorVersion > maj )
      return 1;
    else if( serverMajorVersion < maj )
      return -1;

    if( serverMinorVersion > min )
      return 1;
    else if( serverMinorVersion < min )
      return -1;

    if( serverSubminorVersion > sub )
      return 1;
    else if( serverSubminorVersion < sub )
      return -1;

    return 0;
  }

  String getServerVersion() {
    return serverVersion;
  }
  int getServerMajorVersion() {
    return serverMajorVersion;
  }
  int getServerMinorVersion() {
    return serverMinorVersion;
  }
  int getServerSubminorVersion() {
    return serverSubminorVersion;
  }

  // Return hash for given string
  long hashPassword(String p) {
    long  nr=1345345333;
    long  nr2=7;
    long tmp;
    for (int i=0; i<p.length(); i++) {
    if ((p.charAt(i)==' ') || (p.charAt(i)=='\t') ) continue;
      tmp=(long)p.charAt(i);
      nr^= (((nr & 63)+nr2)*tmp)+ (nr << 8);
      nr2+=tmp;
    }
    return nr & (((long) 1L << 31) -1L);
  }

  // Return encrypted response, given clear password and seed message
  String cryptPassword(String p, String m) {
    if( p == null || p.length() == 0 )
      return p;

    long hp, hm, s1, s2;
    long max_value= 0x01FFFFFF;
    double d;
    byte b;

    hp=this.hashPassword(p);
    hm=this.hashPassword(m);
    long nr=hp^hm;
    nr%=max_value;
    s1=nr;
    s2=nr/2;

    char[] cr = new char[m.length()];
    for (int i=0; i<m.length(); i++) {
      s1=(s1*3+s2)%max_value;
      s2=(s1+s2+33)%max_value;
      d = (double)s1/max_value;
      b = (byte)java.lang.Math.floor((d*31)+64);
      cr[i]=(char)b;
    }
    return new String(cr);
  }

  public gweMysql sqlQuery(String q) throws gweMysqlException {

    // Send query command and sql query string
    packet.clear_w();
    packet.resetPacketSequence();
    packet.writeByte((byte)gweMysql.QUERY);
    packet.writeStringNoNull(q);
    packet.send();

    // Check if we're ok
    packet.next();
    columnCount = packet.readLength();
    if (columnCount==255 || columnCount==-1) { // for NULL FIELD!!
      String errorString = packet.readString();
      throw new gweMysqlException(errorString);
    } else if (columnCount==0) {
      //throw new gweMysqlException("No columns returned.");
      return null;
    } else {
      columnResult = new gweMysqlColumn[(int)columnCount];
      columnName=new Hashtable((int)columnCount);
      columnFullName=new Hashtable((int)columnCount);
      dataStart = new int[(int)columnCount];

      // Read in the column information
      for (int i=0;i<columnCount; i++) {
        packet.next();
        String tableName=packet.readLenString();
        String colName=packet.readLenString();
        int colLength = packet.readnBytes();
        int colType=packet.readnBytes();
        packet.readByte(); // We know it's currently 2
        short colFlag = (short) packet.ub(packet.readByte());
        int colDecimals = packet.ub(packet.readByte());

        columnResult[i] = new gweMysqlColumn(tableName, colName, colLength, colType, colFlag, colDecimals);
        columnName.put(colName, new Integer(i));
        String fullName = tableName.concat(".").concat(colName);
        columnFullName.put(fullName, new Integer(i));

      }
      packet.next(); //Redundant - we know how many, this is end of section marker packet
    }
    return this;
  }


  public int sqlUpdate(String q) throws gweMysqlException {

    // Send query command and sql update
    packet.clear_w();
    packet.resetPacketSequence();
    packet.writeByte((byte)gweMysql.QUERY);
    packet.writeStringNoNull(q);
    packet.send();

    // check if we're ok
    packet.next();
    int updateStatus = packet.ub(packet.readByte());
    if (updateStatus==255) {
      String errorString = packet.readString();
      throw new gweMysqlException(errorString);
    }
    long affectedRows = packet.readLength();
    long updateId = packet.readLength();

    //return update id
//    return (int)updateId;
    return (int)affectedRows;
  }


  public void sqlQuit() throws gweMysqlException {

    // Send quit command to mysql
    packet.clear_w();
    packet.resetPacketSequence();
    packet.writeByte((byte)gweMysql.QUIT);
    packet.send();

  }



  public boolean nextResult() throws gweMysqlException {
    int i, p;

    // Get the next incoming packet
    packet.next();
    // It it's not the last one, note start positions of data
    if(!packet.isLastDataPacket_r()) {
      for (i=0; i<columnCount; i++) {
        p=packet.currentPos_r();
        dataStart[i]=p;
        switch(packet.ub(packet.getCurDataPos_r()))
        {
          case 252: p += 2; break;
          case 253: p += 3; break;
          case 254: p += 4; break;
          default: break;
        }
        long next_len = packet.readLength();
        if( next_len == -1 )                 // for NULL FIELD!!
          next_len = 0;                      // for NULL FIELD!!
        packet.setPos_r(1+p+(int)next_len);
      }
      return true;
    }
    return false;
  }

  public int getColumnCount() {
    return (int) columnCount;
  }

  public gweMysqlColumn getColumn(int c) throws gweMysqlException {
    c--;  // Columnd go from 1..columnCount to 0..columnCount-1
    if ((c<columnCount)&&(c>=0)) {
      return columnResult[c];
    } else {
      throw new gweMysqlException("Invalid column number : "+c);
    }
  }

  public boolean isNull(int c) throws gweMysqlException {
    //System.out.println("isNull:" + c);
    c--;
    if ((c<columnCount)&&(c>=0)) {
      packet.setPos_r(dataStart[c]);
      long l = packet.readLength();
      return (l==-1); // for NULL FIELD!!
    } else throw new gweMysqlException("Invalid column number : "+c);
  }

  public int getColumnNum(String cn) throws gweMysqlException {
    Integer nI = (Integer) columnName.get(cn);
    if (nI==null) nI = (Integer) columnFullName.get(cn);
    if (nI!=null) return nI.intValue()+1;
    throw new gweMysqlException("Invalid column name : "+cn);
  }


  public String getString(int c) throws gweMysqlException {
    c--;
    if ((c<columnCount)&&(c>=0)) {
      packet.setPos_r(dataStart[c]);
      return packet.readLenString();
    } else throw new gweMysqlException("Invalid column number : "+c);
  }

  public String getString(String cn) throws gweMysqlException {
    Integer nI = (Integer) columnName.get(cn);
    if (nI==null) nI = (Integer) columnFullName.get(cn);
    if (nI!=null) {
      int c = nI.intValue()+1;
      if (c<columnCount) {
        packet.setPos_r(dataStart[c]);
        return packet.readLenString();
      }
    }
    throw new gweMysqlException("Invalid column string : "+cn);
  }

  public byte[] getBytes(int c) throws gweMysqlException {
    c--;
    if ((c<columnCount)&&(c>=0)) {
      packet.setPos_r(dataStart[c]);
      long l = packet.readLength();
      if( l == -1 )                   // for NULL FIELD!!
        return null;                  // for NULL FIELD!!
      //byte[] r = new byte[(int)l];
      //for (int i=0; i<(int)l; i++) {
      //  r[i]=packet.readByte();
      //}
      byte[] r = packet.readByteBlock((int)l);
      return r;
    }
    throw new gweMysqlException("Invalid column number : "+c);
  }


  public void reConnect() throws gweMysqlException {
    try {
      sqlQuit();
      sock.close();
      this.Connect(host, port, database, user, password);
    } catch (Exception e) {throw new gweMysqlException("gweMysql.reConnect: "+e);}
  }


  public void Close() throws gweMysqlException {
    try {
      sqlQuit();
      sock.close();
    } catch (Exception e) {throw new gweMysqlException("gweMysql.Close: "+e);}
  }

  gweMysqlPacket getPacket(){
    return (packet);
  }

  String getEncoding() {
    return encoding;
  }

  gweNativeUnicodeMapper getNativeUnicodeMapper() {
    return mapperclass;
  }
}

