//////////////////////////////////////////////////////////////////
// mysql JDBC Driver
//
// Version : 0.9.2
//
// JDBC : 1.22
//
// LastModified: 12.03.1997
//
// Copyright (c) 1997 GWE Technologies Ltd. All rights reserved
// See http://gwe.co.uk/mysql/jdbc/copyright.html for full details

package gwe.sql;

import java.sql.*;
import gwe.sql.gweMysql;
import gwe.sql.gweMysqlException;
//import java.util.Map;
import java.util.Hashtable;
import java.io.UnsupportedEncodingException;
import java.util.Properties;
import java.util.Enumeration;

public class gweMysqlConnection implements java.sql.Connection {

  private gweMysql mysql = null;
  /**
   * The JDBC driver for this connection
   */
  private Driver driver;
  /**
   * The URL for this connection
   */
  private String url;
  /**
   * The username for this connection
   */
  private String username;
  /**
   * Catalog string
   */
  private String catalog;
  /**
   * Transaction isolation level
   */
  private int isolation = java.sql.Connection.TRANSACTION_SERIALIZABLE;
  /**
   * type map object associated with this connection
   */
  //private java.util.Map typeMap = null;
  /**
   * characterEncoding
   */
  private String encoding = "Shift_JIS";
  /**
   * nativeUnicodeMapper
   */
  private gweNativeUnicodeMapper mapper = null;
  /**
   * Add 0x5C(\) after 0x5C(\) when you execute PreparedStatement.setString
   */
  private boolean add5CAfter5C = false;
  /**
   * Trace SQL Statement with LogStream
   */
  private boolean debugSQL = false;
  /**
   * autocommit
   */
  private boolean autocommit = true;
  /**
   * Properties of Connection
   */
  private Properties prop;

  /**
   * Connects to the mysql server
   * @exception SQLException raised in the event of connection failure
   * @param host the IP address of the host on which the mysql server resides
   * @param root true if this is a root connection, false if otherwise
   * @param database the database to which a connection should be made
   * @param u the URL used to connect to this database
   * @param d the driver that instantiated this connection
   */
  public gweMysqlConnection(String host, int port, String user, String password, String database, String u, Properties info, Driver d) throws SQLException {

    if( user == null || user == "" ) user = "nobody";

    prop = new Properties();
    Enumeration info_keys = info.propertyNames();
    while( info_keys.hasMoreElements() )
    {
      String info_key = (String)info_keys.nextElement();
      String info_val = (String)info.get(info_key);
      prop.put(info_key, info_val);
    }

    ////////////////////////////////////////////////////////////////
    // Parse ?characterEncoding=xxxxx&etc...
    int query_idx = database.indexOf('?');
    if( query_idx != -1 )
    {
      String query_string = database.substring(query_idx+1);
      database = database.substring(0, query_idx);

      for(;;)
      {
        String q_name_val = null;
        int q_name_val_idx = query_string.indexOf('&');
        if( q_name_val_idx != -1 )
          q_name_val = query_string.substring(0, q_name_val_idx);
        else
          q_name_val = query_string;

        String q_name = null;
        String q_val  = null;
        int q_val_idx = q_name_val.indexOf('=');
        if( q_val_idx != -1 )
        {
          q_name = q_name_val.substring(0, q_val_idx);
          q_val  = q_name_val.substring(q_val_idx+1);
        }
        else
        {
          q_name = q_name_val;
          q_val  = "";
        }

        if( q_name.length() > 0 )
          prop.put(q_name, q_val);

        if( q_name_val_idx == -1 )
          break;
        query_string = query_string.substring(q_name_val_idx+1);
      }
    }
    ////////////////////////////////////////////////////////////////

    String temp_encoding = prop.getProperty("characterEncoding");
    if( temp_encoding != null )
      encoding = temp_encoding;

    // nativeUnicodeMapper support!!
    String temp_mapperclass = prop.getProperty("nativeUnicodeMapper");
    if( temp_mapperclass != null )
    {
      try {
        mapper = (gweNativeUnicodeMapper)Class.forName(temp_mapperclass).newInstance();
      }
      catch(Exception ex) {
        throw new SQLException("mysql exception: " + ex.toString());
      }
    }
    if( temp_encoding != null && temp_mapperclass != null )
      throw new SQLException("mysql exception: " + "Please specify either characterEncoding or nativeUnicodeMapper");

    String add5c_mode = prop.getProperty("add5CAfter5C");
    if( add5c_mode != null )
      add5CAfter5C = Boolean.valueOf(add5c_mode).booleanValue();

    String debug_mode = prop.getProperty("debugSQL");
    if( debug_mode != null )
      debugSQL = Boolean.valueOf(debug_mode).booleanValue();

    try {
      mysql = new gweMysql(encoding, mapper);
      mysql.Connect(host, port, database, user, password);
    }
    catch( gweMysqlException e ) {
      mysql = null;
      throw new SQLException("mysql exception: " + e.getMessage());
    }
    url = u;
    catalog = database;
    username = user;
    driver = d;
  }

  /**
   * JDBC draft specification method for returning a SQL statement
   * object.
   * @see java.sql.Connection#createStatement
   * @exception SQLException thrown in the event the creation fails
   * @return a new statement object
   */
  public Statement createStatement() throws SQLException {
    return createStatement(gweMysqlResultSetCore.TYPE_FORWARD_ONLY, gweMysqlResultSetCore.CONCUR_READ_ONLY);
  }

  /**
   * JDBC draft specification method for returning an SQL pre-compiled
   * statement.  Note that mysql does not support such things, so this
   * thing just throws an exception.
   * @see java.sql.Connection#prepareStatement
   * @exception SQLException gets thrown any time this is called
   */
  public PreparedStatement prepareStatement(String sql) throws SQLException {
//    throw new SQLException("mysql does not support prepared statements.");
    return prepareStatement(sql, gweMysqlResultSetCore.TYPE_FORWARD_ONLY, gweMysqlResultSetCore.CONCUR_READ_ONLY);
  }

  /**
   * JDBC draft specification method for creating a stored procedure call.
   * Note that mysql does not support stored procedures, so this method
   * throws an exception.
   * @see java.sql.Connection#prepareCall
   * @exception SQLException gets thrown any time this is called
   */
  public CallableStatement prepareCall(String sql) throws SQLException {
    throw new SQLException("mysql does not support stored procedures.");
  }

  /**
   * We do not currently try to map SQL-92 to mysql :-)
   * @see java.sql.Connection#nativeSQL
   * @param sql the query which just gets thrown right back out
   * @return the SQL string
   */
  public String nativeSQL(String sql) throws SQLException {
    return sql;
  }

  /**
   * JDBC 1.0 specification method for returning the auto-close
   * status of the connection.
   * @return true
   */
  public boolean getAutoClose() throws SQLException {
    return true;
  }

  /**
   * JDBC 1.0 specification method for setting auto-close.
   * @param b meaningless
   */
  public void setAutoClose(boolean b) throws SQLException {
  }

  /**
   * JDBC 1.0 specification method for determining the
   * auto-commit state of the connection.
   * @see java.sql.Connection#getAutoCommit
   */
  public boolean getAutoCommit() throws SQLException {
    return autocommit;
  }

  /**
   * JDBC 1.0  specification method for setting auto-commit.  Since
   * mysql has no transaction management, this connection object acts
   * exactly as if auto-commit were set.  So this method does nothing.
   * @see java.sql.Connection#setAutoCommit
   * @param b throw an exception if this is false
   */
  public void setAutoCommit(boolean b) throws SQLException {
    int judge = mysql.judgeServerVersion(3, 23, 15);
    if( !b && judge < 0 )
      throw new SQLException("mysql older than Ver3.23.15 does not support transaction.");

    if( judge >= 0 )
    {
      String sqlstr = "SET AUTOCOMMIT=" + (b ? "1" : "0") + ";";
      executeUpdate(sqlstr);
      autocommit = b;
    }
  }

  /**
   * JDBC draft specification method for committing a transaction.
   * @see java.sql.Connection#commit
   */
  public void commit() throws SQLException {
    int judge = mysql.judgeServerVersion(3, 23, 15);
    if( judge < 0 )
      throw new SQLException("mysql older than Ver3.23.15 does not support commit.");
    String sqlstr = "COMMIT;";
    executeUpdate(sqlstr);
  }

  /**
   * JDBC draft specification method for rolling back a transaction.  Since
   * mysql has no support for rollbacks, this method throws an exception.
   * @see java.sql.Connection#rollback
   * @exception SQLException gets thrown if this ever gets called
   */
  public void rollback() throws SQLException {
    int judge = mysql.judgeServerVersion(3, 23, 15);
    if( judge < 0 )
      throw new SQLException("mysql older than Ver3.23.15 does not support rollback.");
    String sqlstr = "ROLLBACK;";
    executeUpdate(sqlstr);
  }

  /**
   * JDBC draft specification method for closing the database connection.
   * @see java.sql.Connection#close
   * @exception SQLException thrown in the event of a gweMysqlException
   */
  public void close() throws SQLException {
    try {
      mysql.Close();
      mysql = null;
    }
    catch( gweMysqlException e ) {
      throw new SQLException("mysql exception: " + e.getMessage());
    }
  }

  /**
   * JDBC draft specification method for letting others know the connection
   * status.
   * @see java.sql.Connection#isClosed
   * @exception SQLException who knows why Sun thought this needed an exception
   * @return true if the connection is closed, false otherwise
   */
  public boolean isClosed() throws SQLException {
    return (mysql == null);
  }

  /**
   * JDBC draft specification method to return information about
   * the database. This is yet to be written.
   * @see java.sql.Connection#getMetData
   * @exception SQLException thrown if an error occurs reading meta-data
   * @return a DatabaseMetaData object with database info
   */
  public DatabaseMetaData getMetaData() throws SQLException {
    return new gweMysqlDatabaseMetaData(this);
  }

  /**
   * JDBC draft specification method to put the connection in read-only mode.
   * @see java.sql.Connection#setReadOnly
   * @param b boolean read only
   */
  public void setReadOnly(boolean b) throws SQLException {
  }

  /**
   * JDBC draft specification method to return the read-only status of the
   * connection.  Since mysql has no such status, this always returns false.
   * @see java.sql.Connection#isReadOnly
   * @return always false
   */
  public boolean isReadOnly() throws SQLException {
    return false;
  }

  /**
   * JDBC draft specification method to select a sub-space of the target
   * database.  It basically has no menaing to mysql.
   * @see java.sql.Connection#setCatalog
   * @param str the catalog
   */
  public void setCatalog(String str) throws SQLException {
    String sqlstr = "USE " + str + ";";
    executeUpdate(sqlstr);
    catalog = str;
  }

  /**
   * JDBC draft specification method to return the catalog.  This has no
   * meaning to mysql.
   * @see java.sql.Connection#getCatalog
   * @return the catalog name
   */
  public String getCatalog() throws SQLException {
    return catalog;
  }

  /**
   * JDBC draft specification method for setting the transaction isolation
   * level for the connection.  This has no meaning to mysql.
   * @see java.sql.Connection#setTransactionIsolation
   * @param x the isolation level
   */
  public void setTransactionIsolation(int x) throws SQLException {
    int judge = mysql.judgeServerVersion(3, 23, 36);
    if( judge < 0 )
      throw new SQLException("mysql older than Ver3.23.36 does not support Transaction Isolation Level.");

    String sqlstr = "SET SESSION TRANSACTION ISOLATION LEVEL";
    switch(x)
    {
      case java.sql.Connection.TRANSACTION_NONE:
        sqlstr += ";";
        break;
      case java.sql.Connection.TRANSACTION_READ_COMMITTED:
        sqlstr += " READ COMMITTED;";
        executeUpdate(sqlstr);
        break;
      case java.sql.Connection.TRANSACTION_READ_UNCOMMITTED:
        sqlstr += " READ UNCOMMITTED;";
        executeUpdate(sqlstr);
        break;
      case java.sql.Connection.TRANSACTION_REPEATABLE_READ:
        sqlstr += " REPEATABLE READ;";
        executeUpdate(sqlstr);
        break;
      case java.sql.Connection.TRANSACTION_SERIALIZABLE:
        sqlstr += " SERIALIZABLE;";
        executeUpdate(sqlstr);
        break;
      default:
        throw new SQLException("Unknown Transaction Isolation Level.");
    }
    isolation = x;
  }

  /**
   * JDBC draft specification method for retrieving the transaction
   * isolation level for the connection.  This information has no meaning
   * to mysql.
   * @see java.sql.Connection#getTransactionIsolation
   * @return the transaction isolation level
   */
  public int getTransactionIsolation() throws SQLException {
    return isolation;
  }

  /**
   * @return the characterEncoding
   */
  public String getEncoding() {
    return encoding;
  }

  /**
   * @return the nativeUnicodeMapper
   */
  public gweNativeUnicodeMapper getNativeUnicodeMapper() {
    return mapper;
  }

  /**
   * @return the add5CAfter5C flag
   */
  public boolean isAdd5CAfter5C() {
    return add5CAfter5C;
  }

  /**
   * @return the debugSQL flag
   */
  public boolean isDebugSQL() {
    return debugSQL;
  }

  /**
   * JDBC draft specification method for disabling of closing of statements
   * and result sets.  This has no meaning to mysql.
   * @see java.sql.Connection#disableAutoClose
   */
  public void disableAutoClose() throws SQLException {
  }

  /**
   * JDBC draft specification method for retrieving a chain of warnings
   * related to the connection.
   * @see java.sql.Connection#getWarnings
   * @return the chain of warnings for this connection
   */
  public SQLWarning getWarnings() throws SQLException {
    return null;
  }

  /**
   * JDBC draft specification method for clearing the warning chain.
   * @see java.sql.Connection#clearWarnings
   */
  public void clearWarnings() throws SQLException {
  }

  /**
   * Executes an SQL Query statement.
   * @param sql the statement to be executed
   * @return results of the statement
   */
  public gweMysql executeQuery(String sql) throws SQLException {
    // write log
    if( debugSQL && DriverManager.getLogStream() != null )
    {
      Timestamp t = new Timestamp(System.currentTimeMillis());
      DriverManager.println(t.toString() + ":" + sql);
    }

    try {
      return mysql.sqlQuery(sql);
    }
    catch( gweMysqlException e ) {
      throw new SQLException("mysql exception: " + e.getMessage());
    }
  }

  /**
   * Executes an SQL Update statement.
   * @param sql the statement to be executed
   * @return the row id if an insert on an auto-incremented table,
   *         0 otherwise.
   */
  public int executeUpdate(String sql) throws SQLException {
    // write log
    if( debugSQL && DriverManager.getLogStream() != null )
    {
      Timestamp t = new Timestamp(System.currentTimeMillis());
      DriverManager.println(t.toString() + ":" + sql);
    }

    try {
      return mysql.sqlUpdate(sql);
    }
    catch( gweMysqlException e ) {
      throw new SQLException("mysql exception: " + e.getMessage());
    }
  }

  gweMysqlPacket getPacket(){
    return mysql.getPacket();
  }

  ////////////////////////////////////////////////////////////////////
  // to avoid creating new resultset before close old resultset!!
  private gweMysqlResultSetCore curResultSet = null;
  void setCurResultSet(gweMysqlResultSetCore result) {
    //if( curResultSet != null )
    //{
    //  try {
    //    curResultSet.close();
    //  }
    //  catch (Exception ex) {
    //    ;
    //  }
    //}
    curResultSet = result;
  }
  ////////////////////////////////////////////////////////////////////

  //--------------------------JDBC 2.0-----------------------------------

  public Statement createStatement(int resultSetType, int resultSetConcurrency) throws SQLException {
    if( resultSetType == gweMysqlResultSetCore.TYPE_FORWARD_ONLY &&
        resultSetConcurrency == gweMysqlResultSetCore.CONCUR_READ_ONLY )
    {
      return new gweMysqlStatement(this, resultSetType, resultSetConcurrency);
    }
    // Add New Type ResultSet!!
    if( resultSetType == gweMysqlResultSetCore.TYPE_SCROLL_INSENSITIVE &&
        resultSetConcurrency == gweMysqlResultSetCore.CONCUR_READ_ONLY )
    {
      return new gweMysqlStatement(this, resultSetType, resultSetConcurrency);
    }
    throw new SQLException("mysql does not support createStatement(int, int).");
  }

  public PreparedStatement prepareStatement(String sql, int resultSetType, int resultSetConcurrency) throws SQLException {
    if( resultSetType == gweMysqlResultSetCore.TYPE_FORWARD_ONLY &&
        resultSetConcurrency == gweMysqlResultSetCore.CONCUR_READ_ONLY )
    {
      return new gweMysqlPreparedStatement(this, sql, resultSetType, resultSetConcurrency, encoding, mapper, add5CAfter5C);
    }
    // Add New Type ResultSet!!
    if( resultSetType == gweMysqlResultSetCore.TYPE_SCROLL_INSENSITIVE &&
        resultSetConcurrency == gweMysqlResultSetCore.CONCUR_READ_ONLY )
    {
      return new gweMysqlPreparedStatement(this, sql, resultSetType, resultSetConcurrency, encoding, mapper, add5CAfter5C);
    }
    throw new SQLException("mysql does not support prepareStatement(String, int, int).");
  }

  public CallableStatement prepareCall(String sql, int resultSetType, int resultSetConcurrency) throws SQLException {
    throw new SQLException("mysql does not support prepareCall(String, int, int).");
  }

/*
  public Map getTypeMap() throws SQLException {
    return typeMap;
  }

  public void setTypeMap(Map map) throws SQLException {
    typeMap = map;
  }
*/

  // return    x > 0 or 0 or x < 0
  int judgeServerVersion(int maj, int min, int sub) {
    return mysql.judgeServerVersion(maj, min, sub);
  }

  String getUrl() {
    return url;
  }

  String getUsername() {
    return username;
  }

  String getServerVersion() {
    return mysql.getServerVersion();
  }

  int getDriverMajorVersion() {
    return driver.getMajorVersion();
  }

  int getDriverMinorVersion() {
    return driver.getMinorVersion();
  }
}
