//////////////////////////////////////////////////////////////////
// mysql JDBC Driver
//
// Version : 0.9.2
//
// JDBC : 1.22
//
// LastModified: 16.03.1997
//en
// Copyright (c) 1997 GWE Technologies Ltd. All rights reserved
// See http://gwe.co.uk/mysql/jdbc/copyright.html for full details

package gwe.sql;

import java.sql.*;
import java.util.Vector;
import java.util.StringTokenizer;

public class gweMysqlDatabaseMetaData implements DatabaseMetaData {

  private gweMysqlConnection conn;
  private static final String ProductName = "MySQL";
  private static final String DriverName = "gweMysql JDBC Driver";
  private static final String DriverVersion = "0.9.2";
  private boolean add_quote = false;

  public gweMysqlDatabaseMetaData(gweMysqlConnection org_conn) {
    this.conn = org_conn;

    //if( conn.judgeServerVersion(3, 23, 6) >= 0 )
    //  add_quote = true;
    add_quote = false;
  }

  String trimQuote(String quoted_str) {
    int idx1 = quoted_str.indexOf("`");
    int idx2 = quoted_str.lastIndexOf("`");
    if( idx1 != -1 && idx2 != -1 && idx1 != idx2 )
      return quoted_str.substring(idx1+1, idx2);
    return quoted_str;
  }

  /**
   * Can all the procedures returned by getProcedures be called by the
   * current user?
   *
   * @return true if so
   */
  public boolean allProceduresAreCallable() throws SQLException {
    return false;
  }

  /**
   * Can all the tables returned by getTable be SELECTed by the
   * current user?
   *
   * @return true if so
   */
  public boolean allTablesAreSelectable() throws SQLException {
    return false;
  }

  /**
   * What's the url for this database?
   *
   * @return the url or null if it can't be generated
   */
  public String getURL() throws SQLException {
    return conn.getUrl();
  }

  /**
   * What's our user name as known to the database?
   *
   * @return our database user name
   */
  public String getUserName() throws SQLException {
    return conn.getUsername();
  }

  /**
   * Is the database in read-only mode?
   *
   * @return true if so
   */
  public boolean isReadOnly() throws SQLException {
    return false;
  }

  /**
   * Are NULL values sorted high?
   *
   * @return true if so
   */
  public boolean nullsAreSortedHigh() throws SQLException {
    return false;
  }

  /**
   * Are NULL values sorted low?
   *
   * @return true if so
   */
  public boolean nullsAreSortedLow() throws SQLException {
    return true;
  }

  /**
   * Are NULL values sorted at the start regardless of sort order?
   *
   * @return true if so
   */
  public boolean nullsAreSortedAtStart() throws SQLException {
    return false;
  }

  /**
   * Are NULL values sorted at the end regardless of sort order?
   *
   * @return true if so
   */
  public boolean nullsAreSortedAtEnd() throws SQLException {
    return false;
  }

  /**
   * What's the name of this database product?
   *
   * @return database product name
   */
  public String getDatabaseProductName() throws SQLException {
    return ProductName;
  }

  /**
   * What's the version of this database product?
   *
   * @return database version
   */
  public String getDatabaseProductVersion() throws SQLException {
    return conn.getServerVersion();
  }

  /**
   * What's the name of this JDBC driver?
   *
   * @return JDBC driver name
   */
  public String getDriverName() throws SQLException {
    return DriverName;
  }

  /**
   * What's the version of this JDBC driver?
   *
   * @return JDBC driver version
   */
  public String getDriverVersion() throws SQLException {
    return DriverVersion;
  }

  /**
   * What's this JDBC driver's major version number?
   *
   * @return JDBC driver major version
   */
  public int getDriverMajorVersion() {
    return conn.getDriverMajorVersion();
  }

  /**
   * What's this JDBC driver's minor version number?
   *
   * @return JDBC driver minor version number
   */
  public int getDriverMinorVersion() {
    return conn.getDriverMinorVersion();
  }

  /**
   * Does the database store tables in a local file?
   *
   * @return true if so
   */
  public boolean usesLocalFiles() throws SQLException {
    return false;
  }

  /**
   * Does the database use a file for each table?
   *
   * @return true if the database uses a local file for each table
   */
  public boolean usesLocalFilePerTable() throws SQLException {
    return false;
  }

  /**
   * Does the database support mixed case unquoted SQL identifiers?
   *
   * @return true if so
   */
  public boolean supportsMixedCaseIdentifiers() throws SQLException {
    return false;
  }

  /**
   * Does the database store mixed case unquoted SQL identifiers in
   * upper case?
   *
   * @return true if so
   */
  public boolean storesUpperCaseIdentifiers() throws SQLException {
    return false;
  }

  /**
   * Does the database store mixed case unquoted SQL identifiers in
   * lower case?
   *
   * @return true if so
   */
  public boolean storesLowerCaseIdentifiers() throws SQLException {
    return false;
  }

  /**
   * Does the database store mixed case unquoted SQL identifiers in
   * mixed case?
   *
   * @return true if so
   */
  public boolean storesMixedCaseIdentifiers() throws SQLException {
    return false;
  }

  /**
   * Does the database support mixed case quoted SQL identifiers?
   *
   * A JDBC compliant driver will always return true.
   *
   * @return true if so
   */
  public boolean supportsMixedCaseQuotedIdentifiers() throws SQLException {
    return false;
  }

  /**
   * Does the database store mixed case quoted SQL identifiers in
   * upper case?
   *
   * A JDBC compliant driver will always return true.
   *
   * @return true if so
   */
  public boolean storesUpperCaseQuotedIdentifiers() throws SQLException {
    return false;
  }

  /**
   * Does the database store mixed case quoted SQL identifiers in
   * lower case?
   *
   * A JDBC compliant driver will always return false.
   *
   * @return true if so
   */
  public boolean storesLowerCaseQuotedIdentifiers() throws SQLException {
    return false;
  }

  /**
   * Does the database store mixed case quoted SQL identifiers in
   * mixed case?
   *
   * A JDBC compliant driver will always return false.
   *
   * @return true if so
   */
  public boolean storesMixedCaseQuotedIdentifiers() throws SQLException {
    return false;
  }

  /**
   * What's the string used to quote SQL identifiers?
   * This returns a space " " if identifier quoting isn't supported.
   *
   * A JDBC compliant driver always uses a double quote character.
   *
   * @return the quoting string
   */
  public String getIdentifierQuoteString() throws SQLException {
    //return " ";
    return "";
  }

  /**
   * Get a comma separated list of all a database's SQL keywords
   * that are NOT also SQL92 keywords.
   *
   * @return the list
   */
  public String getSQLKeywords() throws SQLException {
    String SQLKeywords = "AUTO_INCREMENT,BINARY,BLOB,ENUM,INFILE,LOAD,MEDIUMINT,OPTION,OUTFILE,REPLACE,SET,TEXT,UNSIGNED,ZEROFILL";
    return SQLKeywords;
  }

  /**
   * Get a comma separated list of math functions.
   *
   * @return the list
   */
  public String getNumericFunctions() throws SQLException {
    String NumericFunctions = "ABS,ACOS,ASIN,ATAN,ATAN2,BIT_COUNT,CEILING,COS,COT,DEGREES,EXP,FLOOR,LOG,LOG10,MAX,MIN,MOD,PI,POW,POWER,RADIANS,RAND,ROUND,SIN,SQRT,TAN,TRUNCATE";
    return NumericFunctions;
  }

  /**
   * Get a comma separated list of string functions.
   *
   * @return the list
   */
  public String getStringFunctions() throws SQLException {
    String StringFunctions = "ACII,CHAR,CHAR_LENGTH,CHARACTER_LENGTH,CONCAT,ELT,FIELD,FIND_IN_SET,INSERT,INSTR,INTERVAL,LCASE,LEFT,LENGTH,LOCATE,LOWER,LTRIM,MID,POSITION,OCTET_LENGTH,REPEAT,REPLACE,REVERSE,RIGHT,RTRIM,SPACE,SOUNDEX,SUBSTRING,SUBSTRING_INDEX,TRIM,UCASE,UPPER";
    return StringFunctions;
  }

  /**
   * Get a comma separated list of system functions.
   *
   * @return the list
   */
  public String getSystemFunctions() throws SQLException {
    String SystemFunctions = "DATABASE,USER,SYSTEM_USER,SESSION_USER,PASSWORD,ENCRYPT,LAST_INSERT_ID,VERSION";
    return SystemFunctions;
  }

  /**
   * Get a comma separated list of time and date functions.
   *
   * @return the list
   */
  public String getTimeDateFunctions() throws SQLException {
    String TimeDateFunctions = "DAYOFWEEK,WEEKDAY,DAYOFMONTH,DAYOFYEAR,MONTH,DAYNAME,MONTHNAME,QUARTER,WEEK,YEAR,HOUR,MINUTE,SECOND,PERIOD_ADD,PERIOD_DIFF,TO_DAYS,FROM_DAYS,DATE_FORMAT,TIME_FORMAT,CURDATE,CURRENT_DATE,CURTIME,CURRENT_TIME,NOW,SYSDATE,CURRENT_TIMESTAMP,UNIX_TIMESTAMP,FROM_UNIXTIME,SEC_TO_TIME,TIME_TO_SEC";
    return TimeDateFunctions;
  }

  /**
   * This is the string that can be used to escape '_' or '%' in
   * the string pattern style catalog search parameters.
   *
   * The '_' character represents any single character.
   * The '%' character represents any sequence of zero or
   * more characters.
   * @return the string used to escape wildcard characters
   */
  public String getSearchStringEscape() throws SQLException {
    String SearchStringEscape = "\\";
    return SearchStringEscape;
  }

  /**
   * Get all the "extra" characters that can be used in unquoted
   * identifier names (those beyond a-z, 0-9 and _).
   *
   * @return the string containing the extra characters
   */
  public String getExtraNameCharacters() throws SQLException {
    String ExtraNameCharacters = "";
    return ExtraNameCharacters;
  }

  /**
   * Is "ALTER TABLE" with add column supported?
   *
   * @return true if so
   */
  public boolean supportsAlterTableWithAddColumn() throws SQLException {
    return true;
  }

  /**
   * Is "ALTER TABLE" with drop column supported?
   *
   * @return true if so
   */
  public boolean supportsAlterTableWithDropColumn() throws SQLException {
    return true;
  }

  /**
   * Is column aliasing supported?
   *
   * If so, the SQL AS clause can be used to provide names for
   * computed columns or to provide alias names for columns as
   * required.
   *
   * A JDBC compliant driver always returns true.
   *
   * @return true if so
   */
  public boolean supportsColumnAliasing() throws SQLException {
    return true;
  }

  /**
   * Are concatenations between NULL and non-NULL values NULL?
   *
   * A JDBC compliant driver always returns true.
   *
   * @return true if so
   */
  public boolean nullPlusNonNullIsNull() throws SQLException {
    return true;
  }

  /**
   * Is the CONVERT function between SQL types supported?
   *
   * @return true if so
   */
  public boolean supportsConvert() throws SQLException {
    return false;
  }

  /**
   * Is CONVERT between the given SQL types supported?
   *
   * @param fromType the type to convert from
   * @param toType the type to convert to
   * @return true if so
   * @see Types
   */
  public boolean supportsConvert(int fromType, int toType) throws SQLException {
    boolean retval = false;
    switch(fromType)
    {
      case java.sql.Types.CHAR:
      case java.sql.Types.VARCHAR:
      case java.sql.Types.LONGVARCHAR:
      case java.sql.Types.BINARY:
      case java.sql.Types.VARBINARY:
      case java.sql.Types.LONGVARBINARY:
        switch(toType)
        {
          case java.sql.Types.DECIMAL:
          case java.sql.Types.NUMERIC:
          case java.sql.Types.REAL:
          case java.sql.Types.TINYINT:
          case java.sql.Types.SMALLINT:
          case java.sql.Types.INTEGER:
          case java.sql.Types.BIGINT:
          case java.sql.Types.FLOAT:
          case java.sql.Types.DOUBLE:
          case java.sql.Types.CHAR:
          case java.sql.Types.VARCHAR:
          case java.sql.Types.LONGVARCHAR:
          case java.sql.Types.BINARY:
          case java.sql.Types.VARBINARY:
          case java.sql.Types.LONGVARBINARY:
          case java.sql.Types.OTHER:
          case java.sql.Types.DATE:
          case java.sql.Types.TIME:
          case java.sql.Types.TIMESTAMP:
            retval = true;
          default:
            break;
        }
        break;
      case java.sql.Types.BIT:
        break;
      case java.sql.Types.DECIMAL:
      case java.sql.Types.NUMERIC:
      case java.sql.Types.REAL:
      case java.sql.Types.TINYINT:
      case java.sql.Types.SMALLINT:
      case java.sql.Types.INTEGER:
      case java.sql.Types.BIGINT:
      case java.sql.Types.FLOAT:
      case java.sql.Types.DOUBLE:
        switch(toType)
        {
          case java.sql.Types.DECIMAL:
          case java.sql.Types.NUMERIC:
          case java.sql.Types.REAL:
          case java.sql.Types.TINYINT:
          case java.sql.Types.SMALLINT:
          case java.sql.Types.INTEGER:
          case java.sql.Types.BIGINT:
          case java.sql.Types.FLOAT:
          case java.sql.Types.DOUBLE:
          case java.sql.Types.CHAR:
          case java.sql.Types.VARCHAR:
          case java.sql.Types.LONGVARCHAR:
          case java.sql.Types.BINARY:
          case java.sql.Types.VARBINARY:
          case java.sql.Types.LONGVARBINARY:
            retval = true;
          default:
            break;
        }
        break;
      case java.sql.Types.DATE:
      case java.sql.Types.TIME:
        switch(toType)
        {
          case java.sql.Types.CHAR:
          case java.sql.Types.VARCHAR:
          case java.sql.Types.LONGVARCHAR:
          case java.sql.Types.BINARY:
          case java.sql.Types.VARBINARY:
          case java.sql.Types.LONGVARBINARY:
            retval = true;
          default:
            break;
        }
        break;
      case java.sql.Types.TIMESTAMP:
        switch(toType)
        {
          case java.sql.Types.CHAR:
          case java.sql.Types.VARCHAR:
          case java.sql.Types.LONGVARCHAR:
          case java.sql.Types.BINARY:
          case java.sql.Types.VARBINARY:
          case java.sql.Types.LONGVARBINARY:
          case java.sql.Types.TIME:
          case java.sql.Types.DATE:
            retval = true;
          default:
            break;
        }
        break;
      case java.sql.Types.NULL:
        break;
      case java.sql.Types.OTHER:
        switch(toType)
        {
          case java.sql.Types.CHAR:
          case java.sql.Types.VARCHAR:
          case java.sql.Types.LONGVARCHAR:
          case java.sql.Types.BINARY:
          case java.sql.Types.VARBINARY:
          case java.sql.Types.LONGVARBINARY:
            retval = true;
          default:
            break;
        }
        break;
      default:
        break;
    }
    return retval;
  }

  /**
   * Are table correlation names supported?
   *
   * A JDBC compliant driver always returns true.
   *
   * @return true if so
   */
  public boolean supportsTableCorrelationNames() throws SQLException {
    return true;
  }

  /**
   * If table correlation names are supported, are they restricted
   * to be different from the names of the tables?
   *
   * A JDBC compliant driver always returns true.
   *
   * @return true if so
   */
  public boolean supportsDifferentTableCorrelationNames() throws SQLException {
    return true;
  }

  /**
   * Are expressions in "ORDER BY" lists supported?
   *
   * @return true if so
   */
  public boolean supportsExpressionsInOrderBy() throws SQLException {
    return true;
  }

  /**
   * Can an "ORDER BY" clause use columns not in the SELECT?
   *
   * @return true if so
   */
  public boolean supportsOrderByUnrelated() throws SQLException {
    return false;
  }

  /**
   * Is some form of "GROUP BY" clause supported?
   *
   * @return true if so
   */
  public boolean supportsGroupBy() throws SQLException {
    return true;
  }

  /**
   * Can a "GROUP BY" clause use columns not in the SELECT?
   *
   * @return true if so
   */
  public boolean supportsGroupByUnrelated() throws SQLException {
    return false;
  }

  /**
   * Can a "GROUP BY" clause add columns not in the SELECT
   * provided it specifies all the columns in the SELECT?
   *
   * @return true if so
   */
  public boolean supportsGroupByBeyondSelect() throws SQLException {
    return true;
  }

  /**
   * Is the escape character in "LIKE" clauses supported?
   *
   * A JDBC compliant driver always returns true.
   *
   * @return true if so
   */
  public boolean supportsLikeEscapeClause() throws SQLException {
    return true;
  }

  /**
   * Are multiple ResultSets from a single execute supported?
   *
   * @return true if so
   */
  public boolean supportsMultipleResultSets() throws SQLException {
    return false;
  }

  /**
   * Can we have multiple transactions open at once (on different
   * connections)?
   *
   * @return true if so
   */
  public boolean supportsMultipleTransactions() throws SQLException {
    return true;
  }

  /**
   * Can columns be defined as non-nullable?
   *
   * A JDBC compliant driver always returns true.
   *
   * @return true if so
   */
  public boolean supportsNonNullableColumns() throws SQLException {
    return true;
  }

  /**
   * Is the ODBC Minimum SQL grammar supported?
   *
   * All JDBC compliant drivers must return true.
   *
   * @return true if so
   */
  public boolean supportsMinimumSQLGrammar() throws SQLException {
    return true;
  }

  /**
   * Is the ODBC Core SQL grammar supported?
   *
   * @return true if so
   */
  public boolean supportsCoreSQLGrammar() throws SQLException {
    return true;
  }

  /**
   * Is the ODBC Extended SQL grammar supported?
   *
   * @return true if so
   */
  public boolean supportsExtendedSQLGrammar() throws SQLException {
    return false;
  }

  /**
   * Is the ANSI92 entry level SQL grammar supported?
   *
   * All JDBC compliant drivers must return true.
   *
   * @return true if so
   */
  public boolean supportsANSI92EntryLevelSQL() throws SQLException {
    return true;
  }

  /**
   * Is the ANSI92 intermediate SQL grammar supported?
   *
   * @return true if so
   */
  public boolean supportsANSI92IntermediateSQL() throws SQLException {
    return false;
  }

  /**
   * Is the ANSI92 full SQL grammar supported?
   *
   * @return true if so
   */
  public boolean supportsANSI92FullSQL() throws SQLException {
    return false;
  }

  /**
   * Is the SQL Integrity Enhancement Facility supported?
   *
   * @return true if so
   */
  public boolean supportsIntegrityEnhancementFacility() throws SQLException {
    return false;
  }

  /**
   * Is some form of outer join supported?
   *
   * @return true if so
   */
  public boolean supportsOuterJoins() throws SQLException {
    return true;
  }

  /**
   * Are full nested outer joins supported?
   *
   * @return true if so
   */
  public boolean supportsFullOuterJoins() throws SQLException {
    return false;
  }

  /**
   * Is there limited support for outer joins?  (This will be true
   * if supportFullOuterJoins is true.)
   *
   * @return true if so
   */
  public boolean supportsLimitedOuterJoins() throws SQLException {
    return true;
  }

  /**
   * What's the database vendor's preferred term for "schema"?
   *
   * @return the vendor term
   */
  public String getSchemaTerm() throws SQLException {
    String SchemaTerm = "";
    return SchemaTerm;
  }

  /**
   * What's the database vendor's preferred term for "procedure"?
   *
   * @return the vendor term
   */
  public String getProcedureTerm() throws SQLException {
    String ProcedureTerm = "";
    return ProcedureTerm;
  }

  /**
   * What's the database vendor's preferred term for "catalog"?
   *
   * @return the vendor term
   */
  public String getCatalogTerm() throws SQLException {
    String CatalogTerm = "database";
    return CatalogTerm;
  }

  /**
   * Does a catalog appear at the start of a qualified table name?
   * (Otherwise it appears at the end)
   *
   * @return true if it appears at the start
   */
  public boolean isCatalogAtStart() throws SQLException {
    return true;
  }

  /**
   * What's the separator between catalog and table name?
   *
   * @return the separator string
   */
  public String getCatalogSeparator() throws SQLException {
    String CatalogSeparator = ".";
    return CatalogSeparator;
  }

  /**
   * Can a schema name be used in a data manipulation statement?
   *
   * @return true if so
   */
  public boolean supportsSchemasInDataManipulation() throws SQLException {
    return false;
  }

  /**
   * Can a schema name be used in a procedure call statement?
   *
   * @return true if so
   */
  public boolean supportsSchemasInProcedureCalls() throws SQLException {
    return false;
  }

  /**
   * Can a schema name be used in a table definition statement?
   *
   * @return true if so
   */
  public boolean supportsSchemasInTableDefinitions() throws SQLException {
    return false;
  }

  /**
   * Can a schema name be used in an index definition statement?
   *
   * @return true if so
   */
  public boolean supportsSchemasInIndexDefinitions() throws SQLException {
    return false;
  }

  /**
   * Can a schema name be used in a privilege definition statement?
   *
   * @return true if so
   */
  public boolean supportsSchemasInPrivilegeDefinitions() throws SQLException {
    return false;
  }

  /**
   * Can a catalog name be used in a data manipulation statement?
   *
   * @return true if so
   */
  public boolean supportsCatalogsInDataManipulation() throws SQLException {
    // mysql older than 3.22 does not support this facility.
    if( conn.judgeServerVersion(3, 22, 0) >= 0 )
      return true;
    return false;
  }

  /**
   * Can a catalog name be used in a procedure call statement?
   *
   * @return true if so
   */
  public boolean supportsCatalogsInProcedureCalls() throws SQLException {
    return false;
  }

  /**
   * Can a catalog name be used in a table definition statement?
   *
   * @return true if so
   */
  public boolean supportsCatalogsInTableDefinitions() throws SQLException {
    return false;
  }

  /**
   * Can a catalog name be used in a index definition statement?
   *
   * @return true if so
   */
  public boolean supportsCatalogsInIndexDefinitions() throws SQLException {
    return false;
  }

  /**
   * Can a catalog name be used in a privilege definition statement?
   *
   * @return true if so
   */
  public boolean supportsCatalogsInPrivilegeDefinitions() throws SQLException {
    return false;
  }

  /**
   * Is positioned DELETE supported?
   *
   * @return true if so
   */
  public boolean supportsPositionedDelete() throws SQLException {
    return false;
  }

  /**
   * Is positioned UPDATE supported?
   *
   * @return true if so
   */
  public boolean supportsPositionedUpdate() throws SQLException {
    return false;
  }

  /**
   * Is SELECT for UPDATE supported?
   *
   * @return true if so
   */
  public boolean supportsSelectForUpdate() throws SQLException {
    return false;
  }

  /**
   * Are stored procedure calls using the stored procedure escape
   * syntax supported?
   *
   * @return true if so
   */
  public boolean supportsStoredProcedures() throws SQLException {
    return false;
  }

  /**
   * Are subqueries in comparison expressions supported?
   *
   * A JDBC compliant driver always returns true.
   *
   * @return true if so
   */
  public boolean supportsSubqueriesInComparisons() throws SQLException {
    return true;
  }

  /**
   * Are subqueries in exists expressions supported?
   *
   * A JDBC compliant driver always returns true.
   *
   * @return true if so
   */
  public boolean supportsSubqueriesInExists() throws SQLException {
    return false;
  }

  /**
   * Are subqueries in "in" statements supported?
   *
   * A JDBC compliant driver always returns true.
   *
   * @return true if so
   */
  public boolean supportsSubqueriesInIns() throws SQLException {
    return false;
  }

  /**
   * Are subqueries in quantified expressions supported?
   *
   * A JDBC compliant driver always returns true.
   *
   * @return true if so
   */
  public boolean supportsSubqueriesInQuantifieds() throws SQLException {
    return false;
  }

  /**
   * Are correlated subqueries supported?
   *
   * A JDBC compliant driver always returns true.
   *
   * @return true if so
   */
  public boolean supportsCorrelatedSubqueries() throws SQLException {
    return false;
  }

  /**
   * Is SQL UNION supported?
   *
   * A JDBC compliant driver always returns true.
   *
   * @return true if so
   */
  public boolean supportsUnion() throws SQLException {
    return false;
  }

  /**
   * Is SQL UNION ALL supported?
   *
   * A JDBC compliant driver always returns true.
   *
   * @return true if so
   */
  public boolean supportsUnionAll() throws SQLException {
    return false;
  }

  /**
   * Can cursors remain open across commits?
   *
   * @return true if so
   * @see Connection#disableAutoClose
   */
  public boolean supportsOpenCursorsAcrossCommit() throws SQLException {
    return false;
  }

  /**
   * Can cursors remain open across rollbacks?
   *
   * @return true if so
   * @see Connection#disableAutoClose
   */
  public boolean supportsOpenCursorsAcrossRollback() throws SQLException {
    return false;
  }

  /**
   * Can statements remain open across commits?
   *
   * @return true if so
   * @see Connection#disableAutoClose
   */
  public boolean supportsOpenStatementsAcrossCommit() throws SQLException {
    return false;
  }

  /**
   * Can statements remain open across rollbacks?
   *
   * @return true if so
   * @see Connection#disableAutoClose
   */
  public boolean supportsOpenStatementsAcrossRollback() throws SQLException {
    return false;
  }

  /**
   * How many hex characters can you have in an inline binary literal?
   *
   * @return max literal length
   */
  public int getMaxBinaryLiteralLength() throws SQLException {
    return 16777208;
  }

  /**
   * What's the max length for a character literal?
   *
   * @return max literal length
   */
  public int getMaxCharLiteralLength() throws SQLException {
    return 16777208;
  }

  /**
   * What's the limit on column name length?
   *
   * @return max literal length
   */
  public int getMaxColumnNameLength() throws SQLException {
    return 32;
  }

  /**
   * What's the maximum number of columns in a "GROUP BY" clause?
   *
   * @return max number of columns
   */
  public int getMaxColumnsInGroupBy() throws SQLException {
    return 16;
  }

  /**
   * What's the maximum number of columns allowed in an index?
   *
   * @return max columns
   */
  public int getMaxColumnsInIndex() throws SQLException {
    return 16;
  }

  /**
   * What's the maximum number of columns in an "ORDER BY" clause?
   *
   * @return max columns
   */
  public int getMaxColumnsInOrderBy() throws SQLException {
    return 16;
  }

  /**
   * What's the maximum number of columns in a "SELECT" list?
   *
   * @return max columns
   */
  public int getMaxColumnsInSelect() throws SQLException {
    return 256;
  }

  /**
   * What's maximum number of columns in a table?
   *
   * @return max columns
   */
  public int getMaxColumnsInTable() throws SQLException {
    return 512;
  }

  /**
   * How many active connections can we have at a time to this database?
   *
   * @return max number of active connections;
   *   a result of zero means that there is no limit or the limit is not known
   */
  public int getMaxConnections() throws SQLException {
    return 0;
  }

  /**
   * What's the maximum cursor name length?
   *
   * @return max cursor name length in bytes
   */
  public int getMaxCursorNameLength() throws SQLException {
    return 64;
  }

  /**
   * What's the maximum length of an index (in bytes)?
   *
   * @return max index length in bytes
   */
  public int getMaxIndexLength() throws SQLException {
    return 128;
  }

  /**
   * What's the maximum length allowed for a schema name?
   *
   * @return max name length in bytes
   */
  public int getMaxSchemaNameLength() throws SQLException {
    return 0;
  }

  /**
   * What's the maximum length of a procedure name?
   *
   * @return max name length in bytes
   */
  public int getMaxProcedureNameLength() throws SQLException {
    return 0;
  }

  /**
   * What's the maximum length of a catalog name?
   *
   * @return max name length in bytes
   */
  public int getMaxCatalogNameLength() throws SQLException {
    return 32;
  }

  /**
   * What's the maximum length of a single row?
   *
   * @return max row size in bytes
   */
  public int getMaxRowSize() throws SQLException {
    return Integer.MAX_VALUE - 8; // Max buffer size - HEADER
  }

  /**
   * Did getMaxRowSize() include LONGVARCHAR and LONGVARBINARY
   * blobs?
   *
   * @return true if so
   */
  public boolean doesMaxRowSizeIncludeBlobs() throws SQLException {
    return true;
  }

  /**
   * What's the maximum length of a SQL statement?
   *
   * @return max length in bytes
   */
  public int getMaxStatementLength() throws SQLException {
    return (256*256*256) + (256*256) + 256 - 3 - 1;
  }

  /**
   * How many active statements can we have open at one time to this
   * database?
   *
   * @return the maximum
    */
  public int getMaxStatements() throws SQLException {
    return 0;
  }

  /**
   * What's the maximum length of a table name?
   *
   * @return max name length in bytes
   */
  public int getMaxTableNameLength() throws SQLException {
    return 32;
  }

  /**
   * What's the maximum number of tables in a SELECT?
   *
   * @return the maximum
   */
  public int getMaxTablesInSelect() throws SQLException {
    return 256;
  }

  /**
   * What's the maximum length of a user name?
   *
   * @return max name length  in bytes
   */
  public int getMaxUserNameLength() throws SQLException {
    return 16;
  }

  /**
   * What's the database's default transaction isolation level?  The
   * values are defined in java.sql.Connection.
   *
   * @return the default isolation level
   * @see Connection
   */
  public int getDefaultTransactionIsolation() throws SQLException {
    return java.sql.Connection.TRANSACTION_SERIALIZABLE;
  }

  /**
   * Are transactions supported? If not, commit is a noop and the
   * isolation level is TRANSACTION_NONE.
   *
   * @return true if transactions are supported
   */
  public boolean supportsTransactions() throws SQLException {
    if( conn.judgeServerVersion(3, 23, 15) >= 0 )
      return true;
    return false;
  }

  /**
   * Does the database support the given transaction isolation level?
   *
   * @param level the values are defined in java.sql.Connection
   * @return true if so
   * @see Connection
   */
  public boolean supportsTransactionIsolationLevel(int level)
                                                       throws SQLException {
    boolean retval = false;
    switch(level)
    {
      case java.sql.Connection.TRANSACTION_NONE:
      case java.sql.Connection.TRANSACTION_READ_COMMITTED:
      case java.sql.Connection.TRANSACTION_READ_UNCOMMITTED:
      case java.sql.Connection.TRANSACTION_REPEATABLE_READ:
      case java.sql.Connection.TRANSACTION_SERIALIZABLE:
        retval = true;
        break;
      default:
        break;
    }
    return retval;
  }

  /**
   * Are both data definition and data manipulation statements
   * within a transaction supported?
   *
   * @return true if so
   */
  public boolean supportsDataDefinitionAndDataManipulationTransactions()
                                                        throws SQLException {
    return false;
  }

  /**
   * Are only data manipulation statements within a transaction
   * supported?
   *
   * @return true if so
   */
  public boolean supportsDataManipulationTransactionsOnly()
                                                        throws SQLException {
    return false;
  }

  /**
   * Does a data definition statement within a transaction force the
   * transaction to commit?
   *
   * @return true if so
   */
  public boolean dataDefinitionCausesTransactionCommit() throws SQLException {
    return false;
  }

  /**
   * Is a data definition statement within a transaction ignored?
   *
   * @return true if so
   */
  public boolean dataDefinitionIgnoredInTransactions() throws SQLException {
    return false;
  }

  /**
   * Get a description of stored procedures available in a
   * catalog.
   *
   * <P>Only procedure descriptions matching the schema and
   * procedure name criteria are returned.  They are ordered by
   * PROCEDURE_SCHEM, and PROCEDURE_NAME.
   *
   * <P>Each procedure description has the the following columns:
   *  <OL>
   *    <LI><B>PROCEDURE_CAT</B> String => procedure catalog (may be null)
   *    <LI><B>PROCEDURE_SCHEM</B> String => procedure schema (may be null)
   *    <LI><B>PROCEDURE_NAME</B> String => procedure name
   *  <LI> reserved for future use
   *  <LI> reserved for future use
   *  <LI> reserved for future use
   *    <LI><B>REMARKS</B> String => explanatory comment on the procedure
   *    <LI><B>PROCEDURE_TYPE</B> short => kind of procedure:
   *      <UL>
   *      <LI> procedureResultUnknown - May return a result
   *      <LI> procedureNoResult - Does not return a result
   *      <LI> procedureReturnsResult - Returns a result
   *      </UL>
   *  </OL>
   *
   * @param catalog a catalog name; "" retrieves those without a catalog
   * @param schemaPattern a schema name pattern; "" retrieves those
   * without a schema
   * @param procedureNamePattern a procedure name pattern
   * @return ResultSet each row is a procedure description
   * @see #getSearchStringEscape
   */
  public ResultSet getProcedures(String catalog, String schemaPattern,
                            String procedureNamePattern) throws SQLException {
    gweMysqlColumn[] cols = new gweMysqlColumn[8];
    cols[0] = new gweMysqlColumn("", "PROCEDURE_CAT", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[1] = new gweMysqlColumn("", "PROCEDURE_SCHEM", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[2] = new gweMysqlColumn("", "PROCEDURE_NAME", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[3] = new gweMysqlColumn("", "reserved", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[4] = new gweMysqlColumn("", "reserved", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[5] = new gweMysqlColumn("", "reserved", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[6] = new gweMysqlColumn("", "REMARKS", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[7] = new gweMysqlColumn("", "PROCEDURE_TYPE", -1,
                      gweMysqlMetaResult.FIELD_TYPE_SHORT, (short)-1, -1);

    return new gweMysqlVectorResultSet(conn, cols, new Vector());
  }

  /**
   * Get a description of a catalog's stored procedure parameters
   * and result columns.
   *
   * <P>Only descriptions matching the schema, procedure and
   * parameter name criteria are returned.  They are ordered by
   * PROCEDURE_SCHEM and PROCEDURE_NAME. Within this, the return value,
   * if any, is first. Next are the parameter descriptions in call
   * order. The column descriptions follow in column number order.
   *
   * <P>Each row in the ResultSet is a parameter desription or
   * column description with the following fields:
   *  <OL>
   *    <LI><B>PROCEDURE_CAT</B> String => procedure catalog (may be null)
   *    <LI><B>PROCEDURE_SCHEM</B> String => procedure schema (may be null)
   *    <LI><B>PROCEDURE_NAME</B> String => procedure name
   *    <LI><B>COLUMN_NAME</B> String => column/parameter name
   *    <LI><B>COLUMN_TYPE</B> Short => kind of column/parameter:
   *      <UL>
   *      <LI> procedureColumnUnknown - nobody knows
   *      <LI> procedureColumnIn - IN parameter
   *      <LI> procedureColumnInOut - INOUT parameter
   *      <LI> procedureColumnOut - OUT parameter
   *      <LI> procedureColumnReturn - procedure return value
   *      <LI> procedureColumnResult - result column in ResultSet
   *      </UL>
   *  <LI><B>DATA_TYPE</B> short => SQL type from java.sql.Types
   *    <LI><B>TYPE_NAME</B> String => SQL type name
   *    <LI><B>PRECISION</B> int => precision
   *    <LI><B>LENGTH</B> int => length in bytes of data
   *    <LI><B>SCALE</B> short => scale
   *    <LI><B>RADIX</B> short => radix
   *    <LI><B>NULLABLE</B> short => can it contain NULL?
   *      <UL>
   *      <LI> procedureNoNulls - does not allow NULL values
   *      <LI> procedureNullable - allows NULL values
   *      <LI> procedureNullableUnknown - nullability unknown
   *      </UL>
   *    <LI><B>REMARKS</B> String => comment describing parameter/column
   *  </OL>
   *
   * <P><B>Note:</B> Some databases may not return the column
   * descriptions for a procedure. Additional columns beyond
   * REMARKS can be defined by the database.
   *
   * @param catalog a catalog name; "" retrieves those without a catalog
   * @param schemaPattern a schema name pattern; "" retrieves those
   * without a schema
   * @param procedureNamePattern a procedure name pattern
   * @param columnNamePattern a column name pattern
   * @return ResultSet each row is a stored procedure parameter or
   *      column description
   * @see #getSearchStringEscape
   */
  public ResultSet getProcedureColumns(String catalog, String schemaPattern,
                        String procedureNamePattern, String columnNamePattern)
                                                        throws SQLException {
    gweMysqlColumn[] cols = new gweMysqlColumn[14];
    cols[0] = new gweMysqlColumn("", "TABLE_CAT", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[1] = new gweMysqlColumn("", "PROCEDURE_CAT", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[2] = new gweMysqlColumn("", "PROCEDURE_SCHEM", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[3] = new gweMysqlColumn("", "PROCEDURE_NAME", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[4] = new gweMysqlColumn("", "COLUMN_NAME", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[5] = new gweMysqlColumn("", "COLUMN_TYPE", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[6] = new gweMysqlColumn("", "DATA_TYPE", -1,
                      gweMysqlMetaResult.FIELD_TYPE_SHORT, (short)-1, -1);
    cols[7] = new gweMysqlColumn("", "TYPE_NAME", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[8] = new gweMysqlColumn("", "PRECISION", -1,
                      gweMysqlMetaResult.FIELD_TYPE_LONG, (short)-1, -1);
    cols[9] = new gweMysqlColumn("", "LENGTH", -1,
                      gweMysqlMetaResult.FIELD_TYPE_LONG, (short)-1, -1);
    cols[10]= new gweMysqlColumn("", "SCALE", -1,
                      gweMysqlMetaResult.FIELD_TYPE_SHORT, (short)-1, -1);
    cols[11]= new gweMysqlColumn("", "RADIX", -1,
                      gweMysqlMetaResult.FIELD_TYPE_SHORT, (short)-1, -1);
    cols[12]= new gweMysqlColumn("", "NULLABLE", -1,
                      gweMysqlMetaResult.FIELD_TYPE_SHORT, (short)-1, -1);
    cols[13]= new gweMysqlColumn("", "REMARKS", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);

    return new gweMysqlVectorResultSet(conn, cols, new Vector());
  }

  /**
   * Get a description of tables available in a catalog.
   *
   * <P>Only table descriptions matching the catalog, schema, table
   * name and type criteria are returned.  They are ordered by
   * TABLE_TYPE, TABLE_SCHEM and TABLE_NAME.
   *
   * <P>Each table description has the following columns:
   *  <OL>
   *    <LI><B>TABLE_CAT</B> String => table catalog (may be null)
   *    <LI><B>TABLE_SCHEM</B> String => table schema (may be null)
   *    <LI><B>TABLE_NAME</B> String => table name
   *    <LI><B>TABLE_TYPE</B> String => table type.  Typical types are "TABLE",
   *                    "VIEW", "SYSTEM TABLE", "GLOBAL TEMPORARY",
   *                    "LOCAL TEMPORARY", "ALIAS", "SYNONYM".
   *    <LI><B>REMARKS</B> String => explanatory comment on the table
   *  </OL>
   *
   * <P><B>Note:</B> Some databases may not return information for
   * all tables.
   *
   * @param catalog a catalog name; "" retrieves those without a catalog
   * @param schemaPattern a schema name pattern; "" retrieves those
   * without a schema
   * @param tableNamePattern a table name pattern
   * @param types a list of table types to include; null returns all types
   * @return ResultSet each row is a table description
   * @see #getSearchStringEscape
   */
  public ResultSet getTables(String catalog, String schemaPattern,
                             String tableNamePattern, String[] types)
                                                        throws SQLException {
    gweMysqlColumn[] cols = new gweMysqlColumn[5];
    cols[0] = new gweMysqlColumn("", "TABLE_CAT", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[1] = new gweMysqlColumn("", "TABLE_SCHEM", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[2] = new gweMysqlColumn("", "TABLE_NAME", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[3] = new gweMysqlColumn("", "TABLE_TYPE", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[4] = new gweMysqlColumn("", "REMARKS", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);

    String temp_catalog;
    String sqlstr = "SHOW TABLES";
    if( catalog != null && !catalog.equals("") )
      temp_catalog = catalog;
    else
      temp_catalog = conn.getCatalog();
    sqlstr += " FROM " + temp_catalog;
    if( tableNamePattern != null && !tableNamePattern.equals("") )
      sqlstr += " LIKE '" + trimQuote(tableNamePattern) + "'";
    sqlstr += ";";

    Vector rows = new Vector();
    Statement stmt = null;
    ResultSet rs = null;
    try {
      stmt = conn.createStatement();
      rs = stmt.executeQuery(sqlstr);
      while(rs.next())
      {
        String tablename = rs.getString(1);

        Vector rowdata = new Vector();
        rowdata.addElement(temp_catalog);  // TABLE_CAT
        rowdata.addElement("");
        if( add_quote == true )     // TABLE_NAME
          rowdata.addElement("`" + tablename + "`");
        else
          rowdata.addElement(tablename);
        rowdata.addElement("TABLE");       // TABLE_TYPE
        rowdata.addElement("");
        rows.addElement(rowdata);
      }
    }
    finally {
      if( rs != null )
      {
        try { rs.close(); }
        catch(Exception ex) { }
      }
      rs = null;
      if( stmt != null )
      {
        try { stmt.close(); }
        catch(Exception ex) { }
      }
      stmt = null;
    }

    return new gweMysqlVectorResultSet(conn, cols, rows);
  }

  /**
   * Get the schema names available in this database.  The results
   * are ordered by schema name.
   *
   * <P>The schema column is:
   *  <OL>
   *    <LI><B>TABLE_SCHEM</B> String => schema name
   *  </OL>
   *
   * @return ResultSet each row has a single String column that is a
   * schema name
   */
  public ResultSet getSchemas() throws SQLException {
    gweMysqlColumn[] cols = new gweMysqlColumn[1];
    cols[0] = new gweMysqlColumn("", "TABLE_SCHEM", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);

    return new gweMysqlVectorResultSet(conn, cols, new Vector());
  }

  /**
   * Get the catalog names available in this database.  The results
   * are ordered by catalog name.
   *
   * <P>The catalog column is:
   *  <OL>
   *    <LI><B>TABLE_CAT</B> String => catalog name
   *  </OL>
   *
   * @return ResultSet each row has a single String column that is a
   * catalog name
   */
  public ResultSet getCatalogs() throws SQLException {
    gweMysqlColumn[] cols = new gweMysqlColumn[1];
    cols[0] = new gweMysqlColumn("", "TABLE_CAT", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);

    String sqlstr = "SHOW DATABASES;";
    Vector rows = new Vector();
    Statement stmt = null;
    ResultSet rs = null;
    try {
      stmt = conn.createStatement();
      rs = stmt.executeQuery(sqlstr);
      while(rs.next())
      {
        String dbname = rs.getString(1);
        Vector rowdata = new Vector();
        rowdata.addElement(dbname);  // TABLE_CAT
        rows.addElement(rowdata);
      }
    }
    finally {
      if( rs != null )
      {
        try { rs.close(); }
        catch(Exception ex) { }
      }
      rs = null;
      if( stmt != null )
      {
        try { stmt.close(); }
        catch(Exception ex) { }
      }
      stmt = null;
    }

    return new gweMysqlVectorResultSet(conn, cols, rows);
  }

  /**
   * Get the table types available in this database.  The results
   * are ordered by table type.
   *
   * <P>The table type is:
   *  <OL>
   *    <LI><B>TABLE_TYPE</B> String => table type.  Typical types are "TABLE",
   *                    "VIEW", "SYSTEM TABLE", "GLOBAL TEMPORARY",
   *                    "LOCAL TEMPORARY", "ALIAS", "SYNONYM".
   *  </OL>
   *
   * @return ResultSet each row has a single String column that is a
   * table type
   */
  public ResultSet getTableTypes() throws SQLException {
    gweMysqlColumn[] cols = new gweMysqlColumn[1];
    cols[0] = new gweMysqlColumn("", "TABLE_TYPE", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);

    Vector rows = new Vector();
    Vector rowdata = new Vector();
    rowdata.addElement("TABLE");  // TABLE_TYPE
    rows.addElement(rowdata);

    return new gweMysqlVectorResultSet(conn, cols, rows);
  }

  /**
   * Get a description of table columns available in a catalog.
   *
   * <P>Only column descriptions matching the catalog, schema, table
   * and column name criteria are returned.  They are ordered by
   * TABLE_SCHEM, TABLE_NAME and ORDINAL_POSITION.
   *
   * <P>Each column description has the following columns:
   *  <OL>
   *    <LI><B>TABLE_CAT</B> String => table catalog (may be null)
   *    <LI><B>TABLE_SCHEM</B> String => table schema (may be null)
   *    <LI><B>TABLE_NAME</B> String => table name
   *    <LI><B>COLUMN_NAME</B> String => column name
   *    <LI><B>DATA_TYPE</B> short => SQL type from java.sql.Types
   *    <LI><B>TYPE_NAME</B> String => Data source dependent type name
   *    <LI><B>COLUMN_SIZE</B> int => column size.  For char or date
   *        types this is the maximum number of characters, for numeric or
   *        decimal types this is precision.
   *    <LI><B>BUFFER_LENGTH</B> is not used.
   *    <LI><B>DECIMAL_DIGITS</B> int => the number of fractional digits
   *    <LI><B>NUM_PREC_RADIX</B> int => Radix (typically either 10 or 2)
   *    <LI><B>NULLABLE</B> int => is NULL allowed?
   *      <UL>
   *      <LI> columnNoNulls - might not allow NULL values
   *      <LI> columnNullable - definitely allows NULL values
   *      <LI> columnNullableUnknown - nullability unknown
   *      </UL>
   *    <LI><B>REMARKS</B> String => comment describing column (may be null)
   *    <LI><B>COLUMN_DEF</B> String => default value (may be null)
   *    <LI><B>SQL_DATA_TYPE</B> int => unused
   *    <LI><B>SQL_DATETIME_SUB</B> int => unused
   *    <LI><B>CHAR_OCTET_LENGTH</B> int => for char types the
   *       maximum number of bytes in the column
   *    <LI><B>ORDINAL_POSITION</B> int => index of column in table
   *      (starting at 1)
   *    <LI><B>IS_NULLABLE</B> String => "NO" means column definitely
   *      does not allow NULL values; "YES" means the column might
   *      allow NULL values.  An empty string means nobody knows.
   *  </OL>
   *
   * @param catalog a catalog name; "" retrieves those without a catalog
   * @param schemaPattern a schema name pattern; "" retrieves those
   * without a schema
   * @param tableNamePattern a table name pattern
   * @param columnNamePattern a column name pattern
   * @return ResultSet each row is a column description
   * @see #getSearchStringEscape
   */
  public ResultSet getColumns(String catalog, String schemaPattern,
                           String tableNamePattern, String columnNamePattern)
                                                         throws SQLException {
    ResultSet tables = null;
    Vector tablenames = new Vector();
    try {
      tables = getTables(catalog, schemaPattern, tableNamePattern, null);
      while(tables.next())
      {
        String tablename = tables.getString("TABLE_NAME");
        tablenames.addElement(tablename);
      }
    }
    finally {
      if( tables != null )
      {
        try { tables.close(); }
        catch(Exception e) { }
      }
      tables = null;
    }

    gweMysqlColumn[] cols = new gweMysqlColumn[18];
    cols[0] = new gweMysqlColumn("", "TABLE_CAT", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[1] = new gweMysqlColumn("", "TABLE_SCHEM", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[2] = new gweMysqlColumn("", "TABLE_NAME", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[3] = new gweMysqlColumn("", "COLUMN_NAME", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[4] = new gweMysqlColumn("", "DATA_TYPE", -1,
                      gweMysqlMetaResult.FIELD_TYPE_SHORT, (short)-1, -1);
    cols[5] = new gweMysqlColumn("", "TYPE_NAME", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[6] = new gweMysqlColumn("", "COLUMN_SIZE", -1,
                      gweMysqlMetaResult.FIELD_TYPE_LONG, (short)-1, -1);
    cols[7] = new gweMysqlColumn("", "BUFFER_LENGTH", -1,
                      gweMysqlMetaResult.FIELD_TYPE_LONG, (short)-1, -1);
    cols[8] = new gweMysqlColumn("", "DECIMAL_DIGITS", -1,
                      gweMysqlMetaResult.FIELD_TYPE_LONG, (short)-1, -1);
    cols[9] = new gweMysqlColumn("", "NUM_PREC_RADIX", -1,
                      gweMysqlMetaResult.FIELD_TYPE_LONG, (short)-1, -1);
    cols[10]= new gweMysqlColumn("", "NULLABLE", -1,
                      gweMysqlMetaResult.FIELD_TYPE_LONG, (short)-1, -1);
    cols[11]= new gweMysqlColumn("", "REMARKS", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[12]= new gweMysqlColumn("", "COLUMN_DEF", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[13]= new gweMysqlColumn("", "SQL_DATA_TYPE", -1,
                      gweMysqlMetaResult.FIELD_TYPE_LONG, (short)-1, -1);
    cols[14]= new gweMysqlColumn("", "SQL_DATETIME_SUB", -1,
                      gweMysqlMetaResult.FIELD_TYPE_LONG, (short)-1, -1);
    cols[15]= new gweMysqlColumn("", "CHAR_OCTET_LENGTH", -1,
                      gweMysqlMetaResult.FIELD_TYPE_LONG, (short)-1, -1);
    cols[16]= new gweMysqlColumn("", "ORDINAL_POSITION", -1,
                      gweMysqlMetaResult.FIELD_TYPE_LONG, (short)-1, -1);
    cols[17]= new gweMysqlColumn("", "IS_NULLABLE", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);

    Vector rows = new Vector();
    String temp_catalog;
    if( catalog != null && !catalog.equals("") )
      temp_catalog = catalog;
    else
      temp_catalog = conn.getCatalog();
    for(int i = 0; i < tablenames.size(); i++)
    {
      String sqlstr = "SHOW COLUMNS";
      String tablename = (String)tablenames.elementAt(i);
      sqlstr += " FROM " + temp_catalog + "." + tablename;
      if( columnNamePattern != null && !columnNamePattern.equals("") )
        sqlstr += " LIKE '" + trimQuote(columnNamePattern) + "'";
      sqlstr += ";";

      Statement stmt = null;
      ResultSet rs = null;
      try {
        stmt = conn.createStatement();
        rs = stmt.executeQuery(sqlstr);
        int ordinal_position = 0;
        while(rs.next())
        {
          String mysql_field = rs.getString("Field");
          String mysql_type  = rs.getString("Type");
          int jdbc_col_type = getJDBCColumnType(mysql_type);
          String native_col_type = getNativeColumnType(mysql_type);
          int col_size = getColumnSize(mysql_type);
          int dig_size = getDeciamlDigits(mysql_type);
          String null_str = rs.getString("Null");
          boolean allow_null = false;
          if( null_str != null && null_str.equals("YES") )
            allow_null = true;
          String mysql_extra = "";
          try { mysql_extra = rs.getString("Extra"); }
          catch(Exception e) { mysql_extra = ""; }
          String mysql_default = "";
          try { mysql_default = rs.getString("Default"); }
          catch(Exception e) { mysql_default = ""; }
          ordinal_position++;

          Vector rowdata = new Vector();
          rowdata.addElement(temp_catalog);                    // TABLE_CAT
          rowdata.addElement("");
          if( add_quote == true )                       // TABLE_NAME
            rowdata.addElement("`" + tablename + "`");
          else
            rowdata.addElement(tablename);
          if( add_quote == true )                       // COLUMN_NAME
            rowdata.addElement("`" + mysql_field + "`");
          else
            rowdata.addElement(mysql_field);
          rowdata.addElement(new Short((short)jdbc_col_type)); // DATA_TYPE
          rowdata.addElement(native_col_type);                 // TYPE_NAME
          rowdata.addElement(new Integer(col_size));           // COLUMN_SIZE
          rowdata.addElement(new Integer(-1));
          rowdata.addElement(new Integer(dig_size));           // DECIMAL_DIGITS
          rowdata.addElement(new Integer(10));                 // NUM_PREC_RADIX
          if( allow_null == true )                      // NULLABLE
            rowdata.addElement(new Integer(DatabaseMetaData.columnNullable));
          else
            rowdata.addElement(new Integer(DatabaseMetaData.columnNoNulls));
          rowdata.addElement(mysql_extra);                     // REMARKS
          rowdata.addElement(mysql_default);                   // COLUMN_DEF
          rowdata.addElement(new Integer(-1));
          rowdata.addElement(new Integer(-1));
          rowdata.addElement(new Integer(col_size));           // CHAR_OCTET_LENGTH
          rowdata.addElement(new Integer(ordinal_position));   // ORDINAL_POSITION
          if( allow_null == true )                      // IS_NULLABLE
            rowdata.addElement("YES");
          else
            rowdata.addElement("NO");
          rows.addElement(rowdata);
        }
      }
      finally {
        if( rs != null )
        {
          try { rs.close(); }
          catch(Exception ex) { }
        }
        rs = null;
        if( stmt != null )
        {
          try { stmt.close(); }
          catch(Exception ex) { }
        }
        stmt = null;
      }
    }
    return new gweMysqlVectorResultSet(conn, cols, rows);
  }

  // mysql_type = varchar(32) etc...
  private String getNativeColumnType(String mysql_type) {
    String native_col_type;
    int sep_idx = mysql_type.indexOf("(");
    if( sep_idx != -1 )
      native_col_type = mysql_type.substring(0, sep_idx);
    else
      native_col_type = mysql_type;
    return native_col_type.toUpperCase();
  }

  // mysql_type = varchar(32) etc...
  private int getJDBCColumnType(String mysql_type) throws SQLException {
    String temp_type = getNativeColumnType(mysql_type);

    if( temp_type.equals("TINYINT") )
        return java.sql.Types.TINYINT;
    else if( temp_type.equals("SMALLINT") )
      return java.sql.Types.SMALLINT;
    else if( temp_type.equals("MEDIUMINT") )
      return java.sql.Types.SMALLINT;
    else if( temp_type.equals("INT") )
      return java.sql.Types.INTEGER;
    else if( temp_type.equals("INTEGER") )
      return java.sql.Types.INTEGER;
    else if( temp_type.equals("BIGINT") )
      return java.sql.Types.BIGINT;
    else if( temp_type.equals("INT24") )
      return java.sql.Types.BIGINT;
    else if( temp_type.equals("REAL") )
      return java.sql.Types.REAL;
    else if( temp_type.equals("FLOAT") )
      return java.sql.Types.FLOAT;
    else if( temp_type.equals("DECIMAL") )
      return java.sql.Types.DECIMAL;
    else if( temp_type.equals("NUMERIC") )
      return java.sql.Types.NUMERIC;
    else if( temp_type.equals("DOUBLE") )
      return java.sql.Types.DOUBLE;
    else if( temp_type.equals("CHAR") )
      return java.sql.Types.CHAR;
    else if( temp_type.equals("VARCHAR") )
      return java.sql.Types.VARCHAR;
    else if( temp_type.equals("DATE") )
      return java.sql.Types.DATE;
    else if( temp_type.equals("TIME") )
      return java.sql.Types.TIME;
    else if( temp_type.equals("TIMESTAMP") )
      return java.sql.Types.TIMESTAMP;
    else if( temp_type.equals("DATETIME") )
      return java.sql.Types.TIMESTAMP;
    else if( temp_type.equals("TINYBLOB") )
      return java.sql.Types.VARBINARY;
    else if( temp_type.equals("BLOB") )
      return java.sql.Types.LONGVARBINARY;
    else if( temp_type.equals("MEDIUMBLOB") )
      return java.sql.Types.LONGVARBINARY;
    else if( temp_type.equals("LONGBLOB") )
      return java.sql.Types.LONGVARBINARY;
    else if( temp_type.equals("TINYTEXT") )
      return java.sql.Types.VARCHAR;
    else if( temp_type.equals("TEXT") )
      return java.sql.Types.LONGVARCHAR;
    else if( temp_type.equals("MEDIUMTEXT") )
      return java.sql.Types.LONGVARCHAR;
    else if( temp_type.equals("LONGTEXT") ) // Thanks for INDEN-san
      return java.sql.Types.LONGVARCHAR;
    else if( temp_type.equals("ENUM") )
      return java.sql.Types.CHAR;
    else if( temp_type.equals("SET") )
      return java.sql.Types.CHAR;
    else if( temp_type.equals("YEAR") ) // YEAR support!!
      return java.sql.Types.SMALLINT;
    throw new SQLException("Unknown data type.");
  }

  // mysql_type = NUMERIC(10,2) etc...
  private int getColumnSize(String mysql_type) throws SQLException {
    int start_idx = mysql_type.indexOf("(");
    int sep_idx   = mysql_type.indexOf(",");
    int last_idx  = mysql_type.lastIndexOf(")");
    String temp_type = getNativeColumnType(mysql_type);
    try {
      if( temp_type.equals("ENUM") || temp_type.equals("SET") )
      {
        if( start_idx == -1 )
          return 255;

        String value_str = mysql_type.substring(start_idx + 1, last_idx);
        StringTokenizer st = new StringTokenizer(value_str, ",");
        int retval = 0;
        while(st.hasMoreTokens())
        {
          String token = st.nextToken();
          int temp_size = token.length() - 2; // 'xxxxxx'
          if( temp_size > retval )
            retval = temp_size;
        }
        return retval;
      }
      else if( sep_idx != -1 ) // NUMERIC(10,2)
      {
        String precision_str = mysql_type.substring(start_idx + 1, sep_idx);
        return Integer.valueOf(precision_str).intValue();
      }
      else if( start_idx != -1 ) // VARCHAR(32)
      {
        String size_str = mysql_type.substring(start_idx + 1, last_idx);
        return Integer.valueOf(size_str).intValue();
      }
      else
      {
        int retval = 0;
        if( temp_type.equals("TINYINT") )
          retval = 1;
        else if( temp_type.equals("SMALLINT") )
          retval = 6;
        else if( temp_type.equals("MEDIUMINT") )
          retval = 6;
        else if( temp_type.equals("INT") )
          retval = 11;
        else if( temp_type.equals("INTEGER") )
          retval = 11;
        else if( temp_type.equals("BIGINT") )
          retval = 25;
        else if( temp_type.equals("INT24") )
          retval = 25;
        else if( temp_type.equals("REAL") )
          retval = 12;
        else if( temp_type.equals("FLOAT") )
          retval = 12;
        else if( temp_type.equals("DECIMAL") )
          retval = 12;
        else if( temp_type.equals("NUMERIC") )
          retval = 12;
        else if( temp_type.equals("DOUBLE") )
          retval = 22;
        else if( temp_type.equals("CHAR") )
          retval = 1;
        else if( temp_type.equals("VARCHAR") )
          retval = 255;
        else if( temp_type.equals("DATE") )
          retval = 10;
        else if( temp_type.equals("TIME") )
          retval = 8;
        else if( temp_type.equals("TIMESTAMP") )
          retval = 19;
        else if( temp_type.equals("DATETIME") )
          retval = 19;
        else if( temp_type.equals("TINYBLOB") )
          retval = 255;
        else if( temp_type.equals("BLOB") )
          retval = 65535;
        else if( temp_type.equals("MEDIUMBLOB") )
          retval = 16277215;
        else if( temp_type.equals("LONGBLOB") )
          retval = 2147483647;
        else if( temp_type.equals("TINYTEXT") )
          retval = 255;
        else if( temp_type.equals("TEXT") )
          retval = 65535;
        else if( temp_type.equals("MEDIUMTEXT") )
          retval = 16277215;
        else if( temp_type.equals("LONGTEXT") ) // Thanks for INDEN-san
          retval = 2147483647;
        else if( temp_type.equals("YEAR") ) // YEAR support!!
          retval = 4;
        return retval;
      }
    }
    catch(Exception e) {
      throw new SQLException(e.getMessage());
    }
  }

  // mysql_type = NUMERIC(10,2) etc...
  private int getDeciamlDigits(String mysql_type) throws SQLException {
    int sep_idx = mysql_type.indexOf(",");
    int last_idx = mysql_type.lastIndexOf(")");
    String temp_type = getNativeColumnType(mysql_type);
    try {
      if( temp_type.equals("ENUM") || temp_type.equals("SET") )
      {
        return 0;
      }
      if( sep_idx != -1 && last_idx != -1 ) // NUMERIC(10,2)
      {
         String fractional_str = mysql_type.substring(sep_idx + 1, last_idx);
         return Integer.valueOf(fractional_str).intValue();
      }
      return 0;
    }
    catch(Exception e) {
      throw new SQLException(e.getMessage());
    }
  }

  /**
   * Get a description of the access rights for a table's columns.
   *
   * <P>Only privileges matching the column name criteria are
   * returned.  They are ordered by COLUMN_NAME and PRIVILEGE.
   *
   * <P>Each privilige description has the following columns:
   *  <OL>
   *    <LI><B>TABLE_CAT</B> String => table catalog (may be null)
   *    <LI><B>TABLE_SCHEM</B> String => table schema (may be null)
   *    <LI><B>TABLE_NAME</B> String => table name
   *    <LI><B>COLUMN_NAME</B> String => column name
   *    <LI><B>GRANTOR</B> => grantor of access (may be null)
   *    <LI><B>GRANTEE</B> String => grantee of access
   *    <LI><B>PRIVILEGE</B> String => name of access (SELECT,
   *      INSERT, UPDATE, REFRENCES, ...)
   *    <LI><B>IS_GRANTABLE</B> String => "YES" if grantee is permitted
   *      to grant to others; "NO" if not; null if unknown
   *  </OL>
   *
   * @param catalog a catalog name; "" retrieves those without a catalog
   * @param schema a schema name; "" retrieves those without a schema
   * @param table a table name
   * @param columnNamePattern a column name pattern
   * @return ResultSet each row is a column privilege description
   * @see #getSearchStringEscape
   */
  public ResultSet getColumnPrivileges(String catalog, String schema,
                                       String table, String columnNamePattern)
                                                          throws SQLException {
    gweMysqlColumn[] cols = new gweMysqlColumn[8];
    cols[0] = new gweMysqlColumn("", "TABLE_CAT", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[1] = new gweMysqlColumn("", "TABLE_SCHEM", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[2] = new gweMysqlColumn("", "TABLE_NAME", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[3] = new gweMysqlColumn("", "COLUMN_NAME", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[4] = new gweMysqlColumn("", "GRANTOR", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[5] = new gweMysqlColumn("", "GRANTEE", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[6] = new gweMysqlColumn("", "PRIVILEGE", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[7] = new gweMysqlColumn("", "IS_GRANTABLE", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);

    return new gweMysqlVectorResultSet(conn, cols, new Vector());
  }

  /**
   * Get a description of the access rights for each table available
   * in a catalog.
   *
   * <P>Only privileges matching the schema and table name
   * criteria are returned.  They are ordered by TABLE_SCHEM,
   * TABLE_NAME, and PRIVILEGE.
   *
   * <P>Each privilige description has the following columns:
   *  <OL>
   *    <LI><B>TABLE_CAT</B> String => table catalog (may be null)
   *    <LI><B>TABLE_SCHEM</B> String => table schema (may be null)
   *    <LI><B>TABLE_NAME</B> String => table name
   *    <LI><B>COLUMN_NAME</B> String => column name
   *    <LI><B>GRANTOR</B> => grantor of access (may be null)
   *    <LI><B>GRANTEE</B> String => grantee of access
   *    <LI><B>PRIVILEGE</B> String => name of access (SELECT,
   *      INSERT, UPDATE, REFRENCES, ...)
   *    <LI><B>IS_GRANTABLE</B> String => "YES" if grantee is permitted
   *      to grant to others; "NO" if not; null if unknown
   *  </OL>
   *
   * @param catalog a catalog name; "" retrieves those without a catalog
   * @param schemaPattern a schema name pattern; "" retrieves those
   * without a schema
   * @param tableNamePattern a table name pattern
   * @return ResultSet each row is a table privilege description
   * @see #getSearchStringEscape
   */
  public ResultSet getTablePrivileges(String catalog, String schemaPattern,
                                 String tableNamePattern) throws SQLException {
    gweMysqlColumn[] cols = new gweMysqlColumn[8];
    cols[0] = new gweMysqlColumn("", "TABLE_CAT", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[1] = new gweMysqlColumn("", "TABLE_SCHEM", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[2] = new gweMysqlColumn("", "TABLE_NAME", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[3] = new gweMysqlColumn("", "COLUMN_NAME", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[4] = new gweMysqlColumn("", "GRANTOR", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[5] = new gweMysqlColumn("", "GRANTEE", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[6] = new gweMysqlColumn("", "PRIVILEGE", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[7] = new gweMysqlColumn("", "IS_GRANTABLE", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);

    return new gweMysqlVectorResultSet(conn, cols, new Vector());
  }

  /**
   * Get a description of a table's optimal set of columns that
   * uniquely identifies a row. They are ordered by SCOPE.
   *
   * <P>Each column description has the following columns:
   *  <OL>
   *    <LI><B>SCOPE</B> short => actual scope of result
   *      <UL>
   *      <LI> bestRowTemporary - very temporary, while using row
   *      <LI> bestRowTransaction - valid for remainder of current transaction
   *      <LI> bestRowSession - valid for remainder of current session
   *      </UL>
   *    <LI><B>COLUMN_NAME</B> String => column name
   *    <LI><B>DATA_TYPE</B> short => SQL data type from java.sql.Types
   *    <LI><B>TYPE_NAME</B> String => Data source dependent type name
   *    <LI><B>COLUMN_SIZE</B> int => precision
   *    <LI><B>BUFFER_LENGTH</B> int => not used
   *    <LI><B>DECIMAL_DIGITS</B> short  => scale
   *    <LI><B>PSEUDO_COLUMN</B> short => is this a pseudo column
   *      like an Oracle ROWID
   *      <UL>
   *      <LI> bestRowUnknown - may or may not be pseudo column
   *      <LI> bestRowNotPseudo - is NOT a pseudo column
   *      <LI> bestRowPseudo - is a pseudo column
   *      </UL>
   *  </OL>
   *
   * @param catalog a catalog name; "" retrieves those without a catalog
   * @param schema a schema name; "" retrieves those without a schema
   * @param table a table name
   * @param scope the scope of interest; use same values as SCOPE
   * @param nullable include columns that are nullable?
   * @return ResultSet each row is a column description
   */
  public ResultSet getBestRowIdentifier(String catalog, String schema,
                                     String table, int scope, boolean nullable)
                                                          throws SQLException {
    gweMysqlColumn[] cols = new gweMysqlColumn[8];
    cols[0] = new gweMysqlColumn("", "SCOPE", -1,
                      gweMysqlMetaResult.FIELD_TYPE_SHORT, (short)-1, -1);
    cols[1] = new gweMysqlColumn("", "COLUMN_NAME", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[2] = new gweMysqlColumn("", "DATA_TYPE", -1,
                      gweMysqlMetaResult.FIELD_TYPE_SHORT, (short)-1, -1);
    cols[3] = new gweMysqlColumn("", "TYPE_NAME", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[4] = new gweMysqlColumn("", "COLUMN_SIZE", -1,
                      gweMysqlMetaResult.FIELD_TYPE_LONG, (short)-1, -1);
    cols[5] = new gweMysqlColumn("", "BUFFER_LENGTH", -1,
                      gweMysqlMetaResult.FIELD_TYPE_LONG, (short)-1, -1);
    cols[6] = new gweMysqlColumn("", "DECIMAL_DIGITS", -1,
                      gweMysqlMetaResult.FIELD_TYPE_SHORT, (short)-1, -1);
    cols[7] = new gweMysqlColumn("", "PSEUDO_COLUMN", -1,
                      gweMysqlMetaResult.FIELD_TYPE_SHORT, (short)-1, -1);

    Vector rows = new Vector();
    String temp_catalog;
    if( catalog != null && !catalog.equals("") )
      temp_catalog = catalog;
    else
      temp_catalog = conn.getCatalog();
    String sqlstr = "SHOW COLUMNS FROM " + temp_catalog + "." + table + ";";

    Statement stmt = null;
    ResultSet rs = null;
    try {
      stmt = conn.createStatement();
      rs = stmt.executeQuery(sqlstr);
      while(rs.next())
      {
        String mysql_field = rs.getString("Field");
        String mysql_type  = rs.getString("Type");
        int jdbc_col_type = getJDBCColumnType(mysql_type);
        String native_col_type = getNativeColumnType(mysql_type);
        int col_size = getColumnSize(mysql_type);
        int dig_size = getDeciamlDigits(mysql_type);
        String null_str = rs.getString("Null");
        boolean allow_null = false;
        if( null_str != null && null_str.equals("YES") )
          allow_null = true;
        String mysql_key = rs.getString("Key");

        if( !mysql_key.equals("PRI") )
          continue;
        if( nullable == false && allow_null == true )
          continue;

        Vector rowdata = new Vector();
        rowdata.addElement(new Short((short)DatabaseMetaData.bestRowSession));// SCOPE
        if( add_quote == true )                          // COLUMN_NAME
          rowdata.addElement("`" + mysql_field + "`");
        else
          rowdata.addElement(mysql_field);
        rowdata.addElement(new Short((short)jdbc_col_type));    // DATA_TYPE
        rowdata.addElement(native_col_type);                    // TYPE_NAME
        rowdata.addElement(new Integer(col_size));              // COLUMN_SIZE
        rowdata.addElement(new Integer(-1));
        rowdata.addElement(new Short((short)dig_size));         // DECIMAL_DIGITS
        rowdata.addElement(new Short((short)DatabaseMetaData.bestRowNotPseudo));
                                                         // PSEUDO_COLUMN
        rows.addElement(rowdata);
      }
    }
    finally {
      if( rs != null )
      {
        try { rs.close(); }
        catch(Exception ex) { }
      }
      rs = null;
      if( stmt != null )
      {
        try { stmt.close(); }
        catch(Exception ex) { }
      }
      stmt = null;
    }
    return new gweMysqlVectorResultSet(conn, cols, rows);
  }

  /**
   * Get a description of a table's columns that are automatically
   * updated when any value in a row is updated.  They are
   * unordered.
   *
   * <P>Each column description has the following columns:
   *  <OL>
   *    <LI><B>SCOPE</B> short => is not used
   *    <LI><B>COLUMN_NAME</B> String => column name
   *    <LI><B>DATA_TYPE</B> short => SQL data type from java.sql.Types
   *    <LI><B>TYPE_NAME</B> String => Data source dependent type name
   *    <LI><B>COLUMN_SIZE</B> int => precision
   *    <LI><B>BUFFER_LENGTH</B> int => length of column value in bytes
   *    <LI><B>DECIMAL_DIGITS</B> short  => scale
   *    <LI><B>PSEUDO_COLUMN</B> short => is this a pseudo column
   *      like an Oracle ROWID
   *      <UL>
   *      <LI> versionColumnUnknown - may or may not be pseudo column
   *      <LI> versionColumnNotPseudo - is NOT a pseudo column
   *      <LI> versionColumnPseudo - is a pseudo column
   *      </UL>
   *  </OL>
   *
   * @param catalog a catalog name; "" retrieves those without a catalog
   * @param schema a schema name; "" retrieves those without a schema
   * @param table a table name
   * @return ResultSet each row is a column description
   */
  public ResultSet getVersionColumns(String catalog, String schema,
                                          String table) throws SQLException {
    gweMysqlColumn[] cols = new gweMysqlColumn[8];
    cols[0] = new gweMysqlColumn("", "SCOPE", -1,
                      gweMysqlMetaResult.FIELD_TYPE_SHORT, (short)-1, -1);
    cols[1] = new gweMysqlColumn("", "COLUMN_NAME", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[2] = new gweMysqlColumn("", "DATA_TYPE", -1,
                      gweMysqlMetaResult.FIELD_TYPE_SHORT, (short)-1, -1);
    cols[3] = new gweMysqlColumn("", "TYPE_NAME", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[4] = new gweMysqlColumn("", "COLUMN_SIZE", -1,
                      gweMysqlMetaResult.FIELD_TYPE_LONG, (short)-1, -1);
    cols[5] = new gweMysqlColumn("", "BUFFER_LENGTH", -1,
                      gweMysqlMetaResult.FIELD_TYPE_LONG, (short)-1, -1);
    cols[6] = new gweMysqlColumn("", "DECIMAL_DIGITS", -1,
                      gweMysqlMetaResult.FIELD_TYPE_SHORT, (short)-1, -1);
    cols[7] = new gweMysqlColumn("", "PSEUDO_COLUMN", -1,
                      gweMysqlMetaResult.FIELD_TYPE_SHORT, (short)-1, -1);

    return new gweMysqlVectorResultSet(conn, cols, new Vector());
  }

  /**
   * Get a description of a table's primary key columns.  They
   * are ordered by COLUMN_NAME.
   *
   * <P>Each column description has the following columns:
   *  <OL>
   *    <LI><B>TABLE_CAT</B> String => table catalog (may be null)
   *    <LI><B>TABLE_SCHEM</B> String => table schema (may be null)
   *    <LI><B>TABLE_NAME</B> String => table name
   *    <LI><B>COLUMN_NAME</B> String => column name
   *    <LI><B>KEY_SEQ</B> short => sequence number within primary key
   *    <LI><B>PK_NAME</B> String => primary key name (may be null)
   *  </OL>
   *
   * @param catalog a catalog name; "" retrieves those without a catalog
   * @param schema a schema name pattern; "" retrieves those
   * without a schema
   * @param table a table name
   * @return ResultSet each row is a primary key column description
   */
  public ResultSet getPrimaryKeys(String catalog, String schema,
                                  String table) throws SQLException {
    gweMysqlColumn[] cols = new gweMysqlColumn[6];
    cols[0] = new gweMysqlColumn("", "TABLE_CAT", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[1] = new gweMysqlColumn("", "TABLE_SCHEM", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[2] = new gweMysqlColumn("", "TABLE_NAME", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[3] = new gweMysqlColumn("", "COLUMN_NAME", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[4] = new gweMysqlColumn("", "KEY_SEQ", -1,
                      gweMysqlMetaResult.FIELD_TYPE_SHORT, (short)-1, -1);
    cols[5] = new gweMysqlColumn("", "PK_NAME", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);

    Vector rows = new Vector();
    String temp_catalog;
    if( catalog != null && !catalog.equals("") )
      temp_catalog = catalog;
    else
      temp_catalog = conn.getCatalog();
    String sqlstr = "SHOW KEYS FROM " + temp_catalog + "." + table + ";";

    Statement stmt = null;
    ResultSet rs = null;
    try {
      stmt = conn.createStatement();
      rs = stmt.executeQuery(sqlstr);
      while(rs.next())
      {
        String mysql_tbl = rs.getString("Table");
        String mysql_key = rs.getString("Key_name");
        String mysql_col = rs.getString("Column_name");
        String mysql_seq = rs.getString("Seq_in_index");

        if( !mysql_key.equals("PRIMARY") )
          continue;

        Vector rowdata = new Vector();
        rowdata.addElement(temp_catalog);             // TABLE_CAT
        rowdata.addElement("");                       // TABLE_SCHEM
        if( add_quote == true )                // TABLE_NAME
          rowdata.addElement("`" + mysql_tbl + "`");
        else
          rowdata.addElement(mysql_tbl);
        if( add_quote == true )                // COLUMN_NAME
          rowdata.addElement("`" + mysql_col + "`");
        else
          rowdata.addElement(mysql_col);
        try {
          rowdata.addElement(new Short(mysql_seq));   // KEY_SEQ
        }
        catch(Exception e) {
          throw new SQLException(e.getMessage());
        }
        rowdata.addElement(mysql_col);                // PK_NAME
        rows.addElement(rowdata);
      }
    }
    finally {
      if( rs != null )
      {
        try { rs.close(); }
        catch(Exception ex) { }
      }
      rs = null;
      if( stmt != null )
      {
        try { stmt.close(); }
        catch(Exception ex) { }
      }
      stmt = null;
    }
    return new gweMysqlVectorResultSet(conn, cols, rows);
  }

  /**
   * Get a description of the primary key columns that are
   * referenced by a table's foreign key columns (the primary keys
   * imported by a table).  They are ordered by PKTABLE_CAT,
   * PKTABLE_SCHEM, PKTABLE_NAME, and KEY_SEQ.
   *
   * <P>Each primary key column description has the following columns:
   *  <OL>
   *    <LI><B>PKTABLE_CAT</B> String => primary key table catalog
   *      being imported (may be null)
   *    <LI><B>PKTABLE_SCHEM</B> String => primary key table schema
   *      being imported (may be null)
   *    <LI><B>PKTABLE_NAME</B> String => primary key table name
   *      being imported
   *    <LI><B>PKCOLUMN_NAME</B> String => primary key column name
   *      being imported
   *    <LI><B>FKTABLE_CAT</B> String => foreign key table catalog (may be null)
   *    <LI><B>FKTABLE_SCHEM</B> String => foreign key table schema (may be null)
   *    <LI><B>FKTABLE_NAME</B> String => foreign key table name
   *    <LI><B>FKCOLUMN_NAME</B> String => foreign key column name
   *    <LI><B>KEY_SEQ</B> short => sequence number within foreign key
   *    <LI><B>UPDATE_RULE</B> short => What happens to
   *       foreign key when primary is updated:
   *      <UL>
   *      <LI> importedKeyCascade - change imported key to agree
   *               with primary key update
   *      <LI> importedKeyRestrict - do not allow update of primary
   *               key if it has been imported
   *      <LI> importedKeySetNull - change imported key to NULL if
   *               its primary key has been updated
   *      </UL>
   *    <LI><B>DELETE_RULE</B> short => What happens to
   *      the foreign key when primary is deleted.
   *      <UL>
   *      <LI> importedKeyCascade - delete rows that import a deleted key
   *      <LI> importedKeyRestrict - do not allow delete of primary
   *               key if it has been imported
   *      <LI> importedKeySetNull - change imported key to NULL if
   *               its primary key has been deleted
   *      </UL>
   *    <LI><B>FK_NAME</B> String => foreign key name (may be null)
   *    <LI><B>PK_NAME</B> String => primary key name (may be null)
   *    <LI><B>DEFERRABILITY</B> short => can the evaluation of foreign key
   *      constraints be deferred until commit
   *      <UL>
   *      <LI> importedKeyInitiallyDeferred - see SQL92 for definition
   *      <LI> importedKeyInitiallyImmediate - see SQL92 for definition
   *      <LI> importedKeyNotDeferrable - see SQL92 for definition
   *      </UL>
   *  </OL>
   *
   * @param catalog a catalog name; "" retrieves those without a catalog
   * @param schema a schema name pattern; "" retrieves those
   * without a schema
   * @param table a table name
   * @return ResultSet each row is a primary key column description
   * @see #getExportedKeys
   */
  public ResultSet getImportedKeys(String catalog, String schema,
                                   String table) throws SQLException {
    gweMysqlColumn[] cols = new gweMysqlColumn[14];
    cols[0] = new gweMysqlColumn("", "PKTABLE_CAT", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[1] = new gweMysqlColumn("", "PKTABLE_SCHEM", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[2] = new gweMysqlColumn("", "PKTABLE_NAME", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[3] = new gweMysqlColumn("", "PKCOLUMN_NAME", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[4] = new gweMysqlColumn("", "FKTABLE_CAT", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[5] = new gweMysqlColumn("", "FKTABLE_SCHEM", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[6] = new gweMysqlColumn("", "FKTABLE_NAME", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[7] = new gweMysqlColumn("", "FKCOLUMN_NAME", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[8] = new gweMysqlColumn("", "KEY_SEQ", -1,
                      gweMysqlMetaResult.FIELD_TYPE_SHORT, (short)-1, -1);
    cols[9] = new gweMysqlColumn("", "UPDATE_RULE", -1,
                      gweMysqlMetaResult.FIELD_TYPE_SHORT, (short)-1, -1);
    cols[10]= new gweMysqlColumn("", "DELETE_RULE", -1,
                      gweMysqlMetaResult.FIELD_TYPE_SHORT, (short)-1, -1);
    cols[11]= new gweMysqlColumn("", "FK_NAME", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[12]= new gweMysqlColumn("", "PK_NAME", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[13]= new gweMysqlColumn("", "DEFERRABILITY", -1,
                      gweMysqlMetaResult.FIELD_TYPE_SHORT, (short)-1, -1);

    return new gweMysqlVectorResultSet(conn, cols, new Vector());
  }

  /**
   * Gets a description of the foreign key columns that reference a
   * table's primary key columns (the foreign keys exported by a
   * table).  They are ordered by FKTABLE_CAT, FKTABLE_SCHEM,
   * FKTABLE_NAME, and KEY_SEQ.
   *
   * <P>Each foreign key column description has the following columns:
   *  <OL>
   *    <LI><B>PKTABLE_CAT</B> String => primary key table catalog (may be null)
   *    <LI><B>PKTABLE_SCHEM</B> String => primary key table schema (may be null)
   *    <LI><B>PKTABLE_NAME</B> String => primary key table name
   *    <LI><B>PKCOLUMN_NAME</B> String => primary key column name
   *    <LI><B>FKTABLE_CAT</B> String => foreign key table catalog (may be null)
   *      being exported (may be null)
   *    <LI><B>FKTABLE_SCHEM</B> String => foreign key table schema (may be null)
   *      being exported (may be null)
   *    <LI><B>FKTABLE_NAME</B> String => foreign key table name
   *      being exported
   *    <LI><B>FKCOLUMN_NAME</B> String => foreign key column name
   *      being exported
   *    <LI><B>KEY_SEQ</B> short => sequence number within foreign key
   *    <LI><B>UPDATE_RULE</B> short => What happens to
   *       foreign key when primary is updated:
   *      <UL>
   *      <LI> importedNoAction - do not allow update of primary
   *               key if it has been imported
   *      <LI> importedKeyCascade - change imported key to agree
   *               with primary key update
   *      <LI> importedKeySetNull - change imported key to NULL if
   *               its primary key has been updated
   *      <LI> importedKeySetDefault - change imported key to default values
   *               if its primary key has been updated
   *      <LI> importedKeyRestrict - same as importedKeyNoAction
   *                                 (for ODBC 2.x compatibility)
   *      </UL>
   *    <LI><B>DELETE_RULE</B> short => What happens to
   *      the foreign key when primary is deleted.
   *      <UL>
   *      <LI> importedKeyNoAction - do not allow delete of primary
   *               key if it has been imported
   *      <LI> importedKeyCascade - delete rows that import a deleted key
   *      <LI> importedKeySetNull - change imported key to NULL if
   *               its primary key has been deleted
   *      <LI> importedKeyRestrict - same as importedKeyNoAction
   *                                 (for ODBC 2.x compatibility)
   *      <LI> importedKeySetDefault - change imported key to default if
   *               its primary key has been deleted
   *      </UL>
   *    <LI><B>FK_NAME</B> String => foreign key name (may be null)
   *    <LI><B>PK_NAME</B> String => primary key name (may be null)
   *    <LI><B>DEFERRABILITY</B> short => can the evaluation of foreign key
   *      constraints be deferred until commit
   *      <UL>
   *      <LI> importedKeyInitiallyDeferred - see SQL92 for definition
   *      <LI> importedKeyInitiallyImmediate - see SQL92 for definition
   *      <LI> importedKeyNotDeferrable - see SQL92 for definition
   *      </UL>
   *  </OL>
   *
   * @param catalog a catalog name; "" retrieves those without a
   * catalog; null means drop catalog name from the selection criteria
   * @param schema a schema name; "" retrieves those
   * without a schema
   * @param table a table name
   * @return <code>ResultSet</code> - each row is a foreign key column description
   * @exception SQLException if a database access error occurs
   * @see #getImportedKeys
   */
  public ResultSet getExportedKeys(String catalog, String schema,
                                   String table) throws SQLException {
    gweMysqlColumn[] cols = new gweMysqlColumn[14];
    cols[0] = new gweMysqlColumn("", "PKTABLE_CAT", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[1] = new gweMysqlColumn("", "PKTABLE_SCHEM", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[2] = new gweMysqlColumn("", "PKTABLE_NAME", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[3] = new gweMysqlColumn("", "PKCOLUMN_NAME", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[4] = new gweMysqlColumn("", "FKTABLE_CAT", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[5] = new gweMysqlColumn("", "FKTABLE_SCHEM", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[6] = new gweMysqlColumn("", "FKTABLE_NAME", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[7] = new gweMysqlColumn("", "FKCOLUMN_NAME", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[8] = new gweMysqlColumn("", "KEY_SEQ", -1,
                      gweMysqlMetaResult.FIELD_TYPE_SHORT, (short)-1, -1);
    cols[9] = new gweMysqlColumn("", "UPDATE_RULE", -1,
                      gweMysqlMetaResult.FIELD_TYPE_SHORT, (short)-1, -1);
    cols[10]= new gweMysqlColumn("", "DELETE_RULE", -1,
                      gweMysqlMetaResult.FIELD_TYPE_SHORT, (short)-1, -1);
    cols[11]= new gweMysqlColumn("", "FK_NAME", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[12]= new gweMysqlColumn("", "PK_NAME", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[13]= new gweMysqlColumn("", "DEFERRABILITY", -1,
                      gweMysqlMetaResult.FIELD_TYPE_SHORT, (short)-1, -1);

    return new gweMysqlVectorResultSet(conn, cols, new Vector());
  }

  /**
   * Gets a description of the foreign key columns in the foreign key
   * table that reference the primary key columns of the primary key
   * table (describe how one table imports another's key.) This
   * should normally return a single foreign key/primary key pair
   * (most tables only import a foreign key from a table once.)  They
   * are ordered by FKTABLE_CAT, FKTABLE_SCHEM, FKTABLE_NAME, and
   * KEY_SEQ.
   *
   * <P>Each foreign key column description has the following columns:
   *  <OL>
   *    <LI><B>PKTABLE_CAT</B> String => primary key table catalog (may be null)
   *    <LI><B>PKTABLE_SCHEM</B> String => primary key table schema (may be null)
   *    <LI><B>PKTABLE_NAME</B> String => primary key table name
   *    <LI><B>PKCOLUMN_NAME</B> String => primary key column name
   *    <LI><B>FKTABLE_CAT</B> String => foreign key table catalog (may be null)
   *      being exported (may be null)
   *    <LI><B>FKTABLE_SCHEM</B> String => foreign key table schema (may be null)
   *      being exported (may be null)
   *    <LI><B>FKTABLE_NAME</B> String => foreign key table name
   *      being exported
   *    <LI><B>FKCOLUMN_NAME</B> String => foreign key column name
   *      being exported
   *    <LI><B>KEY_SEQ</B> short => sequence number within foreign key
   *    <LI><B>UPDATE_RULE</B> short => What happens to
   *       foreign key when primary is updated:
   *      <UL>
   *      <LI> importedNoAction - do not allow update of primary
   *               key if it has been imported
   *      <LI> importedKeyCascade - change imported key to agree
   *               with primary key update
   *      <LI> importedKeySetNull - change imported key to NULL if
   *               its primary key has been updated
   *      <LI> importedKeySetDefault - change imported key to default values
   *               if its primary key has been updated
   *      <LI> importedKeyRestrict - same as importedKeyNoAction
   *                                 (for ODBC 2.x compatibility)
   *      </UL>
   *    <LI><B>DELETE_RULE</B> short => What happens to
   *      the foreign key when primary is deleted.
   *      <UL>
   *      <LI> importedKeyNoAction - do not allow delete of primary
   *               key if it has been imported
   *      <LI> importedKeyCascade - delete rows that import a deleted key
   *      <LI> importedKeySetNull - change imported key to NULL if
   *               its primary key has been deleted
   *      <LI> importedKeyRestrict - same as importedKeyNoAction
   *                                 (for ODBC 2.x compatibility)
   *      <LI> importedKeySetDefault - change imported key to default if
   *               its primary key has been deleted
   *      </UL>
   *    <LI><B>FK_NAME</B> String => foreign key name (may be null)
   *    <LI><B>PK_NAME</B> String => primary key name (may be null)
   *    <LI><B>DEFERRABILITY</B> short => can the evaluation of foreign key
   *      constraints be deferred until commit
   *      <UL>
   *      <LI> importedKeyInitiallyDeferred - see SQL92 for definition
   *      <LI> importedKeyInitiallyImmediate - see SQL92 for definition
   *      <LI> importedKeyNotDeferrable - see SQL92 for definition
   *      </UL>
   *  </OL>
   *
   * @param primaryCatalog a catalog name; "" retrieves those without a
   * catalog; null means drop catalog name from the selection criteria
   * @param primarySchema a schema name; "" retrieves those
   * without a schema
   * @param primaryTable the table name that exports the key
   * @param foreignCatalog a catalog name; "" retrieves those without a
   * catalog; null means drop catalog name from the selection criteria
   * @param foreignSchema a schema name; "" retrieves those
   * without a schema
   * @param foreignTable the table name that imports the key
   * @return <code>ResultSet</code> - each row is a foreign key column description
   * @exception SQLException if a database access error occurs
   * @see #getImportedKeys
   */
  public ResultSet getCrossReference(String primaryCatalog,
                               String primarySchema, String primaryTable,
                               String foreignCatalog, String foreignSchema,
                                    String foreignTable) throws SQLException {
    gweMysqlColumn[] cols = new gweMysqlColumn[14];
    cols[0] = new gweMysqlColumn("", "PKTABLE_CAT", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[1] = new gweMysqlColumn("", "PKTABLE_SCHEM", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[2] = new gweMysqlColumn("", "PKTABLE_NAME", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[3] = new gweMysqlColumn("", "PKCOLUMN_NAME", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[4] = new gweMysqlColumn("", "FKTABLE_CAT", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[5] = new gweMysqlColumn("", "FKTABLE_SCHEM", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[6] = new gweMysqlColumn("", "FKTABLE_NAME", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[7] = new gweMysqlColumn("", "FKCOLUMN_NAME", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[8] = new gweMysqlColumn("", "KEY_SEQ", -1,
                      gweMysqlMetaResult.FIELD_TYPE_SHORT, (short)-1, -1);
    cols[9] = new gweMysqlColumn("", "UPDATE_RULE", -1,
                      gweMysqlMetaResult.FIELD_TYPE_SHORT, (short)-1, -1);
    cols[10]= new gweMysqlColumn("", "DELETE_RULE", -1,
                      gweMysqlMetaResult.FIELD_TYPE_SHORT, (short)-1, -1);
    cols[11]= new gweMysqlColumn("", "FK_NAME", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[12]= new gweMysqlColumn("", "PK_NAME", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[13]= new gweMysqlColumn("", "DEFERRABILITY", -1,
                      gweMysqlMetaResult.FIELD_TYPE_SHORT, (short)-1, -1);

    return new gweMysqlVectorResultSet(conn, cols, new Vector());
  }

  /**
   * Gets a description of all the standard SQL types supported by
   * this database. They are ordered by DATA_TYPE and then by how
   * closely the data type maps to the corresponding JDBC SQL type.
   *
   * <P>Each type description has the following columns:
   *  <OL>
   *    <LI><B>TYPE_NAME</B> String => Type name
   *    <LI><B>DATA_TYPE</B> short => SQL data type from java.sql.Types
   *    <LI><B>PRECISION</B> int => maximum precision
   *    <LI><B>LITERAL_PREFIX</B> String => prefix used to quote a literal
   *      (may be null)
   *    <LI><B>LITERAL_SUFFIX</B> String => suffix used to quote a literal
            (may be null)
   *    <LI><B>CREATE_PARAMS</B> String => parameters used in creating
   *      the type (may be null)
   *    <LI><B>NULLABLE</B> short => can you use NULL for this type?
   *      <UL>
   *      <LI> typeNoNulls - does not allow NULL values
   *      <LI> typeNullable - allows NULL values
   *      <LI> typeNullableUnknown - nullability unknown
   *      </UL>
   *    <LI><B>CASE_SENSITIVE</B> boolean=> is it case sensitive?
   *    <LI><B>SEARCHABLE</B> short => can you use "WHERE" based on this type:
   *      <UL>
   *      <LI> typePredNone - No support
   *      <LI> typePredChar - Only supported with WHERE .. LIKE
   *      <LI> typePredBasic - Supported except for WHERE .. LIKE
   *      <LI> typeSearchable - Supported for all WHERE ..
   *      </UL>
   *    <LI><B>UNSIGNED_ATTRIBUTE</B> boolean => is it unsigned?
   *    <LI><B>FIXED_PREC_SCALE</B> boolean => can it be a money value?
   *    <LI><B>AUTO_INCREMENT</B> boolean => can it be used for an
   *      auto-increment value?
   *    <LI><B>LOCAL_TYPE_NAME</B> String => localized version of type name
   *      (may be null)
   *    <LI><B>MINIMUM_SCALE</B> short => minimum scale supported
   *    <LI><B>MAXIMUM_SCALE</B> short => maximum scale supported
   *    <LI><B>SQL_DATA_TYPE</B> int => unused
   *    <LI><B>SQL_DATETIME_SUB</B> int => unused
   *    <LI><B>NUM_PREC_RADIX</B> int => usually 2 or 10
   *  </OL>
   *
   * @return <code>ResultSet</code> - each row is an SQL type description
   * @exception SQLException if a database access error occurs
   */
  public ResultSet getTypeInfo() throws SQLException {
    gweMysqlColumn[] cols = new gweMysqlColumn[18];
    cols[0] = new gweMysqlColumn("", "TYPE_NAME", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[1] = new gweMysqlColumn("", "DATA_TYPE", -1,
                      gweMysqlMetaResult.FIELD_TYPE_SHORT, (short)-1, -1);
    cols[2] = new gweMysqlColumn("", "PRECISION", -1,
                      gweMysqlMetaResult.FIELD_TYPE_LONG, (short)-1, -1);
    cols[3] = new gweMysqlColumn("", "LITERAL_PREFIX", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[4] = new gweMysqlColumn("", "LITERAL_SUFFIX", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[5] = new gweMysqlColumn("", "CREATE_PARAMS", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[6] = new gweMysqlColumn("", "NULLABLE", -1,
                      gweMysqlMetaResult.FIELD_TYPE_SHORT, (short)-1, -1);
    cols[7] = new gweMysqlColumn("", "CASE_SENSITIVE", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[8] = new gweMysqlColumn("", "SEARCHABLE", -1,
                      gweMysqlMetaResult.FIELD_TYPE_SHORT, (short)-1, -1);
    cols[9] = new gweMysqlColumn("", "UNSIGNED_ATTRIBUTE", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[10]= new gweMysqlColumn("", "FIXED_PREC_SCALE", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[11]= new gweMysqlColumn("", "AUTO_INCREMENT", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[12]= new gweMysqlColumn("", "LOCAL_TYPE_NAME", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[13]= new gweMysqlColumn("", "MINIMUM_SCALE", -1,
                      gweMysqlMetaResult.FIELD_TYPE_SHORT, (short)-1, -1);
    cols[14]= new gweMysqlColumn("", "MAXIMUM_SCALE", -1,
                      gweMysqlMetaResult.FIELD_TYPE_SHORT, (short)-1, -1);
    cols[15]= new gweMysqlColumn("", "SQL_DATA_TYPE", -1,
                      gweMysqlMetaResult.FIELD_TYPE_LONG, (short)-1, -1);
    cols[16]= new gweMysqlColumn("", "SQL_DATETIME_SUB", -1,
                      gweMysqlMetaResult.FIELD_TYPE_LONG, (short)-1, -1);
    cols[17]= new gweMysqlColumn("", "NUM_PREC_RADIX", -1,
                      gweMysqlMetaResult.FIELD_TYPE_LONG, (short)-1, -1);

    Vector rows = new Vector();
    Vector rowdata;

    //////////////////////////////////////////////////////////////////////
    // mysql : FIELD_TYPE_TINY
    // JDBC  : Types.TINYINT
    rowdata = new Vector();
    rowdata.addElement("TINYINT");                            // TYPE_NAME
    rowdata.addElement(new Short((short)Types.TINYINT));      // DATA_TYPE
    rowdata.addElement(new Integer(3));                       // PRECISION
    rowdata.addElement("");                                   // LITERAL_PREFIX
    rowdata.addElement("");                                   // LITERAL_SUFFIX
    rowdata.addElement("(M) UNSIGNED ZEROFILL");              // CREATE_PARAMS
    rowdata.addElement(new Short((short)DatabaseMetaData.typeNullable)); // NULLABLE
    rowdata.addElement("false");                              // CASE_SENSITIVE
    rowdata.addElement(new Short((short)DatabaseMetaData.typeSearchable)); // SEARCHABLE
    rowdata.addElement("true");                               // UNSIGNED_ATTRIBUTE
    rowdata.addElement("false");                              // FIXED_PREC_SCALE
    rowdata.addElement("true");                               // AUTO_INCREMENT
    rowdata.addElement("TINYINT");                            // LOCAL_TYPE_NAME
    rowdata.addElement(new Short((short)0));                  // MINIMUM_SCALE
    rowdata.addElement(new Short((short)0));                  // MAXIMUM_SCALE
    rowdata.addElement(new Integer(-1));
    rowdata.addElement(new Integer(-1));
    rowdata.addElement(new Integer(10));                      // NUM_PREC_RADIX
    rows.addElement(rowdata);

    //////////////////////////////////////////////////////////////////////
    // mysql : FIELD_TYPE_SHORT
    // JDBC  : Types.SMALLINT
    rowdata = new Vector();
    rowdata.addElement("SMALLINT");                           // TYPE_NAME
    rowdata.addElement(new Short((short)Types.SMALLINT));     // DATA_TYPE
    rowdata.addElement(new Integer(5));                       // PRECISION
    rowdata.addElement("");                                   // LITERAL_PREFIX
    rowdata.addElement("");                                   // LITERAL_SUFFIX
    rowdata.addElement("(M) UNSIGNED ZEROFILL");              // CREATE_PARAMS
    rowdata.addElement(new Short((short)DatabaseMetaData.typeNullable)); // NULLABLE
    rowdata.addElement("false");                              // CASE_SENSITIVE
    rowdata.addElement(new Short((short)DatabaseMetaData.typeSearchable)); // SEARCHABLE
    rowdata.addElement("true");                               // UNSIGNED_ATTRIBUTE
    rowdata.addElement("false");                              // FIXED_PREC_SCALE
    rowdata.addElement("true");                               // AUTO_INCREMENT
    rowdata.addElement("SMALLINT");                           // LOCAL_TYPE_NAME
    rowdata.addElement(new Short((short)0));                  // MINIMUM_SCALE
    rowdata.addElement(new Short((short)0));                  // MAXIMUM_SCALE
    rowdata.addElement(new Integer(-1));
    rowdata.addElement(new Integer(-1));
    rowdata.addElement(new Integer(10));                      // NUM_PREC_RADIX
    rows.addElement(rowdata);

    //////////////////////////////////////////////////////////////////////
    // mysql : FIELD_TYPE_INT24
    // JDBC  : Types.INTEGER
    rowdata = new Vector();
    rowdata.addElement("MEDIUMINT");                          // TYPE_NAME
    rowdata.addElement(new Short((short)Types.INTEGER));      // DATA_TYPE
    rowdata.addElement(new Integer(7));                       // PRECISION
    rowdata.addElement("");                                   // LITERAL_PREFIX
    rowdata.addElement("");                                   // LITERAL_SUFFIX
    rowdata.addElement("(M) UNSIGNED ZEROFILL");              // CREATE_PARAMS
    rowdata.addElement(new Short((short)DatabaseMetaData.typeNullable)); // NULLABLE
    rowdata.addElement("false");                              // CASE_SENSITIVE
    rowdata.addElement(new Short((short)DatabaseMetaData.typeSearchable)); // SEARCHABLE
    rowdata.addElement("true");                               // UNSIGNED_ATTRIBUTE
    rowdata.addElement("false");                              // FIXED_PREC_SCALE
    rowdata.addElement("true");                               // AUTO_INCREMENT
    rowdata.addElement("MEDIUMINT");                          // LOCAL_TYPE_NAME
    rowdata.addElement(new Short((short)0));                  // MINIMUM_SCALE
    rowdata.addElement(new Short((short)0));                  // MAXIMUM_SCALE
    rowdata.addElement(new Integer(-1));
    rowdata.addElement(new Integer(-1));
    rowdata.addElement(new Integer(10));                      // NUM_PREC_RADIX
    rows.addElement(rowdata);

    //////////////////////////////////////////////////////////////////////
    // mysql : FIELD_TYPE_LONG
    // JDBC  : Types.INTEGER
    rowdata = new Vector();
    rowdata.addElement("INTEGER");                            // TYPE_NAME
    rowdata.addElement(new Short((short)Types.INTEGER));      // DATA_TYPE
    rowdata.addElement(new Integer(10));                      // PRECISION
    rowdata.addElement("");                                   // LITERAL_PREFIX
    rowdata.addElement("");                                   // LITERAL_SUFFIX
    rowdata.addElement("(M) UNSIGNED ZEROFILL");              // CREATE_PARAMS
    rowdata.addElement(new Short((short)DatabaseMetaData.typeNullable)); // NULLABLE
    rowdata.addElement("false");                              // CASE_SENSITIVE
    rowdata.addElement(new Short((short)DatabaseMetaData.typeSearchable)); // SEARCHABLE
    rowdata.addElement("true");                               // UNSIGNED_ATTRIBUTE
    rowdata.addElement("false");                              // FIXED_PREC_SCALE
    rowdata.addElement("true");                               // AUTO_INCREMENT
    rowdata.addElement("INTEGER");                            // LOCAL_TYPE_NAME
    rowdata.addElement(new Short((short)0));                  // MINIMUM_SCALE
    rowdata.addElement(new Short((short)0));                  // MAXIMUM_SCALE
    rowdata.addElement(new Integer(-1));
    rowdata.addElement(new Integer(-1));
    rowdata.addElement(new Integer(10));                      // NUM_PREC_RADIX
    rows.addElement(rowdata);

    //////////////////////////////////////////////////////////////////////
    // mysql : FIELD_TYPE_LONGLONG
    // JDBC  : Types.BIGINT
    rowdata = new Vector();
    rowdata.addElement("BIGINT");                             // TYPE_NAME
    rowdata.addElement(new Short((short)Types.BIGINT));       // DATA_TYPE
    rowdata.addElement(new Integer(19));                      // PRECISION
    rowdata.addElement("");                                   // LITERAL_PREFIX
    rowdata.addElement("");                                   // LITERAL_SUFFIX
    rowdata.addElement("(M) UNSIGNED ZEROFILL");              // CREATE_PARAMS
    rowdata.addElement(new Short((short)DatabaseMetaData.typeNullable)); // NULLABLE
    rowdata.addElement("false");                              // CASE_SENSITIVE
    rowdata.addElement(new Short((short)DatabaseMetaData.typeSearchable)); // SEARCHABLE
    rowdata.addElement("true");                               // UNSIGNED_ATTRIBUTE
    rowdata.addElement("false");                              // FIXED_PREC_SCALE
    rowdata.addElement("true");                               // AUTO_INCREMENT
    rowdata.addElement("BIGINT");                             // LOCAL_TYPE_NAME
    rowdata.addElement(new Short((short)0));                  // MINIMUM_SCALE
    rowdata.addElement(new Short((short)0));                  // MAXIMUM_SCALE
    rowdata.addElement(new Integer(-1));
    rowdata.addElement(new Integer(-1));
    rowdata.addElement(new Integer(10));                      // NUM_PREC_RADIX
    rows.addElement(rowdata);

    //////////////////////////////////////////////////////////////////////
    // mysql : FIELD_TYPE_FLOAT
    // JDBC  : Types.FLOAT
    rowdata = new Vector();
    rowdata.addElement("FLOAT");                              // TYPE_NAME
    rowdata.addElement(new Short((short)Types.FLOAT));        // DATA_TYPE
    rowdata.addElement(new Integer(10));                      // PRECISION
    rowdata.addElement("");                                   // LITERAL_PREFIX
    rowdata.addElement("");                                   // LITERAL_SUFFIX
    rowdata.addElement("(M,D) ZEROFILL");                     // CREATE_PARAMS
    rowdata.addElement(new Short((short)DatabaseMetaData.typeNullable)); // NULLABLE
    rowdata.addElement("false");                              // CASE_SENSITIVE
    rowdata.addElement(new Short((short)DatabaseMetaData.typeSearchable)); // SEARCHABLE
    rowdata.addElement("false");                              // UNSIGNED_ATTRIBUTE
    rowdata.addElement("false");                              // FIXED_PREC_SCALE
    rowdata.addElement("true");                               // AUTO_INCREMENT
    rowdata.addElement("FLOAT");                              // LOCAL_TYPE_NAME
    rowdata.addElement(new Short((short)-38));                // MINIMUM_SCALE
    rowdata.addElement(new Short((short)38));                 // MAXIMUM_SCALE
    rowdata.addElement(new Integer(-1));
    rowdata.addElement(new Integer(-1));
    rowdata.addElement(new Integer(10));                      // NUM_PREC_RADIX
    rows.addElement(rowdata);

    //////////////////////////////////////////////////////////////////////
    // mysql : FIELD_TYPE_DOUBLE
    // JDBC  : Types.DOUBLE
    rowdata = new Vector();
    rowdata.addElement("DOUBLE");                             // TYPE_NAME
    rowdata.addElement(new Short((short)Types.DOUBLE));       // DATA_TYPE
    rowdata.addElement(new Integer(17));                      // PRECISION
    rowdata.addElement("");                                   // LITERAL_PREFIX
    rowdata.addElement("");                                   // LITERAL_SUFFIX
    rowdata.addElement("(M,D) ZEROFILL");                     // CREATE_PARAMS
    rowdata.addElement(new Short((short)DatabaseMetaData.typeNullable)); // NULLABLE
    rowdata.addElement("false");                              // CASE_SENSITIVE
    rowdata.addElement(new Short((short)DatabaseMetaData.typeSearchable)); // SEARCHABLE
    rowdata.addElement("false");                              // UNSIGNED_ATTRIBUTE
    rowdata.addElement("false");                              // FIXED_PREC_SCALE
    rowdata.addElement("true");                               // AUTO_INCREMENT
    rowdata.addElement("DOUBLE");                             // LOCAL_TYPE_NAME
    rowdata.addElement(new Short((short)-308));               // MINIMUM_SCALE
    rowdata.addElement(new Short((short)308));                // MAXIMUM_SCALE
    rowdata.addElement(new Integer(-1));
    rowdata.addElement(new Integer(-1));
    rowdata.addElement(new Integer(10));                      // NUM_PREC_RADIX
    rows.addElement(rowdata);

    //////////////////////////////////////////////////////////////////////
    // mysql : FIELD_TYPE_DECIMAL
    // JDBC  : Types.DECIMAL
    rowdata = new Vector();
    rowdata.addElement("DECIMAL");                            // TYPE_NAME
    rowdata.addElement(new Short((short)Types.DECIMAL));      // DATA_TYPE
    rowdata.addElement(new Integer(17));                      // PRECISION
    rowdata.addElement("");                                   // LITERAL_PREFIX
    rowdata.addElement("");                                   // LITERAL_SUFFIX
    rowdata.addElement("(M,D) ZEROFILL");                     // CREATE_PARAMS
    rowdata.addElement(new Short((short)DatabaseMetaData.typeNullable)); // NULLABLE
    rowdata.addElement("false");                              // CASE_SENSITIVE
    rowdata.addElement(new Short((short)DatabaseMetaData.typeSearchable)); // SEARCHABLE
    rowdata.addElement("false");                              // UNSIGNED_ATTRIBUTE
    rowdata.addElement("false");                              // FIXED_PREC_SCALE
    rowdata.addElement("true");                               // AUTO_INCREMENT
    rowdata.addElement("DECIMAL");                            // LOCAL_TYPE_NAME
    rowdata.addElement(new Short((short)-308));               // MINIMUM_SCALE
    rowdata.addElement(new Short((short)308));                // MAXIMUM_SCALE
    rowdata.addElement(new Integer(-1));
    rowdata.addElement(new Integer(-1));
    rowdata.addElement(new Integer(10));                      // NUM_PREC_RADIX
    rows.addElement(rowdata);

    //////////////////////////////////////////////////////////////////////
    // mysql : FIELD_TYPE_DATE
    // JDBC  : Types.DATE
    rowdata = new Vector();
    rowdata.addElement("DATE");                               // TYPE_NAME
    rowdata.addElement(new Short((short)Types.DATE));         // DATA_TYPE
    rowdata.addElement(new Integer(0));                       // PRECISION
    rowdata.addElement("'");                                  // LITERAL_PREFIX
    rowdata.addElement("'");                                  // LITERAL_SUFFIX
    rowdata.addElement("");                                   // CREATE_PARAMS
    rowdata.addElement(new Short((short)DatabaseMetaData.typeNullable)); // NULLABLE
    rowdata.addElement("false");                              // CASE_SENSITIVE
    rowdata.addElement(new Short((short)DatabaseMetaData.typeSearchable)); // SEARCHABLE
    rowdata.addElement("false");                              // UNSIGNED_ATTRIBUTE
    rowdata.addElement("false");                              // FIXED_PREC_SCALE
    rowdata.addElement("false");                              // AUTO_INCREMENT
    rowdata.addElement("DATE");                               // LOCAL_TYPE_NAME
    rowdata.addElement(new Short((short)0));                  // MINIMUM_SCALE
    rowdata.addElement(new Short((short)0));                  // MAXIMUM_SCALE
    rowdata.addElement(new Integer(-1));
    rowdata.addElement(new Integer(-1));
    rowdata.addElement(new Integer(10));                      // NUM_PREC_RADIX
    rows.addElement(rowdata);

    //////////////////////////////////////////////////////////////////////
    // mysql : FIELD_TYPE_DATETIME
    // JDBC  : Types.TIMESTAMP
    rowdata = new Vector();
    rowdata.addElement("DATETIME");                           // TYPE_NAME
    rowdata.addElement(new Short((short)Types.TIMESTAMP));    // DATA_TYPE
    rowdata.addElement(new Integer(0));                       // PRECISION
    rowdata.addElement("'");                                  // LITERAL_PREFIX
    rowdata.addElement("'");                                  // LITERAL_SUFFIX
    rowdata.addElement("");                                   // CREATE_PARAMS
    rowdata.addElement(new Short((short)DatabaseMetaData.typeNullable)); // NULLABLE
    rowdata.addElement("false");                              // CASE_SENSITIVE
    rowdata.addElement(new Short((short)DatabaseMetaData.typeSearchable)); // SEARCHABLE
    rowdata.addElement("false");                              // UNSIGNED_ATTRIBUTE
    rowdata.addElement("false");                              // FIXED_PREC_SCALE
    rowdata.addElement("false");                              // AUTO_INCREMENT
    rowdata.addElement("DATETIME");                           // LOCAL_TYPE_NAME
    rowdata.addElement(new Short((short)0));                  // MINIMUM_SCALE
    rowdata.addElement(new Short((short)0));                  // MAXIMUM_SCALE
    rowdata.addElement(new Integer(-1));
    rowdata.addElement(new Integer(-1));
    rowdata.addElement(new Integer(10));                      // NUM_PREC_RADIX
    rows.addElement(rowdata);

    //////////////////////////////////////////////////////////////////////
    // mysql : FIELD_TYPE_TIMESTAMP
    // JDBC  : Types.TIMESTAMP
    rowdata = new Vector();
    rowdata.addElement("TIMESTAMP");                          // TYPE_NAME
    rowdata.addElement(new Short((short)Types.TIMESTAMP));    // DATA_TYPE
    rowdata.addElement(new Integer(0));                       // PRECISION
    rowdata.addElement("'");                                  // LITERAL_PREFIX
    rowdata.addElement("'");                                  // LITERAL_SUFFIX
    rowdata.addElement("(M)");                                // CREATE_PARAMS
    rowdata.addElement(new Short((short)DatabaseMetaData.typeNullable)); // NULLABLE
    rowdata.addElement("false");                              // CASE_SENSITIVE
    rowdata.addElement(new Short((short)DatabaseMetaData.typeSearchable)); // SEARCHABLE
    rowdata.addElement("false");                              // UNSIGNED_ATTRIBUTE
    rowdata.addElement("false");                              // FIXED_PREC_SCALE
    rowdata.addElement("false");                              // AUTO_INCREMENT
    rowdata.addElement("TIMESTAMP");                          // LOCAL_TYPE_NAME
    rowdata.addElement(new Short((short)0));                  // MINIMUM_SCALE
    rowdata.addElement(new Short((short)0));                  // MAXIMUM_SCALE
    rowdata.addElement(new Integer(-1));
    rowdata.addElement(new Integer(-1));
    rowdata.addElement(new Integer(10));                      // NUM_PREC_RADIX
    rows.addElement(rowdata);

    //////////////////////////////////////////////////////////////////////
    // mysql : FIELD_TYPE_TIME
    // JDBC  : Types.TIME
    rowdata = new Vector();
    rowdata.addElement("TIME");                               // TYPE_NAME
    rowdata.addElement(new Short((short)Types.TIME));         // DATA_TYPE
    rowdata.addElement(new Integer(0));                       // PRECISION
    rowdata.addElement("'");                                  // LITERAL_PREFIX
    rowdata.addElement("'");                                  // LITERAL_SUFFIX
    rowdata.addElement("");                                   // CREATE_PARAMS
    rowdata.addElement(new Short((short)DatabaseMetaData.typeNullable)); // NULLABLE
    rowdata.addElement("false");                              // CASE_SENSITIVE
    rowdata.addElement(new Short((short)DatabaseMetaData.typeSearchable)); // SEARCHABLE
    rowdata.addElement("false");                              // UNSIGNED_ATTRIBUTE
    rowdata.addElement("false");                              // FIXED_PREC_SCALE
    rowdata.addElement("false");                              // AUTO_INCREMENT
    rowdata.addElement("TIME");                               // LOCAL_TYPE_NAME
    rowdata.addElement(new Short((short)0));                  // MINIMUM_SCALE
    rowdata.addElement(new Short((short)0));                  // MAXIMUM_SCALE
    rowdata.addElement(new Integer(-1));
    rowdata.addElement(new Integer(-1));
    rowdata.addElement(new Integer(10));                      // NUM_PREC_RADIX
    rows.addElement(rowdata);

    //////////////////////////////////////////////////////////////////////
    // mysql : FIELD_TYPE_STRING
    // JDBC  : Types.CHAR
    rowdata = new Vector();
    rowdata.addElement("CHAR");                               // TYPE_NAME
    rowdata.addElement(new Short((short)Types.CHAR));         // DATA_TYPE
    rowdata.addElement(new Integer(255));                     // PRECISION
    rowdata.addElement("'");                                  // LITERAL_PREFIX
    rowdata.addElement("'");                                  // LITERAL_SUFFIX
    rowdata.addElement("(M) [BINARY]");                       // CREATE_PARAMS
    rowdata.addElement(new Short((short)DatabaseMetaData.typeNullable)); // NULLABLE
    rowdata.addElement("false");                              // CASE_SENSITIVE
    rowdata.addElement(new Short((short)DatabaseMetaData.typeSearchable)); // SEARCHABLE
    rowdata.addElement("false");                              // UNSIGNED_ATTRIBUTE
    rowdata.addElement("false");                              // FIXED_PREC_SCALE
    rowdata.addElement("false");                              // AUTO_INCREMENT
    rowdata.addElement("CHAR");                               // LOCAL_TYPE_NAME
    rowdata.addElement(new Short((short)0));                  // MINIMUM_SCALE
    rowdata.addElement(new Short((short)0));                  // MAXIMUM_SCALE
    rowdata.addElement(new Integer(-1));
    rowdata.addElement(new Integer(-1));
    rowdata.addElement(new Integer(10));                      // NUM_PREC_RADIX
    rows.addElement(rowdata);

    //////////////////////////////////////////////////////////////////////
    // mysql : FIELD_TYPE_VAR_STRING
    // JDBC  : Types.VARCHAR
    rowdata = new Vector();
    rowdata.addElement("VARCHAR");                            // TYPE_NAME
    rowdata.addElement(new Short((short)Types.VARCHAR));      // DATA_TYPE
    rowdata.addElement(new Integer(255));                     // PRECISION
    rowdata.addElement("'");                                  // LITERAL_PREFIX
    rowdata.addElement("'");                                  // LITERAL_SUFFIX
    rowdata.addElement("(M) [BINARY]");                       // CREATE_PARAMS
    rowdata.addElement(new Short((short)DatabaseMetaData.typeNullable)); // NULLABLE
    rowdata.addElement("false");                              // CASE_SENSITIVE
    rowdata.addElement(new Short((short)DatabaseMetaData.typeSearchable)); // SEARCHABLE
    rowdata.addElement("false");                              // UNSIGNED_ATTRIBUTE
    rowdata.addElement("false");                              // FIXED_PREC_SCALE
    rowdata.addElement("false");                              // AUTO_INCREMENT
    rowdata.addElement("VARCHAR");                            // LOCAL_TYPE_NAME
    rowdata.addElement(new Short((short)0));                  // MINIMUM_SCALE
    rowdata.addElement(new Short((short)0));                  // MAXIMUM_SCALE
    rowdata.addElement(new Integer(-1));
    rowdata.addElement(new Integer(-1));
    rowdata.addElement(new Integer(10));                      // NUM_PREC_RADIX
    rows.addElement(rowdata);

    //////////////////////////////////////////////////////////////////////
    // mysql : FIELD_TYPE_TINY_BLOB
    // JDBC  : Types.VARBINARY
    rowdata = new Vector();
    rowdata.addElement("TINYBLOB");                           // TYPE_NAME
    rowdata.addElement(new Short((short)Types.VARBINARY));    // DATA_TYPE
    rowdata.addElement(new Integer(255));                     // PRECISION
    rowdata.addElement("'");                                  // LITERAL_PREFIX
    rowdata.addElement("'");                                  // LITERAL_SUFFIX
    rowdata.addElement("");                                   // CREATE_PARAMS
    rowdata.addElement(new Short((short)DatabaseMetaData.typeNullable)); // NULLABLE
    rowdata.addElement("true");                               // CASE_SENSITIVE
    rowdata.addElement(new Short((short)DatabaseMetaData.typeSearchable)); // SEARCHABLE
    rowdata.addElement("false");                              // UNSIGNED_ATTRIBUTE
    rowdata.addElement("false");                              // FIXED_PREC_SCALE
    rowdata.addElement("false");                              // AUTO_INCREMENT
    rowdata.addElement("TINYBLOB");                           // LOCAL_TYPE_NAME
    rowdata.addElement(new Short((short)0));                  // MINIMUM_SCALE
    rowdata.addElement(new Short((short)0));                  // MAXIMUM_SCALE
    rowdata.addElement(new Integer(-1));
    rowdata.addElement(new Integer(-1));
    rowdata.addElement(new Integer(10));                      // NUM_PREC_RADIX
    rows.addElement(rowdata);

    //////////////////////////////////////////////////////////////////////
    // mysql : FIELD_TYPE_BLOB
    // JDBC  : Types.LONGVARBINARY
    rowdata = new Vector();
    rowdata.addElement("BLOB");                               // TYPE_NAME
    rowdata.addElement(new Short((short)Types.LONGVARBINARY));// DATA_TYPE
    rowdata.addElement(new Integer(65535));                   // PRECISION
    rowdata.addElement("'");                                  // LITERAL_PREFIX
    rowdata.addElement("'");                                  // LITERAL_SUFFIX
    rowdata.addElement("");                                   // CREATE_PARAMS
    rowdata.addElement(new Short((short)DatabaseMetaData.typeNullable)); // NULLABLE
    rowdata.addElement("true");                               // CASE_SENSITIVE
    rowdata.addElement(new Short((short)DatabaseMetaData.typeSearchable)); // SEARCHABLE
    rowdata.addElement("false");                              // UNSIGNED_ATTRIBUTE
    rowdata.addElement("false");                              // FIXED_PREC_SCALE
    rowdata.addElement("false");                              // AUTO_INCREMENT
    rowdata.addElement("BLOB");                               // LOCAL_TYPE_NAME
    rowdata.addElement(new Short((short)0));                  // MINIMUM_SCALE
    rowdata.addElement(new Short((short)0));                  // MAXIMUM_SCALE
    rowdata.addElement(new Integer(-1));
    rowdata.addElement(new Integer(-1));
    rowdata.addElement(new Integer(10));                      // NUM_PREC_RADIX
    rows.addElement(rowdata);

    //////////////////////////////////////////////////////////////////////
    // mysql : FIELD_TYPE_MEDIUM_BLOB
    // JDBC  : Types.LONGVARBINARY
    rowdata = new Vector();
    rowdata.addElement("MEDIUMBLOB");                         // TYPE_NAME
    rowdata.addElement(new Short((short)Types.LONGVARBINARY));// DATA_TYPE
    rowdata.addElement(new Integer(16777215));                // PRECISION
    rowdata.addElement("'");                                  // LITERAL_PREFIX
    rowdata.addElement("'");                                  // LITERAL_SUFFIX
    rowdata.addElement("");                                   // CREATE_PARAMS
    rowdata.addElement(new Short((short)DatabaseMetaData.typeNullable)); // NULLABLE
    rowdata.addElement("true");                               // CASE_SENSITIVE
    rowdata.addElement(new Short((short)DatabaseMetaData.typeSearchable)); // SEARCHABLE
    rowdata.addElement("false");                              // UNSIGNED_ATTRIBUTE
    rowdata.addElement("false");                              // FIXED_PREC_SCALE
    rowdata.addElement("false");                              // AUTO_INCREMENT
    rowdata.addElement("MEDIUMBLOB");                         // LOCAL_TYPE_NAME
    rowdata.addElement(new Short((short)0));                  // MINIMUM_SCALE
    rowdata.addElement(new Short((short)0));                  // MAXIMUM_SCALE
    rowdata.addElement(new Integer(-1));
    rowdata.addElement(new Integer(-1));
    rowdata.addElement(new Integer(10));                      // NUM_PREC_RADIX
    rows.addElement(rowdata);

    //////////////////////////////////////////////////////////////////////
    // mysql : FIELD_TYPE_LONG_BLOB
    // JDBC  : Types.LONGVARBINARY
    rowdata = new Vector();
    rowdata.addElement("LONGBLOB");                           // TYPE_NAME
    rowdata.addElement(new Short((short)Types.LONGVARBINARY));// DATA_TYPE
    rowdata.addElement(new Integer(0x7fffffff)); //4294967295 // PRECISION
    rowdata.addElement("'");                                  // LITERAL_PREFIX
    rowdata.addElement("'");                                  // LITERAL_SUFFIX
    rowdata.addElement("");                                   // CREATE_PARAMS
    rowdata.addElement(new Short((short)DatabaseMetaData.typeNullable)); // NULLABLE
    rowdata.addElement("true");                               // CASE_SENSITIVE
    rowdata.addElement(new Short((short)DatabaseMetaData.typeSearchable)); // SEARCHABLE
    rowdata.addElement("false");                              // UNSIGNED_ATTRIBUTE
    rowdata.addElement("false");                              // FIXED_PREC_SCALE
    rowdata.addElement("false");                              // AUTO_INCREMENT
    rowdata.addElement("LONGBLOB");                           // LOCAL_TYPE_NAME
    rowdata.addElement(new Short((short)0));                  // MINIMUM_SCALE
    rowdata.addElement(new Short((short)0));                  // MAXIMUM_SCALE
    rowdata.addElement(new Integer(-1));
    rowdata.addElement(new Integer(-1));
    rowdata.addElement(new Integer(10));                      // NUM_PREC_RADIX
    rows.addElement(rowdata);

// Thanks for INDEN-san
    //////////////////////////////////////////////////////////////////////
    // mysql : FIELD_TYPE_TINY_BLOB (TEXT)
    // JDBC  : Types.VARCHAR
    rowdata = new Vector();
    rowdata.addElement("TINYTEXT");                           // TYPE_NAME
    rowdata.addElement(new Short((short)Types.VARCHAR));      // DATA_TYPE
    rowdata.addElement(new Integer(255));                     // PRECISION
    rowdata.addElement("'");                                  // LITERAL_PREFIX
    rowdata.addElement("'");                                  // LITERAL_SUFFIX
    rowdata.addElement("");                                   // CREATE_PARAMS
    rowdata.addElement(new Short((short)DatabaseMetaData.typeNullable)); // NULLABLE
    rowdata.addElement("true");                               // CASE_SENSITIVE
    rowdata.addElement(new Short((short)DatabaseMetaData.typeSearchable)); // SEARCHABLE
    rowdata.addElement("false");                              // UNSIGNED_ATTRIBUTE
    rowdata.addElement("false");                              // FIXED_PREC_SCALE
    rowdata.addElement("false");                              // AUTO_INCREMENT
    rowdata.addElement("TINYTEXT");                           // LOCAL_TYPE_NAME
    rowdata.addElement(new Short((short)0));                  // MINIMUM_SCALE
    rowdata.addElement(new Short((short)0));                  // MAXIMUM_SCALE
    rowdata.addElement(new Integer(-1));
    rowdata.addElement(new Integer(-1));
    rowdata.addElement(new Integer(10));                      // NUM_PREC_RADIX
    rows.addElement(rowdata);

// Thanks for INDEN-san
    //////////////////////////////////////////////////////////////////////
    // mysql : FIELD_TYPE_BLOB (TEXT)
    // JDBC  : Types.LONGVARCHAR
    rowdata = new Vector();
    rowdata.addElement("TEXT");                               // TYPE_NAME
    rowdata.addElement(new Short((short)Types.LONGVARCHAR));  // DATA_TYPE
    rowdata.addElement(new Integer(65535));                   // PRECISION
    rowdata.addElement("'");                                  // LITERAL_PREFIX
    rowdata.addElement("'");                                  // LITERAL_SUFFIX
    rowdata.addElement("");                                   // CREATE_PARAMS
    rowdata.addElement(new Short((short)DatabaseMetaData.typeNullable)); // NULLABLE
    rowdata.addElement("true");                               // CASE_SENSITIVE
    rowdata.addElement(new Short((short)DatabaseMetaData.typeSearchable)); // SEARCHABLE
    rowdata.addElement("false");                              // UNSIGNED_ATTRIBUTE
    rowdata.addElement("false");                              // FIXED_PREC_SCALE
    rowdata.addElement("false");                              // AUTO_INCREMENT
    rowdata.addElement("TEXT");                               // LOCAL_TYPE_NAME
    rowdata.addElement(new Short((short)0));                  // MINIMUM_SCALE
    rowdata.addElement(new Short((short)0));                  // MAXIMUM_SCALE
    rowdata.addElement(new Integer(-1));
    rowdata.addElement(new Integer(-1));
    rowdata.addElement(new Integer(10));                      // NUM_PREC_RADIX
    rows.addElement(rowdata);

// Thanks for INDEN-san
    //////////////////////////////////////////////////////////////////////
    // mysql : FIELD_TYPE_MEDIUM_BLOB (TEXT)
    // JDBC  : Types.LONGVARCHAR
    rowdata = new Vector();
    rowdata.addElement("MEDIUMTEXT");                         // TYPE_NAME
    rowdata.addElement(new Short((short)Types.LONGVARCHAR));  // DATA_TYPE
    rowdata.addElement(new Integer(16777215));                // PRECISION
    rowdata.addElement("'");                                  // LITERAL_PREFIX
    rowdata.addElement("'");                                  // LITERAL_SUFFIX
    rowdata.addElement("");                                   // CREATE_PARAMS
    rowdata.addElement(new Short((short)DatabaseMetaData.typeNullable)); // NULLABLE
    rowdata.addElement("true");                               // CASE_SENSITIVE
    rowdata.addElement(new Short((short)DatabaseMetaData.typeSearchable)); // SEARCHABLE
    rowdata.addElement("false");                              // UNSIGNED_ATTRIBUTE
    rowdata.addElement("false");                              // FIXED_PREC_SCALE
    rowdata.addElement("false");                              // AUTO_INCREMENT
    rowdata.addElement("MEDIUMTEXT");                         // LOCAL_TYPE_NAME
    rowdata.addElement(new Short((short)0));                  // MINIMUM_SCALE
    rowdata.addElement(new Short((short)0));                  // MAXIMUM_SCALE
    rowdata.addElement(new Integer(-1));
    rowdata.addElement(new Integer(-1));
    rowdata.addElement(new Integer(10));                      // NUM_PREC_RADIX
    rows.addElement(rowdata);

// Thanks for INDEN-san
    //////////////////////////////////////////////////////////////////////
    // mysql : FIELD_TYPE_LONG_BLOB (TEXT)
    // JDBC  : Types.LONGVARCHAR
    rowdata = new Vector();
    rowdata.addElement("LONGTEXT");                           // TYPE_NAME
    rowdata.addElement(new Short((short)Types.LONGVARCHAR));  // DATA_TYPE
    rowdata.addElement(new Integer(0x7fffffff)); //4294967295 // PRECISION
    rowdata.addElement("'");                                  // LITERAL_PREFIX
    rowdata.addElement("'");                                  // LITERAL_SUFFIX
    rowdata.addElement("");                                   // CREATE_PARAMS
    rowdata.addElement(new Short((short)DatabaseMetaData.typeNullable)); // NULLABLE
    rowdata.addElement("true");                               // CASE_SENSITIVE
    rowdata.addElement(new Short((short)DatabaseMetaData.typeSearchable)); // SEARCHABLE
    rowdata.addElement("false");                              // UNSIGNED_ATTRIBUTE
    rowdata.addElement("false");                              // FIXED_PREC_SCALE
    rowdata.addElement("false");                              // AUTO_INCREMENT
    rowdata.addElement("LONGTEXT");                           // LOCAL_TYPE_NAME
    rowdata.addElement(new Short((short)0));                  // MINIMUM_SCALE
    rowdata.addElement(new Short((short)0));                  // MAXIMUM_SCALE
    rowdata.addElement(new Integer(-1));
    rowdata.addElement(new Integer(-1));
    rowdata.addElement(new Integer(10));                      // NUM_PREC_RADIX
    rows.addElement(rowdata);

// YEAR support!!
    //////////////////////////////////////////////////////////////////////
    // mysql : FIELD_TYPE_YEAR
    // JDBC  : Types.SMALLINT
    rowdata = new Vector();
    rowdata.addElement("YEAR");                               // TYPE_NAME
    rowdata.addElement(new Short((short)Types.SMALLINT));     // DATA_TYPE
    rowdata.addElement(new Integer(4));                       // PRECISION
    rowdata.addElement("");                                   // LITERAL_PREFIX
    rowdata.addElement("");                                   // LITERAL_SUFFIX
    rowdata.addElement("(M) UNSIGNED ZEROFILL");              // CREATE_PARAMS
    rowdata.addElement(new Short((short)DatabaseMetaData.typeNullable)); // NULLABLE
    rowdata.addElement("false");                              // CASE_SENSITIVE
    rowdata.addElement(new Short((short)DatabaseMetaData.typeSearchable)); // SEARCHABLE
    rowdata.addElement("true");                               // UNSIGNED_ATTRIBUTE
    rowdata.addElement("false");                              // FIXED_PREC_SCALE
    rowdata.addElement("false");                              // AUTO_INCREMENT
    rowdata.addElement("YEAR");                               // LOCAL_TYPE_NAME
    rowdata.addElement(new Short((short)0));                  // MINIMUM_SCALE
    rowdata.addElement(new Short((short)0));                  // MAXIMUM_SCALE
    rowdata.addElement(new Integer(-1));
    rowdata.addElement(new Integer(-1));
    rowdata.addElement(new Integer(10));                      // NUM_PREC_RADIX
    rows.addElement(rowdata);

    return new gweMysqlVectorResultSet(conn, cols, rows);
  }

  /**
   * Gets a description of a table's indices and statistics. They are
   * ordered by NON_UNIQUE, TYPE, INDEX_NAME, and ORDINAL_POSITION.
   *
   * <P>Each index column description has the following columns:
   *  <OL>
   *    <LI><B>TABLE_CAT</B> String => table catalog (may be null)
   *    <LI><B>TABLE_SCHEM</B> String => table schema (may be null)
   *    <LI><B>TABLE_NAME</B> String => table name
   *    <LI><B>NON_UNIQUE</B> boolean => Can index values be non-unique?
   *      false when TYPE is tableIndexStatistic
   *    <LI><B>INDEX_QUALIFIER</B> String => index catalog (may be null);
   *      null when TYPE is tableIndexStatistic
   *    <LI><B>INDEX_NAME</B> String => index name; null when TYPE is
   *      tableIndexStatistic
   *    <LI><B>TYPE</B> short => index type:
   *      <UL>
   *      <LI> tableIndexStatistic - this identifies table statistics that are
   *           returned in conjuction with a table's index descriptions
   *      <LI> tableIndexClustered - this is a clustered index
   *      <LI> tableIndexHashed - this is a hashed index
   *      <LI> tableIndexOther - this is some other style of index
   *      </UL>
   *    <LI><B>ORDINAL_POSITION</B> short => column sequence number
   *      within index; zero when TYPE is tableIndexStatistic
   *    <LI><B>COLUMN_NAME</B> String => column name; null when TYPE is
   *      tableIndexStatistic
   *    <LI><B>ASC_OR_DESC</B> String => column sort sequence, "A" => ascending,
   *      "D" => descending, may be null if sort sequence is not supported;
   *      null when TYPE is tableIndexStatistic
   *    <LI><B>CARDINALITY</B> int => When TYPE is tableIndexStatistic, then
   *      this is the number of rows in the table; otherwise, it is the
   *      number of unique values in the index.
   *    <LI><B>PAGES</B> int => When TYPE is  tableIndexStatisic then
   *      this is the number of pages used for the table, otherwise it
   *      is the number of pages used for the current index.
   *    <LI><B>FILTER_CONDITION</B> String => Filter condition, if any.
   *      (may be null)
   *  </OL>
   *
   * @param catalog a catalog name; "" retrieves those without a
   * catalog; null means drop catalog name from the selection criteria
   * @param schema a schema name; "" retrieves those without a schema
   * @param table a table name
   * @param unique when true, return only indices for unique values;
   *     when false, return indices regardless of whether unique or not
   * @param approximate when true, result is allowed to reflect approximate
   *     or out of data values; when false, results are requested to be
   *     accurate
   * @return <code>ResultSet</code> - each row is an index column description
   * @exception SQLException if a database access error occurs
   */
  public ResultSet getIndexInfo(String catalog, String schema, String table,
                                boolean unique, boolean approximate)
                                                             throws SQLException {
    gweMysqlColumn[] cols = new gweMysqlColumn[13];
    cols[0] = new gweMysqlColumn("", "TABLE_CAT", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[1] = new gweMysqlColumn("", "TABLE_SCHEM", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[2] = new gweMysqlColumn("", "TABLE_NAME", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[3] = new gweMysqlColumn("", "NON_UNIQUE", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[4] = new gweMysqlColumn("", "INDEX_QUALIFIER", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[5] = new gweMysqlColumn("", "INDEX_NAME", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[6] = new gweMysqlColumn("", "TYPE", -1,
                      gweMysqlMetaResult.FIELD_TYPE_SHORT, (short)-1, -1);
    cols[7] = new gweMysqlColumn("", "ORDINAL_POSITION", -1,
                      gweMysqlMetaResult.FIELD_TYPE_SHORT, (short)-1, -1);
    cols[8] = new gweMysqlColumn("", "COLUMN_NAME", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[9] = new gweMysqlColumn("", "ASC_OR_DESC", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[10]= new gweMysqlColumn("", "CARDINALITY", -1,
                      gweMysqlMetaResult.FIELD_TYPE_LONG, (short)-1, -1);
    cols[11]= new gweMysqlColumn("", "PAGES", -1,
                      gweMysqlMetaResult.FIELD_TYPE_LONG, (short)-1, -1);
    cols[12]= new gweMysqlColumn("", "FILTER_CONDITION", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);

    Vector rows = new Vector();
    String temp_catalog;
    if( catalog != null && !catalog.equals("") )
      temp_catalog = catalog;
    else
      temp_catalog = conn.getCatalog();
    String sqlstr = "SHOW INDEX FROM " + temp_catalog + "." + table + ";";

    Statement stmt = null;
    ResultSet rs = null;
    try {
      stmt = conn.createStatement();
      rs = stmt.executeQuery(sqlstr);
      while(rs.next())
      {
        String  tablename   = rs.getString("Table");
        boolean non_unique  = rs.getBoolean("Non_unique");
        String  indexname   = rs.getString("Key_name");
        short   seq_index   = rs.getShort("Seq_in_index");
        String  columnname  = rs.getString("Column_name");
        String  collation   = rs.getString("Collation");
        int     cardinality = rs.getInt("Cardinality");

        if( unique == true && non_unique == true )
          continue;

        Vector rowdata = new Vector();
        rowdata.addElement(temp_catalog);             // TABLE_CAT
        rowdata.addElement("");
        if( add_quote == true )                // TABLE_NAME
          rowdata.addElement("`" + tablename + "`");
        else
          rowdata.addElement(tablename);
        if( non_unique == true )               // NON_UNIQUE
          rowdata.addElement("true");
        else
          rowdata.addElement("false");
        rowdata.addElement("");                       // INDEX_QUALIFIER
        rowdata.addElement(indexname);                // INDEX_NAME
        rowdata.addElement(new Short((short)DatabaseMetaData.tableIndexOther)); // TYPE
        rowdata.addElement(new Short(seq_index));     // ORDINAL_POSITION
        if( add_quote == true )                // COLUMN_NAME
          rowdata.addElement("`" + columnname + "`");
        else
          rowdata.addElement(columnname);
        rowdata.addElement(collation);                // ASC_OR_DESC
        rowdata.addElement(new Integer(cardinality)); // CARDINALITY
        rowdata.addElement(new Integer(0));           // PAGES
        rowdata.addElement("");                       // FILTER_CONDITION
        rows.addElement(rowdata);
      }
    }
    finally {
      if( rs != null )
      {
        try { rs.close(); }
        catch(Exception ex) { }
      }
      rs = null;
      if( stmt != null )
      {
        try { stmt.close(); }
        catch(Exception ex) { }
      }
      stmt = null;
    }
    return new gweMysqlVectorResultSet(conn, cols, rows);
  }

  //--------------------------JDBC 2.0-----------------------------

  /**
   * Does the database support the given result set type?
   *
   * @param type defined in java.sql.ResultSet
   * @return true if so; false otherwise
   * @exception SQLException if a database access error occurs
   * @see Connection
   * @since 1.2
   */
  public boolean supportsResultSetType(int type) throws SQLException {
    boolean retval = false;
    if( type == gweMysqlResultSetCore.TYPE_FORWARD_ONLY       ||
        type == gweMysqlResultSetCore.TYPE_SCROLL_INSENSITIVE )
    {
      retval = true;
    }
    return retval;
  }

  /**
   * Does the database support the concurrency type in combination
   * with the given result set type?
   *
   * @param type defined in java.sql.ResultSet
   * @param concurrency type defined in java.sql.ResultSet
   * @return true if so; false otherwise
   * @exception SQLException if a database access error occurs
   * @see Connection
   * @since 1.2
   */
  public boolean supportsResultSetConcurrency(int type, int concurrency)
                                                       throws SQLException {
    boolean retval = false;
    if( type == gweMysqlResultSetCore.TYPE_FORWARD_ONLY      &&
        concurrency == gweMysqlResultSetCore.CONCUR_READ_ONLY )
    {
      retval = true;
    }
    else if( type == gweMysqlResultSetCore.TYPE_SCROLL_INSENSITIVE &&
             concurrency == gweMysqlResultSetCore.CONCUR_READ_ONLY )
    {
      retval = true;
    }
    return retval;
  }

  /**
   * Determine whether a result set's own changes visible.
   *
   * @param result set type, i.e. ResultSet.TYPE_XXX
   * @return true if changes are visible for the result set type
   * @exception SQLException if a database-access error occurs.
   */
  public boolean ownUpdatesAreVisible(int type) throws SQLException {
    return false;
  }
  public boolean ownDeletesAreVisible(int type) throws SQLException {
    return false;
  }
  public boolean ownInsertsAreVisible(int type) throws SQLException {
    return false;
  }

  /**
   * Determine whether a result set's own changes visible.
   *
   * @param result set type, i.e. ResultSet.TYPE_XXX
   * @return true if changes are visible for the result set type
   * @exception SQLException if a database-access error occurs.
   */
  public boolean othersUpdatesAreVisible(int type) throws SQLException {
    return false;
  }
  public boolean othersDeletesAreVisible(int type) throws SQLException {
    return false;
  }
  public boolean othersInsertsAreVisible(int type) throws SQLException {
    return false;
  }

  /**
   * Determine whether or not a visible row update can be detected by
   * calling ResultSet.rowUpdated().
   *
   * @param result set type, i.e. ResultSet.TYPE_XXX
   * @return true if changes are detected by the resultset type
   * @exception SQLException if a database-access error occurs.
   */
  public boolean updatesAreDetected(int type) throws SQLException {
    return false;
  }

  /**
   * Determine whether or not a visible row delete can be detected by
   * calling ResultSet.rowDeleted().  If deletesAreDetected()
   * returns false, then deleted rows are removed from the result set.
   *
   * @param result set type, i.e. ResultSet.TYPE_XXX
   * @return true if changes are detected by the resultset type
   * @exception SQLException if a database-access error occurs.
   */
  public boolean deletesAreDetected(int type) throws SQLException {
    return false;
  }

  /**
   * Determine whether or not a visible row insert can be detected
   * by calling ResultSet.rowInserted().
   *
   * @param result set type, i.e. ResultSet.TYPE_XXX
   * @return true if changes are detected by the resultset type
   * @exception SQLException if a database-access error occurs.
   */
  public boolean insertsAreDetected(int type) throws SQLException {
    return false;
  }

  /**
   * Indicates whether the driver supports batch updates.
   * @return true if the driver supports batch updates; false otherwise
   * @since 1.2
   */
  public boolean supportsBatchUpdates() throws SQLException {
    return true;
  }

  /**
   * Get a description of the user-defined types defined in a particular
   * schema.  Schema specific UDTs may have type JAVA_OBJECT, STRUCT,
   * or DISTINCT.
   *
   * <P>Only types matching the catalog, schema, type name and type
   * criteria are returned.  They are ordered by DATA_TYPE, TYPE_SCHEM
   * and TYPE_NAME.  The type name parameter may be a fully qualified
   * name.  In this case, the catalog and schemaPattern parameters are
   * ignored.
   *
   * <P>Each type description has the following columns:
   *  <OL>
   *	<LI><B>TYPE_CAT</B> String => the type's catalog (may be null)
   *	<LI><B>TYPE_SCHEM</B> String => type's schema (may be null)
   *	<LI><B>TYPE_NAME</B> String => type name
   *  <LI><B>CLASS_NAME</B> String => Java class name
   *	<LI><B>DATA_TYPE</B> String => type value defined in java.sql.Types.
   *  One of JAVA_OBJECT, STRUCT, or DISTINCT
   *    <LI><B>REMARKS</B> String => explanatory comment on the type
   *  </OL>
   *
   * <P><B>Note:</B> If the driver does not support UDTs then an empty
   * result set is returned.
   *
   * @param catalog a catalog name; "" retrieves those without a
   * catalog; null means drop catalog name from the selection criteria
   * @param schemaPattern a schema name pattern; "" retrieves those
   * without a schema
   * @param typeNamePattern a type name pattern; may be a fully qualified
   * name
   * @param types a list of user-named types to include (JAVA_OBJECT,
   * STRUCT, or DISTINCT); null returns all types
   * @return ResultSet - each row is a type description
   * @exception SQLException if a database-access error occurs.
   */
  public ResultSet getUDTs(String catalog, String schemaPattern,
                   String typeNamePattern, int[] types) throws SQLException {
    throw new SQLException("mysql does not support getUDTs(String, String, String, int[]).");
  }

  /**
   * Return the connection that produced this metadata object.
   */
  public Connection getConnection() throws SQLException {
    return conn;
  }
}