//////////////////////////////////////////////////////////////////
// mysql JDBC Driver
//
// Version : 0.9.2
//
// JDBC : 1.22
//
// LastModified: 16.03.1997
//
// Copyright (c) 1997 GWE Technologies Ltd. All rights reserved
// See http://gwe.co.uk/mysql/jdbc/copyright.html for full details

package gwe.sql;

import java.net.*;
import java.io.*;
import java.util.Hashtable;
import gwe.sql.gweMysqlColumn;
import gwe.sql.gweMysqlException;

// Class to read/write packets to the mysql server

public class gweMysqlPacket {
  DataInputStream sin;
  DataOutputStream sout;
  byte packetSequence;
  int bpos_w, bpos_r, packetLength_r;
  public boolean TRACING=false;
  public final int HEADERLENGTH=4;
  public final int MAXBUFFER=65535;
  byte buffer_w[]=new byte[MAXBUFFER];
  byte buffer_r[]=new byte[MAXBUFFER];
  String encoding;
  gweNativeUnicodeMapper mapperclass;

  // Set up given an input and output datastream
  public gweMysqlPacket(DataInputStream i, DataOutputStream o, String enc, gweNativeUnicodeMapper mapper) {
    sin=i;
    sout=o;
    encoding = enc;
    mapperclass = mapper;
  }

  // Set to -1; packetSequence gets ++'ed before sending
  void resetPacketSequence() {
    packetSequence=-1;
  }

  // Read in next packet; get upset if need be
  void next() throws gweMysqlException {
    try {
      int i;
      byte[] b3 = new byte[3];

      // [3: packetLength] [packetSequence]
      for (i=0; i<3; i++) b3[i]=sin.readByte();
      packetLength_r=(int) (ub(b3[0])+(256*ub(b3[1]))+(256*256*ub(b3[2])));
      packetSequence=sin.readByte();

      if (TRACING) {
        System.out.println("Packet length = "+packetLength_r);
        System.out.println("Packet sequence = "+packetSequence);
      }

      // Get the data (block if necessary - thread higher up the us)
      if( buffer_r.length < packetLength_r+1 ) // 1=terminator(0)
      {
        buffer_r = new byte[packetLength_r+1]; // 1=terminator(0)
      }
      sin.readFully(buffer_r, 0, packetLength_r);
      buffer_r[packetLength_r]=0;
      bpos_r=0;

      if (TRACING) {
        for (int j=0; j<packetLength_r; j++) {
          System.out.print(j+" : "+ub(buffer_r[j])+"  : ");
          if (buffer_r[j]>30) System.out.println((char)buffer_r[j]);
          else System.out.println("");
        }

      }
    } catch (Exception e) {throw new gweMysqlException("gweMysql.Next: "+e);}
  }


  // Return unsigned byte as int
  int ub(byte b) {
    return b<0?(int)(256+b):b;
  }

  // Where are we?
  int currentPos_r() {
    return bpos_r;
  }

  // Set position withtin packet data
  void setPos_r(int p) {
    bpos_r=p;
  }

  // Read byte at current position
  byte getCurDataPos_r() {
    return buffer_r[bpos_r];
  }

  byte readByte() {
    return buffer_r[bpos_r++];
  }

  byte[] readByteBlock(int l) {
    byte[] block = new byte[l];
    if( l > 0 )
    {
      System.arraycopy(buffer_r, bpos_r, block, 0, l);
      bpos_r += l;
    }
    return block;
  }

  int read2Bytes() {
    return (int)(ub(buffer_r[bpos_r++])+(256*ub(buffer_r[bpos_r++])));
  }

  int read3Bytes() {
    return (int)(ub(buffer_r[bpos_r++])+(256*ub(buffer_r[bpos_r++]))+(256*256*ub(buffer_r[bpos_r++])));
  }

  long read4Bytes() {
    return (long)(ub(buffer_r[bpos_r++])+(256*ub(buffer_r[bpos_r++]))+(256*256*ub(buffer_r[bpos_r++]))+(256*256*256*ub(buffer_r[bpos_r++])));
  }


  // Read n bytes depending
  int readnBytes() {
    switch(ub(buffer_r[bpos_r++])) {
      case 1 : return ub(buffer_r[bpos_r++]);
      case 2 : return this.read2Bytes();
      case 3 : return this.read3Bytes();
      case 4 : return (int)this.read4Bytes();
      default : return 255;
    }
  }

  // read [:length]
  long readLength() {
    //System.out.println("flag:" + ub(buffer_r[bpos_r]));
    switch(ub(buffer_r[bpos_r])) {
      case 251 : {bpos_r++; return (long) -1; } // for NULL FIELD!!
      case 252 : {bpos_r++; return (long) read2Bytes();}
      case 253 : {bpos_r++; return (long) read3Bytes();}
      case 254 : {bpos_r++; return (long) read4Bytes();}
      default  : return (long) ub(buffer_r[bpos_r++]);
    }
  }


  // Read null-terminated string
  String readString() {
    StringBuffer b = new StringBuffer();
    int i=bpos_r;
    while(buffer_r[i]!=0) b.append((char)buffer_r[i++]);
    bpos_r=i+1;
    return b.toString();
  }

  // Read given-length string
  String readLenString() {
    long len = this.readLength();
    if (len==-1) return null; // for NULL FIELD!!
    if (len==0) return "";
    StringBuffer b = new StringBuffer((int)len);
    for (int i=0; i<len; i++) {
      if (buffer_r[bpos_r]==0) break;
      else b.append((char)buffer_r[bpos_r++]);
    }
    return b.toString();
  }

  // Is this the last packet?
  boolean isLastDataPacket_r() {
    return ((packetLength_r==1)&&(ub(buffer_r[0])==254));
  }

  // Reset position to start of packet data
  void clear_w() {
    bpos_w=HEADERLENGTH;
  }

  // Send packet
  void send() {
    try {
      int l=bpos_w;
      bpos_w=0;
      packetSequence++;
      this.write3Bytes(l-HEADERLENGTH);
      this.writeByte(packetSequence);

      if (TRACING) {
        for (int j=0; j<l; j++) {
          System.out.print(j+" : "+ub(buffer_w[j])+"  : ");
          if (buffer_w[j]>30) System.out.println((char)buffer_w[j]);
          else System.out.println("");
        }
      }

      sout.write(buffer_w, 0, l);
      sout.flush();
    } catch (Exception e) {System.out.println("send() Error : "+e);}
  }

  void writeByte(byte b) {
    buffer_w[bpos_w++]=b;
  }

  void write2Bytes(int i) {
    int r;
    r=i%256;
    buffer_w[bpos_w++]=(byte)r;
    i=i/256;
    buffer_w[bpos_w++]=(byte)i;
  }

  void write3Bytes(int i) {
    int r;
    r=i%256;
    buffer_w[bpos_w++]=(byte)r;
    i=i/256;
    r=i%256;
    buffer_w[bpos_w++]=(byte)r;
    i=i/256;
    buffer_w[bpos_w++]=(byte)i;
  }

  void write4Bytes(long i) {
    long r;
    r=i%256;
    buffer_w[bpos_w++]=(byte)r;
    i=i/256;
    r=i%256;
    buffer_w[bpos_w++]=(byte)r;
    i=i/256;
    r=i%256;
    buffer_w[bpos_w++]=(byte)r;
    i=i/256;
    buffer_w[bpos_w++]=(byte)i;
  }

  // Write null-terminated string
  void writeString(String s) {
    int l = s.length();
    for(int i=0; i<l; i++) buffer_w[bpos_w++]=(byte)s.charAt(i);
    buffer_w[bpos_w++]=0;
  }

  // Write string, with no termination
  void writeStringNoNull(String s) {
//    int l = s.length();
//    for(int i=0; i<l; i++) buffer[bpos++]=(byte)s.charAt(i);
    try{
      byte[] temp_bytes;
      if( mapperclass != null )
      {
        char[] cbuf = new char[s.length()];
        s.getChars(0, s.length(), cbuf, 0);
        temp_bytes = mapperclass.Unicode2Native(cbuf);
      }
      else
      {
        temp_bytes = s.getBytes(encoding);
      }
      int l = temp_bytes.length;
      System.arraycopy(temp_bytes, 0, buffer_w, bpos_w, l);
      bpos_w += l;
    }
    catch(UnsupportedEncodingException e){
      int l = s.length();
      for(int i=0; i<l; i++) buffer_w[bpos_w++]=(byte)s.charAt(i);
    }
  }

  // ---------------------------------------------------------------------

  // Send packet
  void sendHeader(int sql_length) throws gweMysqlException {
    bpos_w = 0;
    packetSequence = 0;
    this.write3Bytes(1 + sql_length); // 1=(byte)gweMysql.QUERY
    this.writeByte(packetSequence);
    this.writeByte((byte)gweMysql.QUERY);
    try {
      sout.write(buffer_w, 0, HEADERLENGTH + 1); // 1=(byte)gweMysql.QUERY
      sout.flush();
      bpos_w = 0;
    }
    catch (IOException io_e) {
      throw new gweMysqlException("gweMysqlPacket.sendHeader: "+io_e);
    }
  }

  // Write string, with no termination and Send Buffer
  void writeStringNoNullandSendBuffer(String s) throws gweMysqlException {
    try{
      byte[] temp_bytes;
      if( mapperclass != null )
      {
        char[] cbuf = new char[s.length()];
        s.getChars(0, s.length(), cbuf, 0);
        temp_bytes = mapperclass.Unicode2Native(cbuf);
      }
      else
      {
        temp_bytes = s.getBytes(encoding);
      }
      int l = temp_bytes.length;
      int w = buffer_w.length;
      for(int i=0; i<l; i++)
      {
        buffer_w[bpos_w++] = temp_bytes[i];
        if( bpos_w == w )
        {
          flushSendBuffer();
        }
      }
    }
    catch(UnsupportedEncodingException e){
      int l = s.length();
      int w = buffer_w.length;
      for(int i=0; i<l; i++)
      {
        buffer_w[bpos_w++] = (byte)s.charAt(i);
        if( bpos_w == w )
        {
          flushSendBuffer();
        }
      }
    }
  }

  // Write string, with no termination and Send Buffer
  void writeStringNoNullandSendBufferEx(String s) throws gweMysqlException {
    int l = s.length();
    int w = buffer_w.length;
    for(int i=0; i<l; i++)
    {
      buffer_w[bpos_w++] = (byte)s.charAt(i);
      if( bpos_w == w )
      {
        flushSendBuffer();
      }
    }
  }

  // Flush Send Buffer
  void flushSendBuffer() throws gweMysqlException {
    try {
      sout.write(buffer_w, 0, bpos_w);
      sout.flush();
      bpos_w = 0;
    }
    catch(IOException io_e) {
      throw new gweMysqlException("gweMysqlPacket.writeStringNoNullandSendBuffer: "+io_e);
    }
  }
}
