//////////////////////////////////////////////////////////////////
// mysql JDBC Driver
//
// Version : 0.9.2
//
// JDBC : 1.22
//
// LastModified: 12.03.1997
//
// Copyright (c) 1997 GWE Technologies Ltd. All rights reserved
// See http://gwe.co.uk/mysql/jdbc/copyright.html for full details

package gwe.sql;

import java.sql.*;
import java.io.IOException;
import java.math.BigDecimal;
import java.io.InputStream;
import java.util.StringTokenizer;
import gwe.sql.gweMysql;
import gwe.sql.gweMysqlException;
import java.io.Reader;
import java.util.Calendar;

public class gweMysqlPreparedStatement extends gweMysqlStatement implements PreparedStatement {
  /**
   * A PreparedStatement
   */
  private String prepared_sql = null;
  /**
   * A Parameter Object Array
   */
  private Object[] param_obj_array = null;
  /**
   * A Parameter Type Array
   */
  private int[] param_type_array = null;
  /**
   * characterEncoding
   */
  private String encoding;
  /**
   * nativeUnicodeMapper
   */
  private gweNativeUnicodeMapper mapperclass;
  /**
   * Add 0x5C(\) after 0x5C(\) when you execute PreparedStatement.setString
   */
  private boolean add5CAfter5C;
  /**
   * Trace SQL Statement with LogStream
   */
  private boolean debugSQL;

  static private final String[] hex_table = {
    "00","01","02","03","04","05","06","07","08","09","0a","0b","0c","0d","0e","0f",
    "10","11","12","13","14","15","16","17","18","19","1a","1b","1c","1d","1e","1f",
    "20","21","22","23","24","25","26","27","28","29","2a","2b","2c","2d","2e","2f",
    "30","31","32","33","34","35","36","37","38","39","3a","3b","3c","3d","3e","3f",
    "40","41","42","43","44","45","46","47","48","49","4a","4b","4c","4d","4e","4f",
    "50","51","52","53","54","55","56","57","58","59","5a","5b","5c","5d","5e","5f",
    "60","61","62","63","64","65","66","67","68","69","6a","6b","6c","6d","6e","6f",
    "70","71","72","73","74","75","76","77","78","79","7a","7b","7c","7d","7e","7f",
    "80","81","82","83","84","85","86","87","88","89","8a","8b","8c","8d","8e","8f",
    "90","91","92","93","94","95","96","97","98","99","9a","9b","9c","9d","9e","9f",
    "a0","a1","a2","a3","a4","a5","a6","a7","a8","a9","aa","ab","ac","ad","ae","af",
    "b0","b1","b2","b3","b4","b5","b6","b7","b8","b9","ba","bb","bc","bd","be","bf",
    "c0","c1","c2","c3","c4","c5","c6","c7","c8","c9","ca","cb","cc","cd","ce","cf",
    "d0","d1","d2","d3","d4","d5","d6","d7","d8","d9","da","db","dc","dd","de","df",
    "e0","e1","e2","e3","e4","e5","e6","e7","e8","e9","ea","eb","ec","ed","ee","ef",
    "f0","f1","f2","f3","f4","f5","f6","f7","f8","f9","fa","fb","fc","fd","fe","ff",
  };

  /**
   * Constructs a new gweMysqlStatement object.
   * @param conn the gweMysqlConnection object
   * @param sql the PreparedStatement object
   * @param rs_type the result set type
   * @param rs_concur the result set concurrency
   */
  public gweMysqlPreparedStatement(gweMysqlConnection conn, String sql,
                                   int rs_type, int rs_concur,
                                   String enc, gweNativeUnicodeMapper mapper,
                                   boolean add5C_mode) {
    super(conn, rs_type, rs_concur);
    prepared_sql = sql;
    encoding = enc;
    mapperclass = mapper;
    add5CAfter5C = add5C_mode;
    debugSQL = conn.isDebugSQL();

    int param_count = 0;
    StringTokenizer t_str = new StringTokenizer(prepared_sql, "?", true);
    while(t_str.hasMoreTokens())
    {
      String token = t_str.nextToken();
      if( token.equals("?") )
        param_count++;
    }
    if( param_count > 0 )
    {
      param_obj_array  = new Object[param_count];
      param_type_array = new int[param_count];
      int i;
      for(i = 0; i < param_obj_array.length; i++)
        param_obj_array[0]  = null;
      for(i = 0; i < param_type_array.length; i++)
        param_type_array[0] = Types.NULL;
    }
  }

  private String completeSQLString(){
    StringBuffer sql = new StringBuffer(1024);
    sql.append("");
    int param_index = -1;
    StringTokenizer t_str = new StringTokenizer(prepared_sql, "?", true);
    while(t_str.hasMoreTokens())
    {
      String token = t_str.nextToken();
      if( token.equals("?") )
      {
        param_index++;
        Object obj = param_obj_array[param_index];
        if( obj == null )
        {
          sql = sql.append("NULL");
        }
        else
        {
          switch(param_type_array[param_index])
          {
            case Types.TINYINT:
              //sql = sql.append(((Byte)obj).toString());
              sql = sql.append(obj.toString());
              break;
            case Types.SMALLINT:
              //sql = sql.append(((Short)obj).toString());
              sql = sql.append(obj.toString());
              break;
            case Types.INTEGER:
              //sql = sql.append(((Integer)obj).toString());
              sql = sql.append(obj.toString());
              break;
            case Types.BIGINT:
              //sql = sql.append(((Long)obj).toString());
              sql = sql.append(obj.toString());
              break;
            case Types.FLOAT:
              sql = sql.append("'");
              //sql = sql.append(((Float)obj).toString());
              sql = sql.append(obj.toString());
              sql = sql.append("'");
              break;
            case Types.DOUBLE:
              sql = sql.append("'");
              //sql = sql.append(((Double)obj).toString());
              sql = sql.append(obj.toString());
              sql = sql.append("'");
              break;
            case Types.DECIMAL:
              sql = sql.append("'");
              //sql = sql.append(((BigDecimal)obj).toString());
              sql = sql.append(obj.toString());
              sql = sql.append("'");
              break;
            case Types.VARCHAR:
            case Types.LONGVARCHAR: // Thanks for INDEN-san
              sql = sql.append("'");
              sql = sql.append(convertQuoteAndBS((String)obj));
              sql = sql.append("'");
              break;
            case Types.DATE:
              // ex. '1999-02-14'
              sql = sql.append("'");
              //sql = sql.append(((Date)obj).toString());
              sql = sql.append(obj.toString());
              sql = sql.append("'");
              break;
            case Types.TIME:
              // ex. '12:34:56'
              sql = sql.append("'");
              //sql = sql.append(((Time)obj).toString());
              sql = sql.append(obj.toString());
              sql = sql.append("'");
              break;
            case Types.TIMESTAMP:
              {
              // ex. '1999-02-14 12:34:56'
              sql = sql.append("'");
              //String temp_timestamp = ((Timestamp)obj).toString();
              String temp_timestamp = obj.toString();
              int dot_idx = temp_timestamp.indexOf('.');
              if( dot_idx != -1 )
                sql = sql.append(temp_timestamp.substring(0, dot_idx));
              else
                sql = sql.append(temp_timestamp);
              sql = sql.append("'");
              }
              break;
            case Types.VARBINARY:
              {
              sql = sql.append("0x");
              byte[] buf = (byte[])obj;
              for(int i = 0; i < buf.length; i++)
              {
                int b = (int)buf[i];
                if( b < 0 )
                  b += 256;
                sql = sql.append(gweMysqlPreparedStatement.hex_table[b]);
              }
              }
              break;
            case Types.LONGVARBINARY:
              {
              gweMysqlPreparedObject param_obj = (gweMysqlPreparedObject)obj;
              InputStream in_stream = (InputStream)param_obj.getObj();
              byte[] buf = new byte[param_obj.getLength()];
              int read_len = -1;

              try{
                read_len = in_stream.read(buf);
              }
              catch(IOException io_e){
                ;
              }
              if( read_len > 0 )
              {
                sql = sql.append("0x");
                for(int i = 0; i < read_len; i++)
                {
                  int b = (int)buf[i];
                  if( b < 0 )
                    b += 256;
                  sql = sql.append(gweMysqlPreparedStatement.hex_table[b]);
                }
              }
              }
              break;
            default:
              break;
          }
        }
      }
      else
      {
        sql = sql.append(token);
      }
    }
    return (sql.toString());
  }

  private int getSQLStringLength(){
    int retval = 0;
    StringBuffer sql = new StringBuffer(1024);
    sql.append("");
    int param_index = -1;
    StringTokenizer t_str = new StringTokenizer(prepared_sql, "?", true);
    while(t_str.hasMoreTokens())
    {
      String token = t_str.nextToken();
      if( token.equals("?") )
      {
        param_index++;
        Object obj = param_obj_array[param_index];
        if( obj == null )
        {
          sql = sql.append("NULL");
        }
        else
        {
          switch(param_type_array[param_index])
          {
            case Types.TINYINT:
              //sql = sql.append(((Byte)obj).toString());
              sql = sql.append(obj.toString());
              break;
            case Types.SMALLINT:
              //sql = sql.append(((Short)obj).toString());
              sql = sql.append(obj.toString());
              break;
            case Types.INTEGER:
              //sql = sql.append(((Integer)obj).toString());
              sql = sql.append(obj.toString());
              break;
            case Types.BIGINT:
              //sql = sql.append(((Long)obj).toString());
              sql = sql.append(obj.toString());
              break;
            case Types.FLOAT:
              sql = sql.append("'");
              //sql = sql.append(((Float)obj).toString());
              sql = sql.append(obj.toString());
              sql = sql.append("'");
              break;
            case Types.DOUBLE:
              sql = sql.append("'");
              //sql = sql.append(((Double)obj).toString());
              sql = sql.append(obj.toString());
              sql = sql.append("'");
              break;
            case Types.DECIMAL:
              sql = sql.append("'");
              //sql = sql.append(((BigDecimal)obj).toString());
              sql = sql.append(obj.toString());
              sql = sql.append("'");
              break;
            case Types.VARCHAR:
            case Types.LONGVARCHAR: // Thanks for INDEN-san
              sql = sql.append("'");
              sql = sql.append(convertQuoteAndBS((String)obj));
              sql = sql.append("'");
              break;
            case Types.DATE:
              // ex. '1999-02-14'
              sql = sql.append("'");
              //sql = sql.append(((Date)obj).toString());
              sql = sql.append(obj.toString());
              sql = sql.append("'");
              break;
            case Types.TIME:
              // ex. '12:34:56'
              sql = sql.append("'");
              //sql = sql.append(((Time)obj).toString());
              sql = sql.append(obj.toString());
              sql = sql.append("'");
              break;
            case Types.TIMESTAMP:
              {
              // ex. '1999-02-14 12:34:56'
              sql = sql.append("'");
              //String temp_timestamp = ((Timestamp)obj).toString();
              String temp_timestamp = obj.toString();
              int dot_idx = temp_timestamp.indexOf('.');
              if( dot_idx != -1 )
                sql = sql.append(temp_timestamp.substring(0, dot_idx));
              else
                sql = sql.append(temp_timestamp);
              sql = sql.append("'");
              }
              break;
            case Types.VARBINARY:
              {
              sql = sql.append("0x");

              byte[] buf = (byte[])obj;
              retval += buf.length * 2;
              }
              break;
            case Types.LONGVARBINARY:
              {
              sql = sql.append("0x");

              gweMysqlPreparedObject param_obj = (gweMysqlPreparedObject)obj;
              retval += param_obj.getLength() * 2;
              }
              break;
            default:
              break;
          }
        }
      }
      else
      {
        sql = sql.append(token);
      }  
    }

    String temp_string = sql.toString();
    try{
      byte[] temp_bytes;
      if( mapperclass != null )
      {
        char[] cbuf = new char[temp_string.length()];
        temp_string.getChars(0, temp_string.length(), cbuf, 0);
        temp_bytes = mapperclass.Unicode2Native(cbuf);
      }
      else
      {
        temp_bytes = temp_string.getBytes(encoding);
      }
      retval += temp_bytes.length;
    }
    catch(java.io.UnsupportedEncodingException e){
      retval += temp_string.length();
    }
    return (retval);
  }

  public ResultSet executeQuery() throws java.sql.SQLException {
    return super.executeQuery(completeSQLString());
  }

  public int executeUpdate() throws java.sql.SQLException {
//  return super.executeUpdate(completeSQLString());

    boolean writelog = debugSQL && (DriverManager.getLogStream() != null);
    StringBuffer sqllog = null;

    try{
      gweMysqlConnection gwe_connection = (gweMysqlConnection)super.getConnection();

      // to avoid send new SQL statement before close old resultset!!
      gwe_connection.setCurResultSet(null);

      gweMysqlPacket packet = gwe_connection.getPacket();

      // Send query command and sql update
      packet.sendHeader(getSQLStringLength());

      if( writelog )
      {
        sqllog = new StringBuffer();
        Timestamp t = new Timestamp(System.currentTimeMillis());
        sqllog.append(t.toString() + ":");
      }

      int param_index = -1;
      StringTokenizer t_str = new StringTokenizer(prepared_sql, "?", true);
      while(t_str.hasMoreTokens())
      {
        String token = t_str.nextToken();
        if( token.equals("?") )
        {
          param_index++;
          Object obj = param_obj_array[param_index];
          if( obj == null )
          {
            packet.writeStringNoNullandSendBuffer("NULL");
            if( writelog ) sqllog.append("NULL");
          }
          else
          {
            switch(param_type_array[param_index])
            {
              case Types.TINYINT:
                //packet.writeStringNoNullandSendBuffer(((Byte)obj).toString());
                packet.writeStringNoNullandSendBuffer(obj.toString());
                if( writelog ) sqllog.append(obj.toString());
                break;
              case Types.SMALLINT:
                //packet.writeStringNoNullandSendBuffer(((Short)obj).toString());
                packet.writeStringNoNullandSendBuffer(obj.toString());
                if( writelog ) sqllog.append(obj.toString());
                break;
              case Types.INTEGER:
                //packet.writeStringNoNullandSendBuffer(((Integer)obj).toString());
                packet.writeStringNoNullandSendBuffer(obj.toString());
                if( writelog ) sqllog.append(obj.toString());
                break;
              case Types.BIGINT:
                //packet.writeStringNoNullandSendBuffer(((Long)obj).toString());
                packet.writeStringNoNullandSendBuffer(obj.toString());
                if( writelog ) sqllog.append(obj.toString());
                break;
              case Types.FLOAT:
                //packet.writeStringNoNullandSendBuffer("'" + ((Float)obj).toString() + "'");
                packet.writeStringNoNullandSendBuffer("'" + obj.toString() + "'");
                if( writelog ) sqllog.append("'"+obj.toString()+"'");
                break;
              case Types.DOUBLE:
                //packet.writeStringNoNullandSendBuffer("'" + ((Double)obj).toString() + "'");
                packet.writeStringNoNullandSendBuffer("'" + obj.toString() + "'");
                if( writelog ) sqllog.append("'"+obj.toString()+"'");
                break;
              case Types.DECIMAL:
                //packet.writeStringNoNullandSendBuffer("'" + ((BigDecimal)obj).toString() + "'");
                packet.writeStringNoNullandSendBuffer("'" + obj.toString() + "'");
                if( writelog ) sqllog.append("'"+obj.toString()+"'");
                break;
              case Types.VARCHAR:
              case Types.LONGVARCHAR: // Thanks for INDEN-san
                packet.writeStringNoNullandSendBuffer("'" + convertQuoteAndBS((String)obj) + "'");
                if( writelog ) sqllog.append("'"+convertQuoteAndBS((String)obj)+"'");
                break;
              case Types.DATE:
                // ex. '1999-02-14'
                //packet.writeStringNoNullandSendBuffer("'" + ((Date)obj).toString() + "'");
                packet.writeStringNoNullandSendBuffer("'" + obj.toString() + "'");
                if( writelog ) sqllog.append("'"+obj.toString()+"'");
                break;
              case Types.TIME:
                // ex. '12:34:56'
                //packet.writeStringNoNullandSendBuffer("'" + ((Time)obj).toString() + "'");
                packet.writeStringNoNullandSendBuffer("'" + obj.toString() + "'");
                if( writelog ) sqllog.append("'"+obj.toString()+"'");
                break;
              case Types.TIMESTAMP:
                {
                // ex. '1999-02-14 12:34:56'
                //String temp_timestamp = ((Timestamp)obj).toString();
                String temp_timestamp = obj.toString();
                int dot_idx = temp_timestamp.indexOf('.');
                if( dot_idx != -1 )
                  temp_timestamp = temp_timestamp.substring(0, dot_idx);
                packet.writeStringNoNullandSendBuffer("'" + temp_timestamp + "'");
                if( writelog ) sqllog.append("'"+temp_timestamp+"'");
                }
                break;
              case Types.VARBINARY:
                {
                packet.writeStringNoNullandSendBuffer("0x");
                if( writelog ) sqllog.append("0x");
                
                byte[] buf = (byte[])obj;
                for(int i = 0; i < buf.length; i++)
                {
                  int b = (int)buf[i];
                  if( b < 0 )
                    b += 256;   
                  packet.writeStringNoNullandSendBufferEx(gweMysqlPreparedStatement.hex_table[b]);
                  if( writelog ) sqllog.append(gweMysqlPreparedStatement.hex_table[b]);
                }
                }
                break;
              case Types.LONGVARBINARY:
                {
                packet.writeStringNoNullandSendBuffer("0x");
                if( writelog ) sqllog.append("0x");
                
                gweMysqlPreparedObject param_obj = (gweMysqlPreparedObject)obj;
                InputStream in_stream = (InputStream)param_obj.getObj();
                byte[] buf = new byte[4096];
                while(true)
                {
                  int read_len = -1;
                  try{
                    read_len = in_stream.read(buf, 0, buf.length);
                  }
                  catch(IOException io_e){
                    throw new SQLException("mysql exception: " + io_e.getMessage());
                  }
                  if( read_len < 1 )
                    break;
                  for(int i = 0; i < read_len; i++)
                  {
                    int b = (int)buf[i];
                    if( b < 0 )
                      b += 256;
                    packet.writeStringNoNullandSendBufferEx(gweMysqlPreparedStatement.hex_table[b]);
                    if( writelog ) sqllog.append(gweMysqlPreparedStatement.hex_table[b]);
                  }
                }
                }
                break;
              default:
                break;
            }
          }
        }
        else
        {
          packet.writeStringNoNullandSendBuffer(token);
          if( writelog ) sqllog.append(token);
        }
      }

      packet.flushSendBuffer();

      if( writelog )
        DriverManager.println(sqllog.toString());

      // check if we're ok
      packet.next();
      int updateStatus = packet.ub(packet.readByte());
      if (updateStatus==255) {
        String errorString = packet.readString();
        throw new SQLException("mysql exception: " + errorString);
      }
      long affectedRows = packet.readLength();
      long updateId = packet.readLength();

      return (int)affectedRows;
    }
    catch( gweMysqlException e ) {
      throw new SQLException("mysql exception: " + e.getMessage());
    }
  }

  public void setNull(int parameterIndex, int sqlType) throws java.sql.SQLException {
    this.setObject(parameterIndex, null, sqlType);
  }

  public void setBoolean(int parameterIndex, boolean x) throws java.sql.SQLException {
    throw new SQLException("mysql does not support setBoolean(int, boolean).");
  }

  public void setByte(int parameterIndex, byte x) throws java.sql.SQLException {
    Byte obj = new Byte(x);
    this.setObject(parameterIndex, obj, Types.TINYINT);
  }

  public void setShort(int parameterIndex, short x) throws java.sql.SQLException {
    Short obj = new Short(x);
    this.setObject(parameterIndex, obj, Types.SMALLINT);
  }

  public void setInt(int parameterIndex, int x) throws java.sql.SQLException {
    Integer obj = new Integer(x);
    this.setObject(parameterIndex, obj, Types.INTEGER);
  }

  public void setLong(int parameterIndex, long x) throws java.sql.SQLException {
    Long obj = new Long(x);
    this.setObject(parameterIndex, obj, Types.BIGINT);
  }

  public void setFloat(int parameterIndex, float x) throws java.sql.SQLException {
    Float obj = new Float(x);
    this.setObject(parameterIndex, obj, Types.FLOAT);
  }

  public void setDouble(int parameterIndex, double x) throws java.sql.SQLException {
    Double obj = new Double(x);
    this.setObject(parameterIndex, obj, Types.DOUBLE);
  }

  public void setBigDecimal(int parameterIndex, BigDecimal x) throws java.sql.SQLException {
    this.setObject(parameterIndex, x, Types.DECIMAL);
  }

  /**
   * Add 0x5C(\) after 0x5C(\) when you execute PreparedStatement.setString
   */
  private byte[] add5CAfter5CforShiftJIS(byte[] buf) {
    int buflen = buf.length;
    if( buflen == 0 )
      return buf;
    java.io.ByteArrayOutputStream bos = new java.io.ByteArrayOutputStream();
    int idx = 0;
    while(true)
    {
      int b1 = (int)buf[idx];
      if( b1 < 0 )
        b1 += 256;
      bos.write(b1);
      if( (b1 >= 0x80 && b1 <= 0x9F) || (b1 >= 0xE0 && b1 <= 0xFC) )
      {
        if( idx < buflen-1 )
        {
          int b2 = (int)buf[idx+1];
          if( b2 < 0 )
            b2 += 256;
          if( b2 == 0x5C )
            bos.write(b2);
        }
      }
      idx++;
      if( idx >= buflen )
        break;
    }
    return bos.toByteArray();
  }

  public void setString(int parameterIndex, String x) throws java.sql.SQLException {
    if( add5CAfter5C && x != null )
    {
      try {
        if( mapperclass != null )
        {
          char[] cbuf = new char[x.length()];
          x.getChars(0, x.length(), cbuf, 0);
          byte[] temp_bytes = mapperclass.Unicode2Native(cbuf);
          byte[] buf2 = add5CAfter5CforShiftJIS(temp_bytes);
          x = new String(mapperclass.Native2Unicode(buf2));
        }
        else
        {
          byte[] temp_bytes = x.getBytes(encoding);
          byte[] buf2 = add5CAfter5CforShiftJIS(temp_bytes);
          x = new String(buf2, 0, buf2.length, encoding);
        }
      }
      catch( java.io.UnsupportedEncodingException e ) {
        throw new SQLException("mysql exception: " + e.getMessage());
      }
    }
    this.setObject(parameterIndex, x, Types.VARCHAR);
  }

  public void setBytes(int parameterIndex, byte[] x) throws java.sql.SQLException {
    this.setObject(parameterIndex, x, Types.VARBINARY);
  }

  public void setDate(int parameterIndex, Date x) throws java.sql.SQLException {
    // ex. '1999-02-14'
    this.setObject(parameterIndex, x, Types.DATE);
  }

  public void setTime(int parameterIndex, Time x) throws java.sql.SQLException {
    // ex. '12:34:56'
    this.setObject(parameterIndex, x, Types.TIME);
  }

  public void setTimestamp(int parameterIndex, Timestamp x) throws java.sql.SQLException {
    // ex. '1999-02-14 12:34:56'
    this.setObject(parameterIndex, x, Types.TIMESTAMP);
  }

  public void setAsciiStream(int parameterIndex, InputStream x, int length) throws java.sql.SQLException {
    throw new SQLException("mysql does not support setAsciiStream(int, InputStream, int).");
  }

  public void setUnicodeStream(int parameterIndex, InputStream x, int length) throws java.sql.SQLException {
    throw new SQLException("mysql does not support setUnicodeStream(int, InputStream, int).");
  }

  public void setBinaryStream(int parameterIndex, InputStream x, int length) throws java.sql.SQLException {
    this.setObject(parameterIndex, new gweMysqlPreparedObject(x, length), Types.LONGVARBINARY);
  }

  public void clearParameters() throws java.sql.SQLException {
    if( param_obj_array  != null &&
        param_type_array != null )
    {
      int i;
      for(i = 0; i < param_obj_array.length; i++)
        param_obj_array[0]  = null;
      for(i = 0; i < param_type_array.length; i++)
        param_type_array[0] = Types.NULL;
    }
    else
    {
      throw new SQLException("mysql error : clearParameters(), no parameter.");
    }
  }

  public void setObject(int parameterIndex, Object x, int targetSqlType, int scale) throws java.sql.SQLException {
    this.setObject(parameterIndex, x, targetSqlType);
  }

  public void setObject(int parameterIndex, Object x, int targetSqlType) throws java.sql.SQLException {
    if( param_obj_array  != null &&
        param_type_array != null )
    {
      if( parameterIndex <= param_obj_array.length  &&
          parameterIndex <= param_type_array.length )
      {
        param_obj_array[parameterIndex-1]  = x;
        if( (x instanceof byte[]) && (targetSqlType != Types.VARBINARY) )
          targetSqlType = Types.VARBINARY;
        if( (x instanceof InputStream) && (targetSqlType != Types.LONGVARBINARY) )
          targetSqlType = Types.LONGVARBINARY;
        param_type_array[parameterIndex-1] = targetSqlType;
        return;
      }
    }
    throw new SQLException("gweMysqlPreparedStatement Error : setObject(int, Object, int).");
  }

  public void setObject(int parameterIndex, Object x) throws java.sql.SQLException {
    throw new SQLException("mysql does not support setObject(int, Object).");
  }

  public boolean execute() throws java.sql.SQLException {
    return super.execute(completeSQLString());
  }

  private String convertQuoteAndBS(String str) {
    StringBuffer retval = new StringBuffer(str.length());
    retval.append("");
    StringTokenizer t_str = new StringTokenizer(str, "'\\", true);
    while(t_str.hasMoreTokens())
    {
      String token = t_str.nextToken();
      if( token.equals("'") )
        retval = retval.append("''");
      else if( token.equals("\\") )
        retval = retval.append("\\\\");
      else
        retval = retval.append(token);
    }
    return (retval.toString());
  }

  public ResultSet executeQuery(String sql) throws SQLException {
    throw new SQLException("do not call this method : executeQuery(String)");
  }

  public int executeUpdate(String sql) throws SQLException {
    throw new SQLException("do not call this method : executeUpdate(String)");
  }

  public boolean execute(String sql) throws SQLException {
    throw new SQLException("do not call this method : execute(String)");
  }

  //--------------------------JDBC 2.0-----------------------------------

  public void addBatch() throws SQLException {
    throw new SQLException("mysql does not support addBatch().");
  }

  public void setCharacterStream(int parameterIndex, Reader reader, int length) throws SQLException {
    throw new SQLException("mysql does not support setCharacterStream(int, Reader, int).");
  }

/*
  public void setRef(int i, Ref x) throws SQLException {
    throw new SQLException("mysql does not support setRef(int, Ref).");
  }

  public void setBlob(int i, Blob x) throws SQLException {
    setBinaryStream(i, x.getBinaryStream(), (int)x.length());
  }

  public void setClob(int i, Clob x) throws SQLException {
    throw new SQLException("mysql does not support setClob(int, Clob).");
  }

  public void setArray(int i, Array x) throws SQLException {
    throw new SQLException("mysql does not support setArray(int, Array).");
  }
*/

  public ResultSetMetaData getMetaData() throws SQLException {
    ResultSet rs = getResultSet();
    return rs.getMetaData();
  }

  public void setDate(int parameterIndex, Date x, Calendar cal) throws SQLException {
    // ex. '1999-02-14'
    this.setObject(parameterIndex, x, Types.DATE);
  }

  public void setTime(int parameterIndex, Time x, Calendar cal) throws SQLException {
    // ex. '12:34:56'
    this.setObject(parameterIndex, x, Types.TIME);
  }

  public void setTimestamp(int parameterIndex, Timestamp x, Calendar cal) throws SQLException {
    // ex. '1999-02-14 12:34:56'
    this.setObject(parameterIndex, x, Types.TIMESTAMP);
  }

  public void setNull(int paramIndex, int sqlType, String typeName) throws SQLException {
    throw new SQLException("mysql does not support setNull(int, int, String).");
  }
}

class gweMysqlPreparedObject {
  private int    length;
  private Object obj;
  public gweMysqlPreparedObject(Object x, int len) {
    length = len;
    obj    = x;
  }

  public int getLength() {
    return length;
  }

  public Object getObj() {
    return obj;
  }
}

