//////////////////////////////////////////////////////////////////
// mysql JDBC Driver
//
// Version : 0.9.2
//
// JDBC : 1.22
//
// LastModified: 16.03.1997
//
// Copyright (c) 1997 GWE Technologies Ltd. All rights reserved
// See http://gwe.co.uk/mysql/jdbc/copyright.html for full details

package gwe.sql;

import java.math.BigDecimal;
import java.sql.Date;
import java.sql.Time;
import java.sql.Timestamp;
import java.io.InputStream;
import java.sql.SQLWarning;
import java.sql.ResultSetMetaData;
import java.io.Reader;
import java.sql.Statement;
//import java.util.Map;
//import java.sql.Ref;
//import java.sql.Blob;
//import java.sql.Clob;
//import java.sql.Array;
import java.util.Calendar;
import java.sql.ResultSet;
import java.util.Vector;
import java.sql.SQLException;

public class gweMysqlResultSetBuffer implements ResultSet {
  private gweMysqlResultSetCore org_rs;
  private Vector rows = new Vector();
  private int index = 0;
  private boolean last_null = false;

  public gweMysqlResultSetBuffer(gweMysqlResultSetCore org_rs) {
    this.org_rs = org_rs;

    try {
      try {
        int maxRows = getStatement().getMaxRows();
        int rowcount = 0;
        int colnum = org_rs.getMetaData().getColumnCount();
        while( org_rs.next() )
        {
          Vector vecrow = new Vector(colnum);
          for(int i = 1; i <= colnum; i++)
          {
            Object obj = org_rs.getObject(i);
            if( org_rs.wasNull() )
              obj = null;
            vecrow.addElement(obj);
          }
          rows.addElement(vecrow);
          rowcount++;
          if( maxRows != 0 && rowcount >= maxRows )
          {
            while( org_rs.next() )
              ;
            break;
          }
        }
      }
      finally {
        org_rs.close();
      }
    }
    catch(SQLException meta_e) {
      meta_e.printStackTrace();
      System.err.println("mysql can not get ResultSet and MetaData!! : " +
meta_e.getMessage());
    }
  }

  public boolean next() throws SQLException {
    int row_count = rows.size();
    if( index + 1 <= row_count )
    {
      index++;
      return true;
    }
    return false;
  }

  public void close() throws SQLException {
    org_rs = null;
    rows = null;
  }

  public boolean wasNull() throws SQLException {
    return last_null;
  }

  public String getString(int columnIndex) throws SQLException {
    Object retval = getObject(columnIndex);
    if( retval == null )
      return null;
    if( retval instanceof byte[] )
    {
      byte[] blob = (byte[])retval;
      try {
        gweNativeUnicodeMapper mapperclass = org_rs.getNativeUnicodeMapper();
        if( mapperclass != null )
          return new String(mapperclass.Native2Unicode(blob));
        else
          return new String(blob, 0, blob.length, org_rs.getEncoding());
      }
      catch(Exception enc_e){
        throw new SQLException(enc_e.getMessage());
      }
    }
    else if( retval instanceof java.lang.Character )
    {
      return String.valueOf((int)((java.lang.Character)retval).charValue());
    }
    return retval.toString();
  }

  public boolean getBoolean(int columnIndex) throws SQLException {
    String s = getString(columnIndex);
    try {
      return Boolean.valueOf(s).booleanValue();
    }
    catch(Exception e) {
//    throw new SQLException(e.getMessage());
      return false;
    }
  }

  public byte getByte(int columnIndex) throws SQLException {
    String s = getString(columnIndex);
    try {
      return Byte.valueOf(s).byteValue();
    }
    catch(Exception e) {
//    throw new SQLException(e.getMessage());
      return 0;
    }
  }

  public short getShort(int columnIndex) throws SQLException {
    String s = getString(columnIndex);
    try {
      return Short.valueOf(s).shortValue();
    }
    catch(Exception e) {
//    throw new SQLException(e.getMessage());
      return 0;
    }
  }

  public int getInt(int columnIndex) throws SQLException {
    String s = getString(columnIndex);
    try {
      return Integer.valueOf(s).intValue();
    }
    catch(Exception e) {
//    throw new SQLException(e.getMessage());
      return 0;
    }
  }

  public long getLong(int columnIndex) throws SQLException {
    String s = getString(columnIndex);
    try {
      return Long.valueOf(s).longValue();
    }
    catch(Exception e) {
//    throw new SQLException(e.getMessage());
      return 0;
    }
  }

  public float getFloat(int columnIndex) throws SQLException {
    String s = getString(columnIndex);
    try {
      return Float.valueOf(s).floatValue();
    }
    catch(Exception e) {
//    throw new SQLException(e.getMessage());
      return 0.0f;
    }
  }

  public double getDouble(int columnIndex) throws SQLException {
    String s = getString(columnIndex);
    try {
      return Double.valueOf(s).doubleValue();
    }
    catch(Exception e) {
//    throw new SQLException(e.getMessage());
      return 0.0;
    }
  }

  public BigDecimal getBigDecimal(int columnIndex, int scale) throws SQLException {
    return (BigDecimal)getObject(columnIndex);
  }

  public byte[] getBytes(int columnIndex) throws SQLException {
    return (byte[])getObject(columnIndex);
  }

  public Date getDate(int columnIndex) throws SQLException {
    return (Date)getObject(columnIndex);
  }

  public Time getTime(int columnIndex) throws SQLException {
    return (Time)getObject(columnIndex);
  }

  public Timestamp getTimestamp(int columnIndex) throws SQLException {
    return (Timestamp)getObject(columnIndex);
  }

  public InputStream getAsciiStream(int columnIndex) throws SQLException {
    throw new SQLException("mysql does not support getAsciiStream(int).");
  }

  public InputStream getUnicodeStream(int columnIndex) throws SQLException {
    throw new SQLException("mysql does not support getUnicodeStream(int).");
  }

  public InputStream getBinaryStream(int columnIndex) throws SQLException {
    byte[] bin_buf = getBytes(columnIndex);
    if( bin_buf == null )
      return null;
    return new java.io.ByteArrayInputStream(bin_buf);
  }

  public String getString(String columnName) throws SQLException {
    return getString(findColumn(columnName));
  }

  public boolean getBoolean(String columnName) throws SQLException {
    return getBoolean(findColumn(columnName));
  }

  public byte getByte(String columnName) throws SQLException {
    return getByte(findColumn(columnName));
  }

  public short getShort(String columnName) throws SQLException {
    return getShort(findColumn(columnName));
  }

  public int getInt(String columnName) throws SQLException {
    return getInt(findColumn(columnName));
  }

  public long getLong(String columnName) throws SQLException {
    return getLong(findColumn(columnName));
  }

  public float getFloat(String columnName) throws SQLException {
    return getFloat(findColumn(columnName));
  }

  public double getDouble(String columnName) throws SQLException {
    return getDouble(findColumn(columnName));
  }

  public BigDecimal getBigDecimal(String columnName, int scale) throws SQLException {
    return getBigDecimal(findColumn(columnName));
  }

  public byte[] getBytes(String columnName) throws SQLException {
    return getBytes(findColumn(columnName));
  }

  public Date getDate(String columnName) throws SQLException {
    return getDate(findColumn(columnName));
  }

  public Time getTime(String columnName) throws SQLException {
    return getTime(findColumn(columnName));
  }

  public Timestamp getTimestamp(String columnName) throws SQLException {
    return getTimestamp(findColumn(columnName));
  }

  public InputStream getAsciiStream(String columnName) throws SQLException {
    return getAsciiStream(findColumn(columnName));
  }

  public InputStream getUnicodeStream(String columnName) throws SQLException {
    return getUnicodeStream(findColumn(columnName));
  }

  public InputStream getBinaryStream(String columnName) throws SQLException {
    return getBinaryStream(findColumn(columnName));
  }

  public SQLWarning getWarnings() throws SQLException {
    return null;
  }

  public void clearWarnings() throws SQLException {
  }

  public String getCursorName() throws SQLException {
    return "";
  }

  public ResultSetMetaData getMetaData() throws SQLException {
    return org_rs.getMetaData();
  }

  public Object getObject(int columnIndex) throws SQLException {
    try {
      Vector rowdata = (Vector)rows.elementAt(index-1);
      Object data = rowdata.elementAt(columnIndex-1);
      last_null = data == null ? true : false;
      return data;
    }
    catch(Exception e) {
      return null;
    }
  }

  public Object getObject(String columnName) throws SQLException {
    return getObject(findColumn(columnName));
  }

  public int findColumn(String columnName) throws SQLException {
    return org_rs.findColumn(columnName);
  }

  public Reader getCharacterStream(int columnIndex) throws SQLException {
    return null;
  }

  public Reader getCharacterStream(String columnName) throws SQLException {
    return getCharacterStream(findColumn(columnName));
  }

  public BigDecimal getBigDecimal(int columnIndex) throws SQLException {
    return getBigDecimal(columnIndex, 10);
  }

  public BigDecimal getBigDecimal(String columnName) throws SQLException {
    return getBigDecimal(findColumn(columnName));
  }

  //---------------------------------------------------------------------
  // Traversal/Positioning
  //---------------------------------------------------------------------

  /**
   * Indicates whether the cursor is before the first row in
   * this <code>ResultSet</code> object.
   *
   * @return <code>true</code> if the cursor is before the first row;
   * <code>false</code> if the cursor is at any other position or the
   * result set contains no rows
   * @exception SQLException if a database access error occurs
   * @since 1.2
   * @see <a href="package-summary.html#2.0 API">What Is in the JDBC
   *      2.0 API</a>
   */
  public boolean isBeforeFirst() throws SQLException {
    int row_count = rows.size();
    if( row_count > 0 && index == 0 )
    {
      return true;
    }
    return false;
  }

  /**
   * Indicates whether the cursor is after the last row in
   * this <code>ResultSet</code> object.
   *
   * @return <code>true</code> if the cursor is after the last row;
   * <code>false</code> if the cursor is at any other position or the
   * result set contains no rows
   * @exception SQLException if a database access error occurs
   * @since 1.2
   * @see <a href="package-summary.html#2.0 API">What Is in the JDBC
   *      2.0 API</a>
   */
  public boolean isAfterLast() throws SQLException {
    int row_count = rows.size();
    if( row_count > 0 && index == row_count + 1 )
    {
      return true;
    }
    return false;
  }

  /**
   * Indicates whether the cursor is on the first row of
   * this <code>ResultSet</code> object.
   *
   * @return <code>true</code> if the cursor is on the first row;
   * <code>false</code> otherwise
   * @exception SQLException if a database access error occurs
   * @since 1.2
   * @see <a href="package-summary.html#2.0 API">What Is in the JDBC
   *      2.0 API</a>
   */
  public boolean isFirst() throws SQLException {
    int row_count = rows.size();
    if( row_count > 0 && index == 1 )
    {
      return true;
    }
    return false;
  }

  /**
   * Indicates whether the cursor is on the last row of
   * this <code>ResultSet</code> object.
   * Note: Calling the method <code>isLast</code> may be expensive
   * because the JDBC driver
   * might need to fetch ahead one row in order to determine
   * whether the current row is the last row in the result set.
   *
   * @return <code>true</code> if the cursor is on the last row;
   * <code>false</code> otherwise
   * @exception SQLException if a database access error occurs
   * @since 1.2
   * @see <a href="package-summary.html#2.0 API">What Is in the JDBC
   *      2.0 API</a>
   */
  public boolean isLast() throws SQLException {
    int row_count = rows.size();
    if( row_count > 0 && index == row_count )
    {
      return true;
    }
    return false;
  }

  /**
   * Moves the cursor to the front of
   * this <code>ResultSet</code> object, just before the
   * first row. This method has no effect if the result set contains no rows.
   *
   * @exception SQLException if a database access error
   * occurs or the result set type is <code>TYPE_FORWARD_ONLY</code>
   * @since 1.2
   * @see <a href="package-summary.html#2.0 API">What Is in the JDBC
   *      2.0 API</a>
   */
  public void beforeFirst() throws SQLException {
    if( getType() == gweMysqlResultSetCore.TYPE_FORWARD_ONLY )
      throw new SQLException("TYPE_FORWARD_ONLY ResultSet does not support previous().");
    int row_count = rows.size();
    if( row_count > 0 )
    {
      index = 0;
    }
  }

  /**
   * Moves the cursor to the end of
   * this <code>ResultSet</code> object, just after the
   * last row. This method has no effect if the result set contains no rows.
   * @exception SQLException if a database access error
   * occurs or the result set type is <code>TYPE_FORWARD_ONLY</code>
   * @since 1.2
   * @see <a href="package-summary.html#2.0 API">What Is in the JDBC
   *      2.0 API</a>
   */
  public void afterLast() throws SQLException {
    if( getType() == gweMysqlResultSetCore.TYPE_FORWARD_ONLY )
      throw new SQLException("TYPE_FORWARD_ONLY ResultSet does not support previous().");
    int row_count = rows.size();
    if( row_count > 0 )
    {
      index = row_count + 1;
    }
  }

  /**
   * Moves the cursor to the first row in
   * this <code>ResultSet</code> object.
   *
   * @return <code>true</code> if the cursor is on a valid row;
   * <code>false</code> if there are no rows in the result set
   * @exception SQLException if a database access error
   * occurs or the result set type is <code>TYPE_FORWARD_ONLY</code>
   * @since 1.2
   * @see <a href="package-summary.html#2.0 API">What Is in the JDBC
   *      2.0 API</a>
   */
  public boolean first() throws SQLException {
    if( getType() == gweMysqlResultSetCore.TYPE_FORWARD_ONLY )
      throw new SQLException("TYPE_FORWARD_ONLY ResultSet does not support previous().");
    int row_count = rows.size();
    if( row_count > 0 )
    {
      index = 1;
      return true;
    }
    return false;
  }

  /**
   * Moves the cursor to the last row in
   * this <code>ResultSet</code> object.
   *
   * @return <code>true</code> if the cursor is on a valid row;
   * <code>false</code> if there are no rows in the result set
   * @exception SQLException if a database access error
   * occurs or the result set type is <code>TYPE_FORWARD_ONLY</code>
   * @since 1.2
   * @see <a href="package-summary.html#2.0 API">What Is in the JDBC
   *      2.0 API</a>
   */
  public boolean last() throws SQLException {
    if( getType() == gweMysqlResultSetCore.TYPE_FORWARD_ONLY )
      throw new SQLException("TYPE_FORWARD_ONLY ResultSet does not support previous().");
    int row_count = rows.size();
    if( row_count > 0 )
    {
      index = row_count;
      return true;
    }
    return false;
  }

  /**
   * Retrieves the current row number.  The first row is number 1, the
   * second number 2, and so on.
   *
   * @return the current row number; <code>0</code> if there is no current row
   * @exception SQLException if a database access error occurs
   * @since 1.2
   * @see <a href="package-summary.html#2.0 API">What Is in the JDBC
   *      2.0 API</a>
   */
  public int getRow() throws SQLException {
    return index;
  }

  /**
   * Moves the cursor to the given row number in
   * this <code>ResultSet</code> object.
   *
   * <p>If the row number is positive, the cursor moves to
   * the given row number with respect to the
   * beginning of the result set.  The first row is row 1, the second
   * is row 2, and so on.
   *
   * <p>If the given row number is negative, the cursor moves to
   * an absolute row position with respect to
   * the end of the result set.  For example, calling the method
   * <code>absolute(-1)</code> positions the
   * cursor on the last row; calling the method <code>absolute(-2)</code>
   * moves the cursor to the next-to-last row, and so on.
   *
   * <p>An attempt to position the cursor beyond the first/last row in
   * the result set leaves the cursor before the first row or after
   * the last row.
   *
   * <p><B>Note:</B> Calling <code>absolute(1)</code> is the same
   * as calling <code>first()</code>. Calling <code>absolute(-1)</code>
   * is the same as calling <code>last()</code>.
   *
   * @return <code>true</code> if the cursor is on the result set;
   * <code>false</code> otherwise
   * @exception SQLException if a database access error
   * occurs, the row is <code>0</code>, or the result set type is
   * <code>TYPE_FORWARD_ONLY</code>
   * @since 1.2
   * @see <a href="package-summary.html#2.0 API">What Is in the JDBC
   *      2.0 API</a>
   */
  public boolean absolute(int row) throws SQLException {
    if( getType() == gweMysqlResultSetCore.TYPE_FORWARD_ONLY )
      throw new SQLException("TYPE_FORWARD_ONLY ResultSet does not support previous().");
    if( row == 0 )
      return false;
    int row_count = rows.size();
    if( row < 0 )
      row = row_count + 1 + row;
    index = row;
    if( index >= 1 && index <= row_count )
      return true;
    if( index <= 0 )
      beforeFirst();
    else
      afterLast();
    return false;
  }

  /**
   * Moves the cursor a relative number of rows, either positive or negative.
   * Attempting to move beyond the first/last row in the
   * result set positions the cursor before/after the
   * the first/last row. Calling <code>relative(0)</code> is valid, but does
   * not change the cursor position.
   *
   * <p>Note: Calling the method <code>relative(1)</code>
   * is different from calling the method <code>next()</code>
   * because is makes sense to call <code>next()</code> when there
   * is no current row,
   * for example, when the cursor is positioned before the first row
   * or after the last row of the result set.
   *
   * @return <code>true</code> if the cursor is on a row;
   * <code>false</code> otherwise
   * @exception SQLException if a database access error occurs,
   * there is no current row, or the result set type is
   * <code>TYPE_FORWARD_ONLY</code>
   * @since 1.2
   * @see <a href="package-summary.html#2.0 API">What Is in the JDBC
   *      2.0 API</a>
   */
  public boolean relative(int row) throws SQLException {
    if( getType() == gweMysqlResultSetCore.TYPE_FORWARD_ONLY )
      throw new SQLException("TYPE_FORWARD_ONLY ResultSet does not support previous().");
    int row_count = rows.size();
    index += row;
    if( index >= 1 && index <= row_count )
      return true;
    if( index <= 0 )
      beforeFirst();
    else
      afterLast();
    return false;
  }

  /**
   * Moves the cursor to the previous row in this
   * <code>ResultSet</code> object.
   *
   * <p><B>Note:</B> Calling the method <code>previous()</code> is not the same as
   * calling the method <code>relative(-1)</code> because it
   * makes sense to call</code>previous()</code> when there is no current row.
   *
   * @return <code>true</code> if the cursor is on a valid row;
   * <code>false</code> if it is off the result set
   * @exception SQLException if a database access error
   * occurs or the result set type is <code>TYPE_FORWARD_ONLY</code>
   * @since 1.2
   * @see <a href="package-summary.html#2.0 API">What Is in the JDBC
   *      2.0 API</a>
   */
  public boolean previous() throws SQLException {
    if( getType() == gweMysqlResultSetCore.TYPE_FORWARD_ONLY )
      throw new SQLException("TYPE_FORWARD_ONLY ResultSet does not support previous().");
    if( index - 1 >= 1 )
    {
      index--;
      return true;
    }
    return false;
  }

  public void setFetchDirection(int direction) throws SQLException {
    org_rs.setFetchDirection(direction);
  }

  public int getFetchDirection() throws SQLException {
    return org_rs.getFetchDirection();
  }

  public void setFetchSize(int rows) throws SQLException {
    org_rs.setFetchSize(rows);
  }

  public int getFetchSize() throws SQLException {
    return org_rs.getFetchSize();
  }

  /**
   * Returns the type of this <code>ResultSet</code> object.
   * The type is determined by the <code>Statement</code> object
   * that created the result set.
   *
   * @return <code>TYPE_FORWARD_ONLY</code>,
   * <code>TYPE_SCROLL_INSENSITIVE</code>,
   * or <code>TYPE_SCROLL_SENSITIVE</code>
   * @exception SQLException if a database access error occurs
   * @since 1.2
   * @see <a href="package-summary.html#2.0 API">What Is in the JDBC
   *      2.0 API</a>
   */
  public int getType() throws SQLException {
    return org_rs.getType();
  }

  /**
   * Returns the concurrency mode of this <code>ResultSet</code> object.
   * The concurrency used is determined by the
   * <code>Statement</code> object that created the result set.
   *
   * @return the concurrency type, either <code>CONCUR_READ_ONLY</code>
   * or <code>CONCUR_UPDATABLE</code>
   * @exception SQLException if a database access error occurs
   * @since 1.2
   * @see <a href="package-summary.html#2.0 API">What Is in the JDBC
   *      2.0 API</a>
   */
  public int getConcurrency() throws SQLException {
    return org_rs.getConcurrency();
  }

  public boolean rowUpdated() throws SQLException {
    throw new SQLException("mysql does not support rowUpdated().");
  }

  public boolean rowInserted() throws SQLException {
    throw new SQLException("mysql does not support rowInserted().");
  }

  public boolean rowDeleted() throws SQLException {
    throw new SQLException("mysql does not support rowDeleted().");
  }

  public void updateNull(int columnIndex) throws SQLException {
    throw new SQLException("mysql does not support updateNull(int).");
  }

  public void updateBoolean(int columnIndex, boolean x) throws SQLException {
    throw new SQLException("mysql does not support updateBoolean(int, boolean).");
  }

  public void updateByte(int columnIndex, byte x) throws SQLException {
    throw new SQLException("mysql does not support updateByte(int, byte).");
  }

  public void updateShort(int columnIndex, short x) throws SQLException {
    throw new SQLException("mysql does not support updateShort(int, short).");
  }

  public void updateInt(int columnIndex, int x) throws SQLException {
    throw new SQLException("mysql does not support updateInt(int, int).");
  }

  public void updateLong(int columnIndex, long x) throws SQLException {
    throw new SQLException("mysql does not support updateLong(int, long).");
  }

  public void updateFloat(int columnIndex, float x) throws SQLException {
    throw new SQLException("mysql does not support updateFloat(int, float).");
  }

  public void updateDouble(int columnIndex, double x) throws SQLException {
    throw new SQLException("mysql does not support updateDouble(int, double).");
  }

  public void updateBigDecimal(int columnIndex, BigDecimal x) throws SQLException {
    throw new SQLException("mysql does not support updateBigDecimal(int, BigDecimal).");
  }

  public void updateString(int columnIndex, String x) throws SQLException {
    throw new SQLException("mysql does not support updateString(int, String).");
  }

  public void updateBytes(int columnIndex, byte[] x) throws SQLException {
    throw new SQLException("mysql does not support updateBytes(int, byte[]).");
  }

  public void updateDate(int columnIndex, Date x) throws SQLException {
    throw new SQLException("mysql does not support updateDate(int, Date).");
  }

  public void updateTime(int columnIndex, Time x) throws SQLException {
    throw new SQLException("mysql does not support updateTime(int, Time).");
  }

  public void updateTimestamp(int columnIndex, Timestamp x) throws SQLException {
    throw new SQLException("mysql does not support updateTimestamp(int, Timestamp).");
  }

  public void updateAsciiStream(int columnIndex, InputStream x, int length) throws SQLException {
    throw new SQLException("mysql does not support updateAsciiStream(int, InputStream, int).");
  }

  public void updateBinaryStream(int columnIndex, InputStream x, int length) throws SQLException {
    throw new SQLException("mysql does not support updateBinaryStream(int, InputStream, int).");
  }

  public void updateCharacterStream(int columnIndex, Reader x, int length) throws SQLException {
    throw new SQLException("mysql does not support updateCharacterStream(int, Reader, int).");
  }

  public void updateObject(int columnIndex, Object x, int scale) throws SQLException {
    throw new SQLException("mysql does not support updateObject(int, Object, int).");
  }

  public void updateObject(int columnIndex, Object x) throws SQLException {
    throw new SQLException("mysql does not support updateObject(int, Object).");
  }

  public void updateNull(String columnName) throws SQLException {
    throw new SQLException("mysql does not support updateNull(String).");
  }

  public void updateBoolean(String columnName, boolean x) throws SQLException {
    throw new SQLException("mysql does not support updateBoolean(String, boolean).");
  }

  public void updateByte(String columnName, byte x) throws SQLException {
    throw new SQLException("mysql does not support updateByte(String, byte).");
  }

  public void updateShort(String columnName, short x) throws SQLException {
    throw new SQLException("mysql does not support updateShort(String, short).");
  }

  public void updateInt(String columnName, int x) throws SQLException {
    throw new SQLException("mysql does not support updateInt(String, int).");
  }

  public void updateLong(String columnName, long x) throws SQLException {
    throw new SQLException("mysql does not support updateLong(String, long).");
  }

  public void updateFloat(String columnName, float x) throws SQLException {
    throw new SQLException("mysql does not support updateFloat(String, float).");
  }

  public void updateDouble(String columnName, double x) throws SQLException {
    throw new SQLException("mysql does not support updateDouble(String, double).");
  }

  public void updateBigDecimal(String columnName, BigDecimal x) throws SQLException {
    throw new SQLException("mysql does not support updateBigDecimal(String, BigDecimal).");
  }

  public void updateString(String columnName, String x) throws SQLException {
    throw new SQLException("mysql does not support updateString(String, String).");
  }

  public void updateBytes(String columnName, byte[] x) throws SQLException {
    throw new SQLException("mysql does not support updateBytes(String, byte[]).");
  }

  public void updateDate(String columnName, Date x) throws SQLException {
    throw new SQLException("mysql does not support updateDate(String, Date).");
  }

  public void updateTime(String columnName, Time x) throws SQLException {
    throw new SQLException("mysql does not support updateTime(String, Time).");
  }

  public void updateTimestamp(String columnName, Timestamp x) throws SQLException {
    throw new SQLException("mysql does not support updateTimestamp(String, Timestamp).");
  }

  public void updateAsciiStream(String columnName, InputStream x, int length) throws SQLException {
    throw new SQLException("mysql does not support updateAsciiStream(String, InputStream, int).");
  }

  public void updateBinaryStream(String columnName, InputStream x, int length) throws SQLException {
    throw new SQLException("mysql does not support updateBinaryStream(String, InputStream, int).");
  }

  public void updateCharacterStream(String columnName, Reader reader, int length) throws SQLException {
    throw new SQLException("mysql does not support updateCharacterStream(String, Reader, int).");
  }

  public void updateObject(String columnName, Object x, int scale) throws SQLException {
    throw new SQLException("mysql does not support updateObject(String, Object, int).");
  }

  public void updateObject(String columnName, Object x) throws SQLException {
    throw new SQLException("mysql does not support updateObject(String, Object).");
  }

  public void insertRow() throws SQLException {
    throw new SQLException("mysql does not support insertRow().");
  }

  public void updateRow() throws SQLException {
    throw new SQLException("mysql does not support updateRow().");
  }

  public void deleteRow() throws SQLException {
    throw new SQLException("mysql does not support deleteRow().");
  }

  public void refreshRow() throws SQLException {
    throw new SQLException("mysql does not support refreshRow().");
  }

  public void cancelRowUpdates() throws SQLException {
    throw new SQLException("mysql does not support cancelRowUpdates().");
  }

  public void moveToInsertRow() throws SQLException {
    throw new SQLException("mysql does not support moveToInsertRow().");
  }

  public void moveToCurrentRow() throws SQLException {
    throw new SQLException("mysql does not support moveToCurrentRow().");
  }

  public Statement getStatement() throws SQLException {
    return org_rs.getStatement();
  }

/*
  public Object getObject(int i, Map map) throws SQLException {
    throw new SQLException("mysql does not support getObject(int, Map).");
  }

  public Ref getRef(int i) throws SQLException {
    throw new SQLException("mysql does not support getRef(int).");
  }

  public Blob getBlob(int i) throws SQLException {
    throw new SQLException("mysql does not support getBlob(int).");
  }

  public Clob getClob(int i) throws SQLException {
    throw new SQLException("mysql does not support getClob(int).");
  }

  public Array getArray(int i) throws SQLException {
    throw new SQLException("mysql does not support getArray(int).");
  }

  public Object getObject(String colName, Map map) throws SQLException {
    throw new SQLException("mysql does not support getObject(String, Map).");
  }

  public Ref getRef(String colName) throws SQLException {
    throw new SQLException("mysql does not support getRef(String).");
  }

  public Blob getBlob(String colName) throws SQLException {
    throw new SQLException("mysql does not support getBlob(String).");
  }

  public Clob getClob(String colName) throws SQLException {
    throw new SQLException("mysql does not support getClob(String).");
  }

  public Array getArray(String colName) throws SQLException {
    throw new SQLException("mysql does not support getArray(String).");
  }
*/

  public Date getDate(int columnIndex, Calendar cal) throws SQLException {
    return getDate(columnIndex);
  }

  public Date getDate(String columnName, Calendar cal) throws SQLException {
    return getDate(findColumn(columnName));
  }

  public Time getTime(int columnIndex, Calendar cal) throws SQLException {
    return getTime(columnIndex);
  }

  public Time getTime(String columnName, Calendar cal) throws SQLException {
    return getTime(findColumn(columnName));
  }

  public Timestamp getTimestamp(int columnIndex, Calendar cal) throws SQLException {
    return getTimestamp(columnIndex);
  }

  public Timestamp getTimestamp(String columnName, Calendar cal) throws SQLException {
    return getTimestamp(findColumn(columnName));
  }
}
